/**
 * DevExtreme (cjs/exporter/jspdf/export_data_grid_2.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.exportDataGrid = exportDataGrid;
var _type = require("../../core/utils/type");
var _pdf_grid = require("./pdf_grid");

function exportDataGrid(doc, dataGrid, options) {
    if (!(0, _type.isDefined)(options.rect)) {
        throw "options.rect is required"
    }
    var dataProvider = dataGrid.getDataProvider();
    return new Promise((function(resolve) {
        dataProvider.ready().done((function() {
            var columns = dataProvider.getColumns();
            var pdfGrid = new _pdf_grid.PdfGrid(options.splitToTablesByColumns);
            pdfGrid.startNewTable(options.drawTableBorder, options.rect);
            var dataRowsCount = dataProvider.getRowsCount();
            for (var rowIndex = 0; rowIndex < dataRowsCount; rowIndex++) {
                var currentRow = [];
                for (var cellIndex = 0; cellIndex < columns.length; cellIndex++) {
                    var cellData = dataProvider.getCellData(rowIndex, cellIndex, true);
                    var pdfCell = {
                        text: cellData.value
                    };
                    if (options.onCellExporting) {
                        options.onCellExporting({
                            gridCell: {
                                value: cellData.value
                            },
                            pdfCell: pdfCell
                        })
                    }
                    currentRow.push(pdfCell)
                }
                if (options.onRowExporting) {
                    ! function() {
                        var args = {
                            drawNewTableFromThisRow: {},
                            rowCells: currentRow
                        };
                        options.onRowExporting(args);
                        var _args$drawNewTableFro = args.drawNewTableFromThisRow,
                            startNewTable = _args$drawNewTableFro.startNewTable,
                            addPage = _args$drawNewTableFro.addPage,
                            tableRect = _args$drawNewTableFro.tableRect,
                            splitToTablesByColumns = _args$drawNewTableFro.splitToTablesByColumns;
                        if (true === startNewTable) {
                            pdfGrid.startNewTable(options.drawTableBorder, tableRect, true === addPage, splitToTablesByColumns)
                        }
                        if ((0, _type.isDefined)(args.rowHeight)) {
                            currentRow.forEach((function(cell) {
                                if (!cell.skip && !cell.rect.h) {
                                    cell.rect.h = args.rowHeight
                                }
                            }))
                        }
                    }()
                }
                pdfGrid.addRow(currentRow)
            }
            pdfGrid.drawTo(doc);
            resolve()
        }))
    }))
}
