/**
 * DevExtreme (cjs/exporter/jspdf/pdf_grid.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.PdfGrid = void 0;
var _type = require("../../core/utils/type");
var _pdf_table = require("./pdf_table");

function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread()
}

function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")
}

function _unsupportedIterableToArray(o, minLen) {
    if (!o) {
        return
    }
    if ("string" === typeof o) {
        return _arrayLikeToArray(o, minLen)
    }
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if ("Object" === n && o.constructor) {
        n = o.constructor.name
    }
    if ("Map" === n || "Set" === n) {
        return Array.from(o)
    }
    if ("Arguments" === n || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) {
        return _arrayLikeToArray(o, minLen)
    }
}

function _iterableToArray(iter) {
    if ("undefined" !== typeof Symbol && Symbol.iterator in Object(iter)) {
        return Array.from(iter)
    }
}

function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) {
        return _arrayLikeToArray(arr)
    }
}

function _arrayLikeToArray(arr, len) {
    if (null == len || len > arr.length) {
        len = arr.length
    }
    for (var i = 0, arr2 = new Array(len); i < len; i++) {
        arr2[i] = arr[i]
    }
    return arr2
}
var PdfGrid = function() {
    function PdfGrid(splitByColumns) {
        this._splitByColumns = null !== splitByColumns && void 0 !== splitByColumns ? splitByColumns : [];
        this._newPageTables = [];
        this._tables = [];
        this._currentHorizontalTables = null
    }
    var _proto = PdfGrid.prototype;
    _proto._addLastTableToNewPages = function() {
        this._newPageTables.push(this._currentHorizontalTables[this._currentHorizontalTables.length - 1])
    };
    _proto.startNewTable = function(drawTableBorder, firstTableRect, firstTableOnNewPage, splitByColumns) {
        var _this$_tables, _this = this;
        this._currentHorizontalTables = [new _pdf_table.PdfTable(drawTableBorder, firstTableRect)];
        if (firstTableOnNewPage) {
            this._addLastTableToNewPages()
        }
        if ((0, _type.isDefined)(splitByColumns)) {
            this._splitByColumns = splitByColumns
        }
        if ((0, _type.isDefined)(this._splitByColumns)) {
            this._splitByColumns.forEach((function(splitByColumn) {
                _this._currentHorizontalTables.push(new _pdf_table.PdfTable(drawTableBorder, splitByColumn.tableRect));
                if (splitByColumn.drawOnNewPage) {
                    _this._addLastTableToNewPages()
                }
            }))
        }(_this$_tables = this._tables).push.apply(_this$_tables, _toConsumableArray(this._currentHorizontalTables))
    };
    _proto.addRow = function(cells) {
        var _this2 = this;
        var currentTableIndex = 0;
        var currentTableCells = [];
        var _loop = function(cellIndex) {
            var isNewTableColumn = _this2._splitByColumns.filter((function(splitByColumn) {
                return splitByColumn.columnIndex === cellIndex
            }))[0];
            if (isNewTableColumn) {
                _this2._currentHorizontalTables[currentTableIndex].addRow(currentTableCells);
                currentTableIndex++;
                currentTableCells = []
            }
            currentTableCells.push(cells[cellIndex])
        };
        for (var cellIndex = 0; cellIndex < cells.length; cellIndex++) {
            _loop(cellIndex)
        }
        this._currentHorizontalTables[currentTableIndex].addRow(currentTableCells)
    };
    _proto.drawTo = function(doc) {
        var _this3 = this;
        this._tables.forEach((function(table) {
            if (-1 !== _this3._newPageTables.indexOf(table)) {
                doc.addPage()
            }
            table.drawTo(doc)
        }))
    };
    return PdfGrid
}();
exports.PdfGrid = PdfGrid;
