/**
 * DevExtreme (cjs/renovation/ui/scroll_view/scrollable_native.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.ScrollableNative = exports.ScrollableNativeProps = exports.viewFunction = void 0;
var _inferno = require("inferno");
var _vdom = require("@devextreme/vdom");
var _subscribe_to_event = require("../../utils/subscribe_to_event");
var _widget = require("../common/widget");
var _load_panel = require("./load_panel");
var _combine_classes = require("../../utils/combine_classes");
var _devices = _interopRequireDefault(require("../../../core/devices"));
var _type = require("../../../core/utils/type");
var _base_props = require("../common/base_props");
var _scrollable_props = require("./scrollable_props");
var _top_pocket = require("./top_pocket");
var _bottom_pocket = require("./bottom_pocket");
var _browser = _interopRequireDefault(require("../../../core/utils/browser"));
var _support = require("../../../core/utils/support");
require("../../../events/gesture/emitter.gesture.scroll");
var _index = require("../../../events/utils/index");
var _scrollable_utils = require("./scrollable_utils");
var _scroll_direction = require("./utils/scroll_direction");
var _consts = require("./common/consts");
var _scrollbar = require("./scrollbar");
var _short = require("../../../events/short");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _objectWithoutProperties(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for (i = 0; i < sourceSymbolKeys.length; i++) {
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) {
                continue
            }
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) {
                continue
            }
            target[key] = source[key]
        }
    }
    return target
}

function _objectWithoutPropertiesLoose(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) {
            continue
        }
        target[key] = source[key]
    }
    return target
}

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        })
    } else {
        obj[key] = value
    }
    return obj
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}

function _assertThisInitialized(self) {
    if (void 0 === self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return self
}

function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass)
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}
var HIDE_SCROLLBAR_TIMEOUT = 500;
var viewFunction = function(viewModel) {
    var containerClientHeight = viewModel.containerClientHeight,
        containerClientWidth = viewModel.containerClientWidth,
        containerRef = viewModel.containerRef,
        contentClientHeight = viewModel.contentClientHeight,
        contentClientWidth = viewModel.contentClientWidth,
        contentRef = viewModel.contentRef,
        cssClasses = viewModel.cssClasses,
        direction = viewModel.direction,
        horizontalScrollbarRef = viewModel.horizontalScrollbarRef,
        isLoadPanelVisible = viewModel.isLoadPanelVisible,
        needForceScrollbarsVisibility = viewModel.needForceScrollbarsVisibility,
        _viewModel$props = viewModel.props,
        children = _viewModel$props.children,
        disabled = _viewModel$props.disabled,
        forceGeneratePockets = _viewModel$props.forceGeneratePockets,
        height = _viewModel$props.height,
        needScrollViewContentWrapper = _viewModel$props.needScrollViewContentWrapper,
        needScrollViewLoadPanel = _viewModel$props.needScrollViewLoadPanel,
        pullDownEnabled = _viewModel$props.pullDownEnabled,
        pulledDownText = _viewModel$props.pulledDownText,
        pullingDownText = _viewModel$props.pullingDownText,
        reachBottomEnabled = _viewModel$props.reachBottomEnabled,
        reachBottomText = _viewModel$props.reachBottomText,
        refreshingText = _viewModel$props.refreshingText,
        rtlEnabled = _viewModel$props.rtlEnabled,
        scrollByThumb = _viewModel$props.scrollByThumb,
        showScrollbar = _viewModel$props.showScrollbar,
        visible = _viewModel$props.visible,
        width = _viewModel$props.width,
        restAttributes = viewModel.restAttributes,
        scrollableRef = viewModel.scrollableRef,
        topPocketState = viewModel.topPocketState,
        useSimulatedScrollbar = viewModel.useSimulatedScrollbar,
        verticalScrollbarRef = viewModel.verticalScrollbarRef,
        windowResizeHandler = viewModel.windowResizeHandler,
        wrapperRef = viewModel.wrapperRef;
    return (0, _inferno.normalizeProps)((0, _inferno.createComponentVNode)(2, _widget.Widget, _extends({
        rootElementRef: scrollableRef,
        classes: cssClasses,
        disabled: disabled,
        rtlEnabled: rtlEnabled,
        height: height,
        width: width,
        visible: visible,
        onDimensionChanged: windowResizeHandler
    }, restAttributes, {
        children: [(0, _inferno.createVNode)(1, "div", _consts.SCROLLABLE_WRAPPER_CLASS, (0, _inferno.createVNode)(1, "div", _consts.SCROLLABLE_CONTAINER_CLASS, (0, _inferno.createVNode)(1, "div", _consts.SCROLLABLE_CONTENT_CLASS, [forceGeneratePockets && (0, _inferno.createComponentVNode)(2, _top_pocket.TopPocket, {
            pullingDownText: pullingDownText,
            pulledDownText: pulledDownText,
            refreshingText: refreshingText,
            pocketState: topPocketState,
            useNative: true,
            visible: !!pullDownEnabled
        }), needScrollViewContentWrapper ? (0, _inferno.createVNode)(1, "div", _consts.SCROLLVIEW_CONTENT_CLASS, children, 0) : (0, _inferno.createVNode)(1, "div", null, children, 0), forceGeneratePockets && (0, _inferno.createComponentVNode)(2, _bottom_pocket.BottomPocket, {
            reachBottomText: reachBottomText,
            visible: !!reachBottomEnabled
        })], 0, null, null, contentRef), 2, null, null, containerRef), 2, null, null, wrapperRef), needScrollViewLoadPanel && (0, _inferno.createComponentVNode)(2, _load_panel.ScrollViewLoadPanel, {
            targetElement: scrollableRef,
            refreshingText: refreshingText,
            visible: isLoadPanelVisible
        }), showScrollbar && useSimulatedScrollbar && direction.isHorizontal && (0, _inferno.createComponentVNode)(2, _scrollbar.Scrollbar, {
            direction: "horizontal",
            scrollByThumb: scrollByThumb,
            contentSize: contentClientWidth,
            containerSize: containerClientWidth,
            forceVisibility: needForceScrollbarsVisibility
        }, null, horizontalScrollbarRef), showScrollbar && useSimulatedScrollbar && direction.isVertical && (0, _inferno.createComponentVNode)(2, _scrollbar.Scrollbar, {
            direction: "vertical",
            scrollByThumb: scrollByThumb,
            contentSize: contentClientHeight,
            containerSize: containerClientHeight,
            forceVisibility: needForceScrollbarsVisibility
        }, null, verticalScrollbarRef)]
    })))
};
exports.viewFunction = viewFunction;
var ScrollableNativeProps = _extends({}, _scrollable_props.ScrollableProps);
exports.ScrollableNativeProps = ScrollableNativeProps;
var ScrollableNativePropsType = {
    useNative: ScrollableNativeProps.useNative,
    direction: ScrollableNativeProps.direction,
    showScrollbar: ScrollableNativeProps.showScrollbar,
    scrollByThumb: ScrollableNativeProps.scrollByThumb,
    bounceEnabled: ScrollableNativeProps.bounceEnabled,
    scrollByContent: ScrollableNativeProps.scrollByContent,
    useKeyboard: ScrollableNativeProps.useKeyboard,
    pullDownEnabled: ScrollableNativeProps.pullDownEnabled,
    reachBottomEnabled: ScrollableNativeProps.reachBottomEnabled,
    forceGeneratePockets: ScrollableNativeProps.forceGeneratePockets,
    needScrollViewContentWrapper: ScrollableNativeProps.needScrollViewContentWrapper,
    needScrollViewLoadPanel: ScrollableNativeProps.needScrollViewLoadPanel,
    disabled: _base_props.BaseWidgetProps.disabled,
    visible: _base_props.BaseWidgetProps.visible
};
var ScrollableNative = function(_InfernoComponent) {
    _inheritsLoose(ScrollableNative, _InfernoComponent);

    function ScrollableNative(props) {
        var _this;
        _this = _InfernoComponent.call(this, props) || this;
        _this._currentState = null;
        _this.scrollableRef = (0, _inferno.createRef)();
        _this.wrapperRef = (0, _inferno.createRef)();
        _this.contentRef = (0, _inferno.createRef)();
        _this.containerRef = (0, _inferno.createRef)();
        _this.verticalScrollbarRef = (0, _inferno.createRef)();
        _this.horizontalScrollbarRef = (0, _inferno.createRef)();
        _this.locked = false;
        _this.loadingIndicatorEnabled = true;
        _this.lastLocation = {
            top: 0,
            left: 0
        };
        _this.state = {
            containerClientWidth: 0,
            containerClientHeight: 0,
            contentClientWidth: 0,
            contentClientHeight: 0,
            needForceScrollbarsVisibility: false,
            topPocketState: _consts.TopPocketState.STATE_RELEASED,
            isLoadPanelVisible: false
        };
        _this.content = _this.content.bind(_assertThisInitialized(_this));
        _this.update = _this.update.bind(_assertThisInitialized(_this));
        _this.release = _this.release.bind(_assertThisInitialized(_this));
        _this.scrollBy = _this.scrollBy.bind(_assertThisInitialized(_this));
        _this.scrollTo = _this.scrollTo.bind(_assertThisInitialized(_this));
        _this.scrollToElement = _this.scrollToElement.bind(_assertThisInitialized(_this));
        _this.scrollHeight = _this.scrollHeight.bind(_assertThisInitialized(_this));
        _this.scrollWidth = _this.scrollWidth.bind(_assertThisInitialized(_this));
        _this.scrollOffset = _this.scrollOffset.bind(_assertThisInitialized(_this));
        _this.scrollTop = _this.scrollTop.bind(_assertThisInitialized(_this));
        _this.scrollLeft = _this.scrollLeft.bind(_assertThisInitialized(_this));
        _this.clientHeight = _this.clientHeight.bind(_assertThisInitialized(_this));
        _this.clientWidth = _this.clientWidth.bind(_assertThisInitialized(_this));
        _this.scrollEffect = _this.scrollEffect.bind(_assertThisInitialized(_this));
        _this.getEventArgs = _this.getEventArgs.bind(_assertThisInitialized(_this));
        _this.effectDisabledState = _this.effectDisabledState.bind(_assertThisInitialized(_this));
        _this.lock = _this.lock.bind(_assertThisInitialized(_this));
        _this.unlock = _this.unlock.bind(_assertThisInitialized(_this));
        _this.effectResetInactiveState = _this.effectResetInactiveState.bind(_assertThisInitialized(_this));
        _this.updateScrollbarSize = _this.updateScrollbarSize.bind(_assertThisInitialized(_this));
        _this.windowResizeHandler = _this.windowResizeHandler.bind(_assertThisInitialized(_this));
        _this.updateSizes = _this.updateSizes.bind(_assertThisInitialized(_this));
        _this.moveScrollbars = _this.moveScrollbars.bind(_assertThisInitialized(_this));
        _this.disposeHideScrollbarTimeout = _this.disposeHideScrollbarTimeout.bind(_assertThisInitialized(_this));
        _this.clearHideScrollbarTimeout = _this.clearHideScrollbarTimeout.bind(_assertThisInitialized(_this));
        _this.location = _this.location.bind(_assertThisInitialized(_this));
        _this.initEffect = _this.initEffect.bind(_assertThisInitialized(_this));
        _this.getInitEventData = _this.getInitEventData.bind(_assertThisInitialized(_this));
        _this.moveEffect = _this.moveEffect.bind(_assertThisInitialized(_this));
        _this.handleInit = _this.handleInit.bind(_assertThisInitialized(_this));
        _this.handleMove = _this.handleMove.bind(_assertThisInitialized(_this));
        _this.tryGetAllowedDirection = _this.tryGetAllowedDirection.bind(_assertThisInitialized(_this));
        _this.validate = _this.validate.bind(_assertThisInitialized(_this));
        _this.isLocked = _this.isLocked.bind(_assertThisInitialized(_this));
        _this.isScrollingOutOfBound = _this.isScrollingOutOfBound.bind(_assertThisInitialized(_this));
        return _this
    }
    var _proto = ScrollableNative.prototype;
    _proto.createEffects = function() {
        return [new _vdom.InfernoEffect(this.scrollEffect, [this.props.useSimulatedScrollbar, this.props.direction, this.props.onScroll, this.props.rtlEnabled]), new _vdom.InfernoEffect(this.effectDisabledState, [this.props.disabled]), new _vdom.InfernoEffect(this.effectResetInactiveState, [this.props.direction]), new _vdom.InfernoEffect(this.updateScrollbarSize, [this.containerClientWidth, this.containerClientHeight, this.contentClientWidth, this.contentClientHeight, this.needForceScrollbarsVisibility, this.topPocketState, this.isLoadPanelVisible, this.props.useSimulatedScrollbar, this.props.useNative, this.props.pullingDownText, this.props.pulledDownText, this.props.refreshingText, this.props.reachBottomText, this.props.children, this.props.direction, this.props.showScrollbar, this.props.scrollByThumb, this.props.bounceEnabled, this.props.scrollByContent, this.props.useKeyboard, this.props.classes, this.props.pullDownEnabled, this.props.reachBottomEnabled, this.props.forceGeneratePockets, this.props.needScrollViewContentWrapper, this.props.needScrollViewLoadPanel, this.props.onScroll, this.props.onUpdated, this.props.onPullDown, this.props.onReachBottom, this.props.disabled, this.props.height, this.props.onKeyDown, this.props.rtlEnabled, this.props.visible, this.props.width]), new _vdom.InfernoEffect(this.disposeHideScrollbarTimeout, []), new _vdom.InfernoEffect(this.initEffect, [this.props.direction, this.props.pullDownEnabled, this.props.disabled, this.props.rtlEnabled]), new _vdom.InfernoEffect(this.moveEffect, [this.props.direction, this.props.pullDownEnabled])]
    };
    _proto.updateEffects = function() {
        var _this$_effects$, _this$_effects$2, _this$_effects$3, _this$_effects$4, _this$_effects$5, _this$_effects$6;
        null === (_this$_effects$ = this._effects[0]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([this.props.useSimulatedScrollbar, this.props.direction, this.props.onScroll, this.props.rtlEnabled]);
        null === (_this$_effects$2 = this._effects[1]) || void 0 === _this$_effects$2 ? void 0 : _this$_effects$2.update([this.props.disabled]);
        null === (_this$_effects$3 = this._effects[2]) || void 0 === _this$_effects$3 ? void 0 : _this$_effects$3.update([this.props.direction]);
        null === (_this$_effects$4 = this._effects[3]) || void 0 === _this$_effects$4 ? void 0 : _this$_effects$4.update([this.containerClientWidth, this.containerClientHeight, this.contentClientWidth, this.contentClientHeight, this.needForceScrollbarsVisibility, this.topPocketState, this.isLoadPanelVisible, this.props.useSimulatedScrollbar, this.props.useNative, this.props.pullingDownText, this.props.pulledDownText, this.props.refreshingText, this.props.reachBottomText, this.props.children, this.props.direction, this.props.showScrollbar, this.props.scrollByThumb, this.props.bounceEnabled, this.props.scrollByContent, this.props.useKeyboard, this.props.classes, this.props.pullDownEnabled, this.props.reachBottomEnabled, this.props.forceGeneratePockets, this.props.needScrollViewContentWrapper, this.props.needScrollViewLoadPanel, this.props.onScroll, this.props.onUpdated, this.props.onPullDown, this.props.onReachBottom, this.props.disabled, this.props.height, this.props.onKeyDown, this.props.rtlEnabled, this.props.visible, this.props.width]);
        null === (_this$_effects$5 = this._effects[5]) || void 0 === _this$_effects$5 ? void 0 : _this$_effects$5.update([this.props.direction, this.props.pullDownEnabled, this.props.disabled, this.props.rtlEnabled]);
        null === (_this$_effects$6 = this._effects[6]) || void 0 === _this$_effects$6 ? void 0 : _this$_effects$6.update([this.props.direction, this.props.pullDownEnabled])
    };
    _proto.set_containerClientWidth = function(value) {
        var _this2 = this;
        this.setState((function(state) {
            _this2._currentState = state;
            var newValue = value();
            _this2._currentState = null;
            return {
                containerClientWidth: newValue
            }
        }))
    };
    _proto.set_containerClientHeight = function(value) {
        var _this3 = this;
        this.setState((function(state) {
            _this3._currentState = state;
            var newValue = value();
            _this3._currentState = null;
            return {
                containerClientHeight: newValue
            }
        }))
    };
    _proto.set_contentClientWidth = function(value) {
        var _this4 = this;
        this.setState((function(state) {
            _this4._currentState = state;
            var newValue = value();
            _this4._currentState = null;
            return {
                contentClientWidth: newValue
            }
        }))
    };
    _proto.set_contentClientHeight = function(value) {
        var _this5 = this;
        this.setState((function(state) {
            _this5._currentState = state;
            var newValue = value();
            _this5._currentState = null;
            return {
                contentClientHeight: newValue
            }
        }))
    };
    _proto.set_needForceScrollbarsVisibility = function(value) {
        var _this6 = this;
        this.setState((function(state) {
            _this6._currentState = state;
            var newValue = value();
            _this6._currentState = null;
            return {
                needForceScrollbarsVisibility: newValue
            }
        }))
    };
    _proto.set_topPocketState = function(value) {
        var _this7 = this;
        this.setState((function(state) {
            _this7._currentState = state;
            var newValue = value();
            _this7._currentState = null;
            return {
                topPocketState: newValue
            }
        }))
    };
    _proto.set_isLoadPanelVisible = function(value) {
        var _this8 = this;
        this.setState((function(state) {
            _this8._currentState = state;
            var newValue = value();
            _this8._currentState = null;
            return {
                isLoadPanelVisible: newValue
            }
        }))
    };
    _proto.scrollEffect = function() {
        var _this9 = this;
        return (0, _subscribe_to_event.subscribeToScrollEvent)(this.containerRef.current, (function(e) {
            var _this9$props$onScroll, _this9$props;
            _this9.eventForUserAction = e;
            if (_this9.useSimulatedScrollbar) {
                _this9.moveScrollbars()
            }
            null === (_this9$props$onScroll = (_this9$props = _this9.props).onScroll) || void 0 === _this9$props$onScroll ? void 0 : _this9$props$onScroll.call(_this9$props, _this9.getEventArgs());
            _this9.lastLocation = _this9.location()
        }))
    };
    _proto.effectDisabledState = function() {
        if (this.props.disabled) {
            this.lock()
        } else {
            this.unlock()
        }
    };
    _proto.effectResetInactiveState = function() {
        var containerEl = this.containerRef.current;
        if (this.props.direction === _consts.DIRECTION_BOTH || !(0, _type.isDefined)(containerEl)) {
            return
        }
        containerEl[this.fullScrollInactiveProp] = 0
    };
    _proto.updateScrollbarSize = function() {
        this.updateSizes()
    };
    _proto.disposeHideScrollbarTimeout = function() {
        var _this10 = this;
        return function() {
            return _this10.clearHideScrollbarTimeout()
        }
    };
    _proto.initEffect = function() {
        var _this11 = this;
        _short.dxScrollInit.on(this.wrapperRef.current, (function(e) {
            _this11.handleInit(e)
        }), this.getInitEventData(), {
            namespace: "dxScrollable"
        });
        return function() {
            return _short.dxScrollInit.off(_this11.wrapperRef.current, {
                namespace: "dxScrollable"
            })
        }
    };
    _proto.moveEffect = function() {
        var _this12 = this;
        _short.dxScrollMove.on(this.wrapperRef.current, (function(e) {
            _this12.handleMove(e)
        }), {
            namespace: "dxScrollable"
        });
        return function() {
            return _short.dxScrollMove.off(_this12.wrapperRef.current, {
                namespace: "dxScrollable"
            })
        }
    };
    _proto.getEventArgs = function() {
        return _extends({
            event: this.eventForUserAction,
            scrollOffset: this.scrollOffset()
        }, (0, _scrollable_utils.getBoundaryProps)(this.props.direction, this.scrollOffset(), this.containerRef.current, 0))
    };
    _proto.lock = function() {
        this.locked = true
    };
    _proto.unlock = function() {
        if (!this.props.disabled) {
            this.locked = false
        }
    };
    _proto.windowResizeHandler = function() {
        var _this$props$onUpdated, _this$props;
        this.updateSizes();
        null === (_this$props$onUpdated = (_this$props = this.props).onUpdated) || void 0 === _this$props$onUpdated ? void 0 : _this$props$onUpdated.call(_this$props, this.getEventArgs())
    };
    _proto.updateSizes = function() {
        var containerEl = this.containerRef.current;
        var contentEl = this.contentRef.current;
        if ((0, _type.isDefined)(containerEl)) {
            this.set_containerClientWidth((function() {
                return containerEl.clientWidth
            }));
            this.set_containerClientHeight((function() {
                return containerEl.clientHeight
            }))
        }
        if ((0, _type.isDefined)(contentEl)) {
            this.set_contentClientWidth((function() {
                return contentEl.clientWidth
            }));
            this.set_contentClientHeight((function() {
                return contentEl.clientHeight
            }))
        }
    };
    _proto.moveScrollbars = function() {
        var _this13 = this;
        var _this$location = this.location(),
            left = _this$location.left,
            top = _this$location.top;
        if (this.direction.isHorizontal) {
            var scrollbarEl = this.horizontalScrollbarRef.current;
            if ((0, _type.isDefined)(scrollbarEl)) {
                this.horizontalScrollbarRef.current.moveScrollbar(left)
            }
        }
        if (this.direction.isVertical) {
            var _scrollbarEl = this.verticalScrollbarRef.current;
            if ((0, _type.isDefined)(_scrollbarEl)) {
                this.verticalScrollbarRef.current.moveScrollbar(top)
            }
        }
        this.set_needForceScrollbarsVisibility((function() {
            return true
        }));
        this.clearHideScrollbarTimeout();
        this.hideScrollbarTimeout = setTimeout((function() {
            _this13.set_needForceScrollbarsVisibility((function() {
                return false
            }))
        }), HIDE_SCROLLBAR_TIMEOUT)
    };
    _proto.clearHideScrollbarTimeout = function() {
        clearTimeout(this.hideScrollbarTimeout);
        this.hideScrollbarTimeout = void 0
    };
    _proto.location = function() {
        return {
            top: -this.containerRef.current.scrollTop,
            left: -this.containerRef.current.scrollLeft
        }
    };
    _proto.getInitEventData = function() {
        return {
            getDirection: this.tryGetAllowedDirection,
            validate: this.validate,
            isNative: true,
            scrollTarget: this.containerRef.current
        }
    };
    _proto.handleInit = function(e) {};
    _proto.handleMove = function(e) {
        if (this.locked) {
            e.cancel = true;
            return
        }
        if ((0, _type.isDefined)(this.tryGetAllowedDirection())) {
            e.originalEvent.isScrollingEvent = true
        }
    };
    _proto.tryGetAllowedDirection = function() {
        var _ScrollDirection = new _scroll_direction.ScrollDirection(this.props.direction),
            isBoth = _ScrollDirection.isBoth,
            isHorizontal = _ScrollDirection.isHorizontal,
            isVertical = _ScrollDirection.isVertical;
        var contentEl = this.contentRef.current;
        var containerEl = this.containerRef.current;
        var isOverflowVertical = isVertical && contentEl.clientHeight > containerEl.clientHeight || this.props.pullDownEnabled;
        var isOverflowHorizontal = isHorizontal && contentEl.clientWidth > containerEl.clientWidth || this.props.pullDownEnabled;
        if (isBoth && isOverflowVertical && isOverflowHorizontal) {
            return _consts.DIRECTION_BOTH
        }
        if (isHorizontal && isOverflowHorizontal) {
            return _consts.DIRECTION_HORIZONTAL
        }
        if (isVertical && isOverflowVertical) {
            return _consts.DIRECTION_VERTICAL
        }
        return
    };
    _proto.isLocked = function() {
        return this.locked
    };
    _proto.isScrollingOutOfBound = function(e) {
        var delta = e.delta,
            shiftKey = e.shiftKey;
        var _this$containerRef$cu = this.containerRef.current,
            clientHeight = _this$containerRef$cu.clientHeight,
            clientWidth = _this$containerRef$cu.clientWidth,
            scrollHeight = _this$containerRef$cu.scrollHeight,
            scrollLeft = _this$containerRef$cu.scrollLeft,
            scrollTop = _this$containerRef$cu.scrollTop,
            scrollWidth = _this$containerRef$cu.scrollWidth;
        if (delta > 0) {
            return shiftKey ? !scrollLeft : !scrollTop
        }
        return shiftKey ? clientWidth >= scrollWidth - scrollLeft : clientHeight >= scrollHeight - scrollTop
    };
    _proto.content = function() {
        return this.contentRef.current
    };
    _proto.update = function() {
        var contentEl = this.contentRef.current;
        if ((0, _type.isDefined)(contentEl)) {
            var _this$props$onUpdated2, _this$props2;
            this.updateSizes();
            null === (_this$props$onUpdated2 = (_this$props2 = this.props).onUpdated) || void 0 === _this$props$onUpdated2 ? void 0 : _this$props$onUpdated2.call(_this$props2, this.getEventArgs())
        }
    };
    _proto.release = function() {};
    _proto.scrollBy = function(distance) {
        var location = (0, _scrollable_utils.ensureLocation)(distance);
        if (this.direction.isVertical) {
            this.containerRef.current.scrollTop += Math.round(location.top)
        }
        if (this.direction.isHorizontal) {
            this.containerRef.current.scrollLeft += (0, _scrollable_utils.normalizeCoordinate)("left", Math.round(location.left), this.props.rtlEnabled)
        }
    };
    _proto.scrollTo = function(targetLocation) {
        var location = (0, _scrollable_utils.ensureLocation)(targetLocation);
        var containerPosition = this.scrollOffset();
        var top = location.top - containerPosition.top;
        var left = location.left - containerPosition.left;
        this.scrollBy({
            top: top,
            left: left
        })
    };
    _proto.scrollToElement = function(element, offset) {
        if (void 0 === element || null === element) {
            return
        }
        if (element.closest(".".concat(_consts.SCROLLABLE_CONTENT_CLASS))) {
            var scrollOffset = _extends({
                top: 0,
                left: 0,
                right: 0,
                bottom: 0
            }, offset);
            this.scrollTo({
                top: (0, _scrollable_utils.getElementLocation)(element, scrollOffset, _consts.DIRECTION_VERTICAL, this.containerRef.current, this.props.rtlEnabled),
                left: (0, _scrollable_utils.getElementLocation)(element, scrollOffset, _consts.DIRECTION_HORIZONTAL, this.containerRef.current, this.props.rtlEnabled)
            })
        }
    };
    _proto.scrollHeight = function() {
        return this.content().offsetHeight
    };
    _proto.scrollWidth = function() {
        return this.content().offsetWidth
    };
    _proto.scrollOffset = function() {
        var maxScrollLeftOffset = (0, _scrollable_utils.getMaxScrollOffset)("width", this.containerRef.current);
        return {
            left: (0, _scrollable_utils.getPublicCoordinate)("left", this.containerRef.current.scrollLeft, maxScrollLeftOffset, this.props.rtlEnabled),
            top: this.containerRef.current.scrollTop
        }
    };
    _proto.scrollTop = function() {
        return this.scrollOffset().top
    };
    _proto.scrollLeft = function() {
        return this.scrollOffset().left
    };
    _proto.clientHeight = function() {
        return this.containerRef.current.clientHeight
    };
    _proto.clientWidth = function() {
        return this.containerRef.current.clientWidth
    };
    _proto.validate = function(e) {
        var disabled = this.props.disabled;
        if (this.isLocked() || disabled || (0, _index.isDxMouseWheelEvent)(e) && this.isScrollingOutOfBound(e)) {
            return false
        }
        return (0, _type.isDefined)(this.tryGetAllowedDirection())
    };
    _proto.render = function() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props),
            containerClientWidth: this.containerClientWidth,
            containerClientHeight: this.containerClientHeight,
            contentClientWidth: this.contentClientWidth,
            contentClientHeight: this.contentClientHeight,
            needForceScrollbarsVisibility: this.needForceScrollbarsVisibility,
            topPocketState: this.topPocketState,
            isLoadPanelVisible: this.isLoadPanelVisible,
            scrollableRef: this.scrollableRef,
            wrapperRef: this.wrapperRef,
            contentRef: this.contentRef,
            containerRef: this.containerRef,
            verticalScrollbarRef: this.verticalScrollbarRef,
            horizontalScrollbarRef: this.horizontalScrollbarRef,
            getEventArgs: this.getEventArgs,
            lock: this.lock,
            unlock: this.unlock,
            fullScrollInactiveProp: this.fullScrollInactiveProp,
            windowResizeHandler: this.windowResizeHandler,
            updateSizes: this.updateSizes,
            moveScrollbars: this.moveScrollbars,
            clearHideScrollbarTimeout: this.clearHideScrollbarTimeout,
            location: this.location,
            getInitEventData: this.getInitEventData,
            handleInit: this.handleInit,
            handleMove: this.handleMove,
            tryGetAllowedDirection: this.tryGetAllowedDirection,
            isLocked: this.isLocked,
            isScrollingOutOfBound: this.isScrollingOutOfBound,
            cssClasses: this.cssClasses,
            direction: this.direction,
            useSimulatedScrollbar: this.useSimulatedScrollbar,
            restAttributes: this.restAttributes
        })
    };
    _createClass(ScrollableNative, [{
        key: "containerClientWidth",
        get: function() {
            var state = this._currentState || this.state;
            return state.containerClientWidth
        }
    }, {
        key: "containerClientHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.containerClientHeight
        }
    }, {
        key: "contentClientWidth",
        get: function() {
            var state = this._currentState || this.state;
            return state.contentClientWidth
        }
    }, {
        key: "contentClientHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.contentClientHeight
        }
    }, {
        key: "needForceScrollbarsVisibility",
        get: function() {
            var state = this._currentState || this.state;
            return state.needForceScrollbarsVisibility
        }
    }, {
        key: "topPocketState",
        get: function() {
            var state = this._currentState || this.state;
            return state.topPocketState
        }
    }, {
        key: "isLoadPanelVisible",
        get: function() {
            var state = this._currentState || this.state;
            return state.isLoadPanelVisible
        }
    }, {
        key: "fullScrollInactiveProp",
        get: function() {
            return this.props.direction === _consts.DIRECTION_HORIZONTAL ? "scrollTop" : "scrollLeft"
        }
    }, {
        key: "cssClasses",
        get: function() {
            var _classesMap;
            var _this$props3 = this.props,
                classes = _this$props3.classes,
                direction = _this$props3.direction,
                disabled = _this$props3.disabled,
                showScrollbar = _this$props3.showScrollbar;
            var classesMap = (_classesMap = {}, _defineProperty(_classesMap, "dx-scrollable dx-scrollable-native dx-scrollable-native-".concat(_devices.default.real().platform, " dx-scrollable-renovated"), true), _defineProperty(_classesMap, "dx-scrollable-".concat(direction), true), _defineProperty(_classesMap, _consts.SCROLLABLE_DISABLED_CLASS, !!disabled), _defineProperty(_classesMap, _consts.SCROLLABLE_SCROLLBAR_SIMULATED, showScrollbar && this.useSimulatedScrollbar), _defineProperty(_classesMap, _consts.SCROLLABLE_SCROLLBARS_HIDDEN, !showScrollbar), _defineProperty(_classesMap, "".concat(classes), !!classes), _classesMap);
            return (0, _combine_classes.combineClasses)(classesMap)
        }
    }, {
        key: "direction",
        get: function() {
            return new _scroll_direction.ScrollDirection(this.props.direction)
        }
    }, {
        key: "useSimulatedScrollbar",
        get: function() {
            if (!(0, _type.isDefined)(this.props.useSimulatedScrollbar)) {
                return _support.nativeScrolling && "android" === _devices.default.real().platform && !_browser.default.mozilla
            }
            return this.props.useSimulatedScrollbar
        }
    }, {
        key: "restAttributes",
        get: function() {
            var _this$props4 = this.props,
                restProps = (_this$props4.bounceEnabled, _this$props4.children, _this$props4.classes, _this$props4.direction, _this$props4.disabled, _this$props4.forceGeneratePockets, _this$props4.height, _this$props4.needScrollViewContentWrapper, _this$props4.needScrollViewLoadPanel, _this$props4.onKeyDown, _this$props4.onPullDown, _this$props4.onReachBottom, _this$props4.onScroll, _this$props4.onUpdated, _this$props4.pullDownEnabled, _this$props4.pulledDownText, _this$props4.pullingDownText, _this$props4.reachBottomEnabled, _this$props4.reachBottomText, _this$props4.refreshingText, _this$props4.rtlEnabled, _this$props4.scrollByContent, _this$props4.scrollByThumb, _this$props4.showScrollbar, _this$props4.useKeyboard, _this$props4.useNative, _this$props4.useSimulatedScrollbar, _this$props4.visible, _this$props4.width, _objectWithoutProperties(_this$props4, ["bounceEnabled", "children", "classes", "direction", "disabled", "forceGeneratePockets", "height", "needScrollViewContentWrapper", "needScrollViewLoadPanel", "onKeyDown", "onPullDown", "onReachBottom", "onScroll", "onUpdated", "pullDownEnabled", "pulledDownText", "pullingDownText", "reachBottomEnabled", "reachBottomText", "refreshingText", "rtlEnabled", "scrollByContent", "scrollByThumb", "showScrollbar", "useKeyboard", "useNative", "useSimulatedScrollbar", "visible", "width"]));
            return restProps
        }
    }]);
    return ScrollableNative
}(_vdom.InfernoComponent);
exports.ScrollableNative = ScrollableNative;
ScrollableNative.defaultProps = _extends({}, ScrollableNativePropsType);
