/**
 * DevExtreme (cjs/renovation/ui/scroll_view/scrollable_simulated.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.ScrollableSimulated = exports.viewFunction = void 0;
var _inferno = require("inferno");
var _vdom = require("@devextreme/vdom");
var _subscribe_to_event = require("../../utils/subscribe_to_event");
var _load_panel = require("./load_panel");
var _animated_scrollbar = require("./animated_scrollbar");
var _widget = require("../common/widget");
var _combine_classes = require("../../utils/combine_classes");
var _index = require("../../../events/utils/index");
var _window = require("../../../core/utils/window");
var _type = require("../../../core/utils/type");
var _scrollable_simulated_props = require("./scrollable_simulated_props");
var _common = require("../../../core/utils/common");
require("../../../events/gesture/emitter.gesture.scroll");
var _events_engine = _interopRequireDefault(require("../../../events/core/events_engine"));
var _scrollable_utils = require("./scrollable_utils");
var _scroll_direction = require("./utils/scroll_direction");
var _consts = require("./common/consts");
var _get_element_offset = require("./utils/get_element_offset");
var _get_element_style = require("./utils/get_element_style");
var _top_pocket = require("./top_pocket");
var _bottom_pocket = require("./bottom_pocket");
var _short = require("../../../events/short");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _objectWithoutProperties(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for (i = 0; i < sourceSymbolKeys.length; i++) {
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) {
                continue
            }
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) {
                continue
            }
            target[key] = source[key]
        }
    }
    return target
}

function _objectWithoutPropertiesLoose(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) {
            continue
        }
        target[key] = source[key]
    }
    return target
}

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        })
    } else {
        obj[key] = value
    }
    return obj
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}

function _assertThisInitialized(self) {
    if (void 0 === self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return self
}

function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass)
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}
var viewFunction = function(viewModel) {
    var baseContainerHeight = viewModel.baseContainerHeight,
        baseContainerWidth = viewModel.baseContainerWidth,
        baseContentHeight = viewModel.baseContentHeight,
        baseContentWidth = viewModel.baseContentWidth,
        bottomPocketClientHeight = viewModel.bottomPocketClientHeight,
        bottomPocketRef = viewModel.bottomPocketRef,
        containerClientHeight = viewModel.containerClientHeight,
        containerClientWidth = viewModel.containerClientWidth,
        containerRef = viewModel.containerRef,
        containerStyles = viewModel.containerStyles,
        contentHeight = viewModel.contentHeight,
        contentPositionChange = viewModel.contentPositionChange,
        contentRef = viewModel.contentRef,
        contentStyles = viewModel.contentStyles,
        contentTranslateOffsetChange = viewModel.contentTranslateOffsetChange,
        contentWidth = viewModel.contentWidth,
        cssClasses = viewModel.cssClasses,
        cursorEnterHandler = viewModel.cursorEnterHandler,
        cursorLeaveHandler = viewModel.cursorLeaveHandler,
        direction = viewModel.direction,
        horizontalScrollbarRef = viewModel.horizontalScrollbarRef,
        isHovered = viewModel.isHovered,
        isLoadPanelVisible = viewModel.isLoadPanelVisible,
        onBounce = viewModel.onBounce,
        onPullDown = viewModel.onPullDown,
        onReachBottom = viewModel.onReachBottom,
        onRelease = viewModel.onRelease,
        onWidgetKeyDown = viewModel.onWidgetKeyDown,
        pocketStateChange = viewModel.pocketStateChange,
        _viewModel$props = viewModel.props,
        bounceEnabled = _viewModel$props.bounceEnabled,
        children = _viewModel$props.children,
        contentTranslateOffset = _viewModel$props.contentTranslateOffset,
        disabled = _viewModel$props.disabled,
        forceGeneratePockets = _viewModel$props.forceGeneratePockets,
        height = _viewModel$props.height,
        inertiaEnabled = _viewModel$props.inertiaEnabled,
        needScrollViewContentWrapper = _viewModel$props.needScrollViewContentWrapper,
        needScrollViewLoadPanel = _viewModel$props.needScrollViewLoadPanel,
        pullDownEnabled = _viewModel$props.pullDownEnabled,
        pulledDownText = _viewModel$props.pulledDownText,
        pullingDownText = _viewModel$props.pullingDownText,
        reachBottomEnabled = _viewModel$props.reachBottomEnabled,
        reachBottomText = _viewModel$props.reachBottomText,
        refreshingText = _viewModel$props.refreshingText,
        rtlEnabled = _viewModel$props.rtlEnabled,
        scrollByThumb = _viewModel$props.scrollByThumb,
        showScrollbar = _viewModel$props.showScrollbar,
        useKeyboard = _viewModel$props.useKeyboard,
        visible = _viewModel$props.visible,
        width = _viewModel$props.width,
        restAttributes = viewModel.restAttributes,
        scaleRatioHeight = viewModel.scaleRatioHeight,
        scaleRatioWidth = viewModel.scaleRatioWidth,
        scrollableOffsetLeft = viewModel.scrollableOffsetLeft,
        scrollableOffsetTop = viewModel.scrollableOffsetTop,
        scrollableRef = viewModel.scrollableRef,
        topPocketClientHeight = viewModel.topPocketClientHeight,
        topPocketRef = viewModel.topPocketRef,
        topPocketState = viewModel.topPocketState,
        verticalScrollbarRef = viewModel.verticalScrollbarRef,
        windowResizeHandler = viewModel.windowResizeHandler,
        wrapperRef = viewModel.wrapperRef;
    return (0, _inferno.normalizeProps)((0, _inferno.createComponentVNode)(2, _widget.Widget, _extends({
        rootElementRef: scrollableRef,
        focusStateEnabled: useKeyboard,
        hoverStateEnabled: true,
        classes: cssClasses,
        disabled: disabled,
        rtlEnabled: rtlEnabled,
        height: height,
        width: width,
        visible: visible,
        onKeyDown: onWidgetKeyDown,
        onHoverStart: cursorEnterHandler,
        onHoverEnd: cursorLeaveHandler,
        onDimensionChanged: windowResizeHandler
    }, restAttributes, {
        children: [(0, _inferno.createVNode)(1, "div", _consts.SCROLLABLE_WRAPPER_CLASS, (0, _inferno.createVNode)(1, "div", _consts.SCROLLABLE_CONTAINER_CLASS, [(0, _inferno.createVNode)(1, "div", _consts.SCROLLABLE_CONTENT_CLASS, [forceGeneratePockets && (0, _inferno.createComponentVNode)(2, _top_pocket.TopPocket, {
            topPocketRef: topPocketRef,
            pullingDownText: pullingDownText,
            pulledDownText: pulledDownText,
            refreshingText: refreshingText,
            refreshStrategy: "simulated",
            useNative: false,
            pocketState: topPocketState,
            visible: !!pullDownEnabled
        }), needScrollViewContentWrapper ? (0, _inferno.createVNode)(1, "div", _consts.SCROLLVIEW_CONTENT_CLASS, children, 0) : (0, _inferno.createVNode)(1, "div", null, children, 0), forceGeneratePockets && (0, _inferno.createComponentVNode)(2, _bottom_pocket.BottomPocket, {
            bottomPocketRef: bottomPocketRef,
            reachBottomText: reachBottomText,
            visible: !!reachBottomEnabled
        })], 0, {
            style: (0, _vdom.normalizeStyles)(contentStyles)
        }, null, contentRef), direction.isHorizontal && (0, _inferno.createComponentVNode)(2, _animated_scrollbar.AnimatedScrollbar, {
            direction: "horizontal",
            scaleRatio: scaleRatioWidth,
            scrollableOffset: scrollableOffsetLeft,
            contentSize: contentWidth,
            containerSize: containerClientWidth,
            baseContentSize: baseContentWidth,
            baseContainerSize: baseContainerWidth,
            isScrollableHovered: isHovered,
            contentPositionChange: contentPositionChange,
            contentTranslateOffset: contentTranslateOffset,
            contentTranslateOffsetChange: contentTranslateOffsetChange,
            scrollByThumb: scrollByThumb,
            bounceEnabled: bounceEnabled,
            showScrollbar: showScrollbar,
            inertiaEnabled: inertiaEnabled,
            onBounce: onBounce
        }, null, horizontalScrollbarRef), direction.isVertical && (0, _inferno.createComponentVNode)(2, _animated_scrollbar.AnimatedScrollbar, {
            direction: "vertical",
            scaleRatio: scaleRatioHeight,
            scrollableOffset: scrollableOffsetTop,
            contentSize: contentHeight,
            containerSize: containerClientHeight,
            baseContentSize: baseContentHeight,
            baseContainerSize: baseContainerHeight,
            isScrollableHovered: isHovered,
            contentPositionChange: contentPositionChange,
            contentTranslateOffset: contentTranslateOffset,
            contentTranslateOffsetChange: contentTranslateOffsetChange,
            scrollByThumb: scrollByThumb,
            bounceEnabled: bounceEnabled,
            showScrollbar: showScrollbar,
            inertiaEnabled: inertiaEnabled,
            onBounce: onBounce,
            forceGeneratePockets: forceGeneratePockets,
            topPocketSize: topPocketClientHeight,
            bottomPocketSize: bottomPocketClientHeight,
            onPullDown: onPullDown,
            onRelease: onRelease,
            onReachBottom: onReachBottom,
            pullDownEnabled: pullDownEnabled,
            reachBottomEnabled: reachBottomEnabled,
            pocketState: topPocketState,
            pocketStateChange: pocketStateChange
        }, null, verticalScrollbarRef)], 0, {
            style: (0, _vdom.normalizeStyles)(containerStyles)
        }, null, containerRef), 2, null, null, wrapperRef), needScrollViewLoadPanel && (0, _inferno.createComponentVNode)(2, _load_panel.ScrollViewLoadPanel, {
            targetElement: scrollableRef,
            refreshingText: refreshingText,
            visible: isLoadPanelVisible
        })]
    })))
};
exports.viewFunction = viewFunction;
var ScrollableSimulated = function(_InfernoComponent) {
    _inheritsLoose(ScrollableSimulated, _InfernoComponent);

    function ScrollableSimulated(props) {
        var _this;
        _this = _InfernoComponent.call(this, props) || this;
        _this._currentState = null;
        _this.locked = false;
        _this.loadingIndicatorEnabled = true;
        _this.validDirections = {};
        _this.scrollableRef = (0, _inferno.createRef)();
        _this.wrapperRef = (0, _inferno.createRef)();
        _this.contentRef = (0, _inferno.createRef)();
        _this.containerRef = (0, _inferno.createRef)();
        _this.verticalScrollbarRef = (0, _inferno.createRef)();
        _this.horizontalScrollbarRef = (0, _inferno.createRef)();
        _this.topPocketRef = (0, _inferno.createRef)();
        _this.bottomPocketRef = (0, _inferno.createRef)();
        _this.state = {
            isHovered: false,
            scrollableOffsetLeft: 0,
            scrollableOffsetTop: 0,
            containerClientWidth: 0,
            containerClientHeight: 0,
            contentScrollWidth: 0,
            contentScrollHeight: 0,
            contentClientWidth: 0,
            contentClientHeight: 0,
            containerOffsetWidth: 0,
            containerOffsetHeight: 0,
            contentOffsetWidth: 0,
            contentOffsetHeight: 0,
            topPocketClientHeight: 0,
            bottomPocketClientHeight: 0,
            topPocketState: _consts.TopPocketState.STATE_RELEASED,
            isLoadPanelVisible: false,
            contentTranslateOffset: void 0 !== _this.props.contentTranslateOffset ? _this.props.contentTranslateOffset : _this.props.defaultContentTranslateOffset
        };
        _this.content = _this.content.bind(_assertThisInitialized(_this));
        _this.update = _this.update.bind(_assertThisInitialized(_this));
        _this.refresh = _this.refresh.bind(_assertThisInitialized(_this));
        _this.startLoading = _this.startLoading.bind(_assertThisInitialized(_this));
        _this.finishLoading = _this.finishLoading.bind(_assertThisInitialized(_this));
        _this.release = _this.release.bind(_assertThisInitialized(_this));
        _this.scrollBy = _this.scrollBy.bind(_assertThisInitialized(_this));
        _this.scrollTo = _this.scrollTo.bind(_assertThisInitialized(_this));
        _this.scrollToElement = _this.scrollToElement.bind(_assertThisInitialized(_this));
        _this.scrollHeight = _this.scrollHeight.bind(_assertThisInitialized(_this));
        _this.scrollWidth = _this.scrollWidth.bind(_assertThisInitialized(_this));
        _this.scrollOffset = _this.scrollOffset.bind(_assertThisInitialized(_this));
        _this.scrollTop = _this.scrollTop.bind(_assertThisInitialized(_this));
        _this.scrollLeft = _this.scrollLeft.bind(_assertThisInitialized(_this));
        _this.clientHeight = _this.clientHeight.bind(_assertThisInitialized(_this));
        _this.clientWidth = _this.clientWidth.bind(_assertThisInitialized(_this));
        _this.disposeWheelTimer = _this.disposeWheelTimer.bind(_assertThisInitialized(_this));
        _this.scrollEffect = _this.scrollEffect.bind(_assertThisInitialized(_this));
        _this.getEventArgs = _this.getEventArgs.bind(_assertThisInitialized(_this));
        _this.initEffect = _this.initEffect.bind(_assertThisInitialized(_this));
        _this.getInitEventData = _this.getInitEventData.bind(_assertThisInitialized(_this));
        _this.startEffect = _this.startEffect.bind(_assertThisInitialized(_this));
        _this.moveEffect = _this.moveEffect.bind(_assertThisInitialized(_this));
        _this.endEffect = _this.endEffect.bind(_assertThisInitialized(_this));
        _this.stopEffect = _this.stopEffect.bind(_assertThisInitialized(_this));
        _this.cancelEffect = _this.cancelEffect.bind(_assertThisInitialized(_this));
        _this.onBounce = _this.onBounce.bind(_assertThisInitialized(_this));
        _this.onPullDown = _this.onPullDown.bind(_assertThisInitialized(_this));
        _this.onRelease = _this.onRelease.bind(_assertThisInitialized(_this));
        _this.onReachBottom = _this.onReachBottom.bind(_assertThisInitialized(_this));
        _this.pocketStateChange = _this.pocketStateChange.bind(_assertThisInitialized(_this));
        _this.contentPositionChange = _this.contentPositionChange.bind(_assertThisInitialized(_this));
        _this.triggerScrollEvent = _this.triggerScrollEvent.bind(_assertThisInitialized(_this));
        _this.contentTranslateOffsetChange = _this.contentTranslateOffsetChange.bind(_assertThisInitialized(_this));
        _this.cursorEnterHandler = _this.cursorEnterHandler.bind(_assertThisInitialized(_this));
        _this.cursorLeaveHandler = _this.cursorLeaveHandler.bind(_assertThisInitialized(_this));
        _this.handleInit = _this.handleInit.bind(_assertThisInitialized(_this));
        _this.handleStart = _this.handleStart.bind(_assertThisInitialized(_this));
        _this.handleMove = _this.handleMove.bind(_assertThisInitialized(_this));
        _this.handleEnd = _this.handleEnd.bind(_assertThisInitialized(_this));
        _this.handleStop = _this.handleStop.bind(_assertThisInitialized(_this));
        _this.handleCancel = _this.handleCancel.bind(_assertThisInitialized(_this));
        _this.applyScaleRatio = _this.applyScaleRatio.bind(_assertThisInitialized(_this));
        _this.isCrossThumbScrolling = _this.isCrossThumbScrolling.bind(_assertThisInitialized(_this));
        _this.adjustDistance = _this.adjustDistance.bind(_assertThisInitialized(_this));
        _this.suppressDirections = _this.suppressDirections.bind(_assertThisInitialized(_this));
        _this.validateEvent = _this.validateEvent.bind(_assertThisInitialized(_this));
        _this.prepareDirections = _this.prepareDirections.bind(_assertThisInitialized(_this));
        _this.isContent = _this.isContent.bind(_assertThisInitialized(_this));
        _this.eventHandler = _this.eventHandler.bind(_assertThisInitialized(_this));
        _this.getDirection = _this.getDirection.bind(_assertThisInitialized(_this));
        _this.allowedDirection = _this.allowedDirection.bind(_assertThisInitialized(_this));
        _this.validate = _this.validate.bind(_assertThisInitialized(_this));
        _this.isLocked = _this.isLocked.bind(_assertThisInitialized(_this));
        _this.validateWheel = _this.validateWheel.bind(_assertThisInitialized(_this));
        _this.clearWheelValidationTimer = _this.clearWheelValidationTimer.bind(_assertThisInitialized(_this));
        _this.validateMove = _this.validateMove.bind(_assertThisInitialized(_this));
        _this.onWidgetKeyDown = _this.onWidgetKeyDown.bind(_assertThisInitialized(_this));
        _this.keyDownHandler = _this.keyDownHandler.bind(_assertThisInitialized(_this));
        _this.scrollByLine = _this.scrollByLine.bind(_assertThisInitialized(_this));
        _this.tryGetDevicePixelRatio = _this.tryGetDevicePixelRatio.bind(_assertThisInitialized(_this));
        _this.scrollByPage = _this.scrollByPage.bind(_assertThisInitialized(_this));
        _this.wheelDirection = _this.wheelDirection.bind(_assertThisInitialized(_this));
        _this.scrollToHome = _this.scrollToHome.bind(_assertThisInitialized(_this));
        _this.scrollToEnd = _this.scrollToEnd.bind(_assertThisInitialized(_this));
        _this.effectDisabledState = _this.effectDisabledState.bind(_assertThisInitialized(_this));
        _this.lock = _this.lock.bind(_assertThisInitialized(_this));
        _this.unlock = _this.unlock.bind(_assertThisInitialized(_this));
        _this.effectResetInactiveState = _this.effectResetInactiveState.bind(_assertThisInitialized(_this));
        _this.updateScrollbarSize = _this.updateScrollbarSize.bind(_assertThisInitialized(_this));
        _this.windowResizeHandler = _this.windowResizeHandler.bind(_assertThisInitialized(_this));
        _this.updateSizes = _this.updateSizes.bind(_assertThisInitialized(_this));
        return _this
    }
    var _proto = ScrollableSimulated.prototype;
    _proto.createEffects = function() {
        return [new _vdom.InfernoEffect(this.disposeWheelTimer, []), new _vdom.InfernoEffect(this.scrollEffect, [this.props.onScroll, this.props.direction, this.topPocketClientHeight]), new _vdom.InfernoEffect(this.initEffect, [this.props.direction, this.props.scrollByContent, this.props.scrollByThumb, this.props.onStop, this.topPocketClientHeight, this.contentScrollHeight, this.contentClientHeight, this.containerClientHeight, this.props.bounceEnabled, this.contentScrollWidth, this.contentClientWidth, this.containerClientWidth, this.props.disabled]), new _vdom.InfernoEffect(this.startEffect, [this.props.direction, this.props.onStart, this.topPocketClientHeight]), new _vdom.InfernoEffect(this.moveEffect, [this.props.direction]), new _vdom.InfernoEffect(this.endEffect, [this.props.direction, this.props.onEnd, this.topPocketClientHeight]), new _vdom.InfernoEffect(this.stopEffect, [this.props.direction]), new _vdom.InfernoEffect(this.cancelEffect, [this.props.direction]), new _vdom.InfernoEffect(this.effectDisabledState, [this.props.disabled]), new _vdom.InfernoEffect(this.effectResetInactiveState, [this.props.direction]), new _vdom.InfernoEffect(this.updateScrollbarSize, [this.isHovered, this.scrollableOffsetLeft, this.scrollableOffsetTop, this.containerClientWidth, this.containerClientHeight, this.contentScrollWidth, this.contentScrollHeight, this.contentClientWidth, this.contentClientHeight, this.containerOffsetWidth, this.containerOffsetHeight, this.contentOffsetWidth, this.contentOffsetHeight, this.topPocketClientHeight, this.bottomPocketClientHeight, this.topPocketState, this.isLoadPanelVisible, this.props.inertiaEnabled, this.props.onStart, this.props.onEnd, this.props.onBounce, this.props.onStop, this.__state_contentTranslateOffset, this.props.contentTranslateOffsetChange, this.props.contentPositionChange, this.props.defaultContentTranslateOffset, this.props.useNative, this.props.pullingDownText, this.props.pulledDownText, this.props.refreshingText, this.props.reachBottomText, this.props.children, this.props.direction, this.props.showScrollbar, this.props.scrollByThumb, this.props.bounceEnabled, this.props.scrollByContent, this.props.useKeyboard, this.props.classes, this.props.pullDownEnabled, this.props.reachBottomEnabled, this.props.forceGeneratePockets, this.props.needScrollViewContentWrapper, this.props.needScrollViewLoadPanel, this.props.onScroll, this.props.onUpdated, this.props.onPullDown, this.props.onReachBottom, this.props.disabled, this.props.height, this.props.onKeyDown, this.props.rtlEnabled, this.props.visible, this.props.width])]
    };
    _proto.updateEffects = function() {
        var _this$_effects$, _this$_effects$2, _this$_effects$3, _this$_effects$4, _this$_effects$5, _this$_effects$6, _this$_effects$7, _this$_effects$8, _this$_effects$9, _this$_effects$10;
        null === (_this$_effects$ = this._effects[1]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([this.props.onScroll, this.props.direction, this.topPocketClientHeight]);
        null === (_this$_effects$2 = this._effects[2]) || void 0 === _this$_effects$2 ? void 0 : _this$_effects$2.update([this.props.direction, this.props.scrollByContent, this.props.scrollByThumb, this.props.onStop, this.topPocketClientHeight, this.contentScrollHeight, this.contentClientHeight, this.containerClientHeight, this.props.bounceEnabled, this.contentScrollWidth, this.contentClientWidth, this.containerClientWidth, this.props.disabled]);
        null === (_this$_effects$3 = this._effects[3]) || void 0 === _this$_effects$3 ? void 0 : _this$_effects$3.update([this.props.direction, this.props.onStart, this.topPocketClientHeight]);
        null === (_this$_effects$4 = this._effects[4]) || void 0 === _this$_effects$4 ? void 0 : _this$_effects$4.update([this.props.direction]);
        null === (_this$_effects$5 = this._effects[5]) || void 0 === _this$_effects$5 ? void 0 : _this$_effects$5.update([this.props.direction, this.props.onEnd, this.topPocketClientHeight]);
        null === (_this$_effects$6 = this._effects[6]) || void 0 === _this$_effects$6 ? void 0 : _this$_effects$6.update([this.props.direction]);
        null === (_this$_effects$7 = this._effects[7]) || void 0 === _this$_effects$7 ? void 0 : _this$_effects$7.update([this.props.direction]);
        null === (_this$_effects$8 = this._effects[8]) || void 0 === _this$_effects$8 ? void 0 : _this$_effects$8.update([this.props.disabled]);
        null === (_this$_effects$9 = this._effects[9]) || void 0 === _this$_effects$9 ? void 0 : _this$_effects$9.update([this.props.direction]);
        null === (_this$_effects$10 = this._effects[10]) || void 0 === _this$_effects$10 ? void 0 : _this$_effects$10.update([this.isHovered, this.scrollableOffsetLeft, this.scrollableOffsetTop, this.containerClientWidth, this.containerClientHeight, this.contentScrollWidth, this.contentScrollHeight, this.contentClientWidth, this.contentClientHeight, this.containerOffsetWidth, this.containerOffsetHeight, this.contentOffsetWidth, this.contentOffsetHeight, this.topPocketClientHeight, this.bottomPocketClientHeight, this.topPocketState, this.isLoadPanelVisible, this.props.inertiaEnabled, this.props.onStart, this.props.onEnd, this.props.onBounce, this.props.onStop, this.__state_contentTranslateOffset, this.props.contentTranslateOffsetChange, this.props.contentPositionChange, this.props.defaultContentTranslateOffset, this.props.useNative, this.props.pullingDownText, this.props.pulledDownText, this.props.refreshingText, this.props.reachBottomText, this.props.children, this.props.direction, this.props.showScrollbar, this.props.scrollByThumb, this.props.bounceEnabled, this.props.scrollByContent, this.props.useKeyboard, this.props.classes, this.props.pullDownEnabled, this.props.reachBottomEnabled, this.props.forceGeneratePockets, this.props.needScrollViewContentWrapper, this.props.needScrollViewLoadPanel, this.props.onScroll, this.props.onUpdated, this.props.onPullDown, this.props.onReachBottom, this.props.disabled, this.props.height, this.props.onKeyDown, this.props.rtlEnabled, this.props.visible, this.props.width])
    };
    _proto.set_isHovered = function(value) {
        var _this2 = this;
        this.setState((function(state) {
            _this2._currentState = state;
            var newValue = value();
            _this2._currentState = null;
            return {
                isHovered: newValue
            }
        }))
    };
    _proto.set_scrollableOffsetLeft = function(value) {
        var _this3 = this;
        this.setState((function(state) {
            _this3._currentState = state;
            var newValue = value();
            _this3._currentState = null;
            return {
                scrollableOffsetLeft: newValue
            }
        }))
    };
    _proto.set_scrollableOffsetTop = function(value) {
        var _this4 = this;
        this.setState((function(state) {
            _this4._currentState = state;
            var newValue = value();
            _this4._currentState = null;
            return {
                scrollableOffsetTop: newValue
            }
        }))
    };
    _proto.set_containerClientWidth = function(value) {
        var _this5 = this;
        this.setState((function(state) {
            _this5._currentState = state;
            var newValue = value();
            _this5._currentState = null;
            return {
                containerClientWidth: newValue
            }
        }))
    };
    _proto.set_containerClientHeight = function(value) {
        var _this6 = this;
        this.setState((function(state) {
            _this6._currentState = state;
            var newValue = value();
            _this6._currentState = null;
            return {
                containerClientHeight: newValue
            }
        }))
    };
    _proto.set_contentScrollWidth = function(value) {
        var _this7 = this;
        this.setState((function(state) {
            _this7._currentState = state;
            var newValue = value();
            _this7._currentState = null;
            return {
                contentScrollWidth: newValue
            }
        }))
    };
    _proto.set_contentScrollHeight = function(value) {
        var _this8 = this;
        this.setState((function(state) {
            _this8._currentState = state;
            var newValue = value();
            _this8._currentState = null;
            return {
                contentScrollHeight: newValue
            }
        }))
    };
    _proto.set_contentClientWidth = function(value) {
        var _this9 = this;
        this.setState((function(state) {
            _this9._currentState = state;
            var newValue = value();
            _this9._currentState = null;
            return {
                contentClientWidth: newValue
            }
        }))
    };
    _proto.set_contentClientHeight = function(value) {
        var _this10 = this;
        this.setState((function(state) {
            _this10._currentState = state;
            var newValue = value();
            _this10._currentState = null;
            return {
                contentClientHeight: newValue
            }
        }))
    };
    _proto.set_containerOffsetWidth = function(value) {
        var _this11 = this;
        this.setState((function(state) {
            _this11._currentState = state;
            var newValue = value();
            _this11._currentState = null;
            return {
                containerOffsetWidth: newValue
            }
        }))
    };
    _proto.set_containerOffsetHeight = function(value) {
        var _this12 = this;
        this.setState((function(state) {
            _this12._currentState = state;
            var newValue = value();
            _this12._currentState = null;
            return {
                containerOffsetHeight: newValue
            }
        }))
    };
    _proto.set_contentOffsetWidth = function(value) {
        var _this13 = this;
        this.setState((function(state) {
            _this13._currentState = state;
            var newValue = value();
            _this13._currentState = null;
            return {
                contentOffsetWidth: newValue
            }
        }))
    };
    _proto.set_contentOffsetHeight = function(value) {
        var _this14 = this;
        this.setState((function(state) {
            _this14._currentState = state;
            var newValue = value();
            _this14._currentState = null;
            return {
                contentOffsetHeight: newValue
            }
        }))
    };
    _proto.set_topPocketClientHeight = function(value) {
        var _this15 = this;
        this.setState((function(state) {
            _this15._currentState = state;
            var newValue = value();
            _this15._currentState = null;
            return {
                topPocketClientHeight: newValue
            }
        }))
    };
    _proto.set_bottomPocketClientHeight = function(value) {
        var _this16 = this;
        this.setState((function(state) {
            _this16._currentState = state;
            var newValue = value();
            _this16._currentState = null;
            return {
                bottomPocketClientHeight: newValue
            }
        }))
    };
    _proto.set_topPocketState = function(value) {
        var _this17 = this;
        this.setState((function(state) {
            _this17._currentState = state;
            var newValue = value();
            _this17._currentState = null;
            return {
                topPocketState: newValue
            }
        }))
    };
    _proto.set_isLoadPanelVisible = function(value) {
        var _this18 = this;
        this.setState((function(state) {
            _this18._currentState = state;
            var newValue = value();
            _this18._currentState = null;
            return {
                isLoadPanelVisible: newValue
            }
        }))
    };
    _proto.set_contentTranslateOffset = function(value) {
        var _this19 = this;
        this.setState((function(state) {
            var _this19$props$content, _this19$props;
            _this19._currentState = state;
            var newValue = value();
            null === (_this19$props$content = (_this19$props = _this19.props).contentTranslateOffsetChange) || void 0 === _this19$props$content ? void 0 : _this19$props$content.call(_this19$props, newValue);
            _this19._currentState = null;
            return {
                contentTranslateOffset: newValue
            }
        }))
    };
    _proto.disposeWheelTimer = function() {
        var _this20 = this;
        return function() {
            return _this20.clearWheelValidationTimer()
        }
    };
    _proto.scrollEffect = function() {
        var _this21 = this;
        return (0, _subscribe_to_event.subscribeToScrollEvent)(this.containerRef.current, (function() {
            var _this21$props$onScrol, _this21$props;
            return null === (_this21$props$onScrol = (_this21$props = _this21.props).onScroll) || void 0 === _this21$props$onScrol ? void 0 : _this21$props$onScrol.call(_this21$props, _this21.getEventArgs())
        }))
    };
    _proto.initEffect = function() {
        var _this22 = this;
        _short.dxScrollInit.on(this.wrapperRef.current, (function(e) {
            _this22.handleInit(e)
        }), this.getInitEventData(), {
            namespace: "dxScrollable"
        });
        return function() {
            return _short.dxScrollInit.off(_this22.wrapperRef.current, {
                namespace: "dxScrollable"
            })
        }
    };
    _proto.startEffect = function() {
        var _this23 = this;
        _short.dxScrollStart.on(this.wrapperRef.current, (function(e) {
            _this23.handleStart(e)
        }), {
            namespace: "dxScrollable"
        });
        return function() {
            return _short.dxScrollStart.off(_this23.wrapperRef.current, {
                namespace: "dxScrollable"
            })
        }
    };
    _proto.moveEffect = function() {
        var _this24 = this;
        _short.dxScrollMove.on(this.wrapperRef.current, (function(e) {
            _this24.handleMove(e)
        }), {
            namespace: "dxScrollable"
        });
        return function() {
            return _short.dxScrollMove.off(_this24.wrapperRef.current, {
                namespace: "dxScrollable"
            })
        }
    };
    _proto.endEffect = function() {
        var _this25 = this;
        _short.dxScrollEnd.on(this.wrapperRef.current, (function(e) {
            _this25.handleEnd(e)
        }), {
            namespace: "dxScrollable"
        });
        return function() {
            return _short.dxScrollEnd.off(_this25.wrapperRef.current, {
                namespace: "dxScrollable"
            })
        }
    };
    _proto.stopEffect = function() {
        var _this26 = this;
        _short.dxScrollStop.on(this.wrapperRef.current, (function() {
            _this26.handleStop()
        }), {
            namespace: "dxScrollable"
        });
        return function() {
            return _short.dxScrollStop.off(_this26.wrapperRef.current, {
                namespace: "dxScrollable"
            })
        }
    };
    _proto.cancelEffect = function() {
        var _this27 = this;
        _short.dxScrollCancel.on(this.wrapperRef.current, (function(event) {
            _this27.handleCancel(event)
        }), {
            namespace: "dxScrollable"
        });
        return function() {
            return _short.dxScrollCancel.off(_this27.wrapperRef.current, {
                namespace: "dxScrollable"
            })
        }
    };
    _proto.effectDisabledState = function() {
        if (this.props.disabled) {
            this.lock()
        } else {
            this.unlock()
        }
    };
    _proto.effectResetInactiveState = function() {
        var containerEl = this.containerRef.current;
        if (this.props.direction === _consts.DIRECTION_BOTH || !(0, _type.isDefined)(containerEl)) {
            return
        }
        containerEl[this.fullScrollInactiveProp] = 0
    };
    _proto.updateScrollbarSize = function() {
        var _this28 = this;
        this.set_scrollableOffsetLeft((function() {
            return _this28.scrollableOffset.left
        }));
        this.set_scrollableOffsetTop((function() {
            return _this28.scrollableOffset.top
        }));
        this.updateSizes()
    };
    _proto.startLoading = function() {
        if (this.loadingIndicatorEnabled) {
            this.set_isLoadPanelVisible((function() {
                return true
            }))
        }
        this.lock()
    };
    _proto.finishLoading = function() {
        this.set_isLoadPanelVisible((function() {
            return false
        }));
        this.unlock()
    };
    _proto.getEventArgs = function() {
        var scrollOffset = this.scrollOffset();
        return _extends({
            event: this.eventForUserAction,
            scrollOffset: scrollOffset
        }, (0, _scrollable_utils.getBoundaryProps)(this.props.direction, scrollOffset, this.containerRef.current, this.topPocketClientHeight))
    };
    _proto.getInitEventData = function() {
        return {
            getDirection: this.getDirection,
            validate: this.validate,
            isNative: false,
            scrollTarget: this.containerRef.current
        }
    };
    _proto.onBounce = function() {
        var _this$props$onBounce, _this$props;
        null === (_this$props$onBounce = (_this$props = this.props).onBounce) || void 0 === _this$props$onBounce ? void 0 : _this$props$onBounce.call(_this$props, this.getEventArgs())
    };
    _proto.onPullDown = function() {
        var _this$props$onPullDow, _this$props2;
        this.loadingIndicatorEnabled = false;
        this.startLoading();
        null === (_this$props$onPullDow = (_this$props2 = this.props).onPullDown) || void 0 === _this$props$onPullDow ? void 0 : _this$props$onPullDow.call(_this$props2, {})
    };
    _proto.onRelease = function() {
        var _this$props$onUpdated, _this$props3;
        this.loadingIndicatorEnabled = true;
        this.finishLoading();
        null === (_this$props$onUpdated = (_this$props3 = this.props).onUpdated) || void 0 === _this$props$onUpdated ? void 0 : _this$props$onUpdated.call(_this$props3, this.getEventArgs())
    };
    _proto.onReachBottom = function() {
        var _this$props$onReachBo, _this$props4;
        this.loadingIndicatorEnabled = false;
        this.startLoading();
        null === (_this$props$onReachBo = (_this$props4 = this.props).onReachBottom) || void 0 === _this$props$onReachBo ? void 0 : _this$props$onReachBo.call(_this$props4, {})
    };
    _proto.pocketStateChange = function(state) {
        this.set_topPocketState((function() {
            return state
        }))
    };
    _proto.contentPositionChange = function(scrollProp, location, ratio) {
        var containerEl = this.containerRef.current;
        containerEl[scrollProp] = -location / ratio;
        if (Math.abs(location - containerEl[scrollProp] * ratio) > 1) {
            this.triggerScrollEvent()
        }
    };
    _proto.triggerScrollEvent = function() {
        _events_engine.default.triggerHandler(this.containerRef.current, {
            type: "scroll"
        })
    };
    _proto.contentTranslateOffsetChange = function(translateOffset) {
        var _this29 = this;
        this.set_contentTranslateOffset((function() {
            return _extends({}, _this29.__state_contentTranslateOffset, translateOffset)
        }))
    };
    _proto.cursorEnterHandler = function() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_isHovered((function() {
                return true
            }))
        }
    };
    _proto.cursorLeaveHandler = function() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_isHovered((function() {
                return false
            }))
        }
    };
    _proto.handleInit = function(e) {
        var _this$props$onStop, _this$props5;
        this.suppressDirections(e);
        this.eventForUserAction = e;
        var crossThumbScrolling = this.isCrossThumbScrolling(e);
        this.eventHandler((function(scrollbar) {
            return scrollbar.initHandler(e, crossThumbScrolling)
        }));
        null === (_this$props$onStop = (_this$props5 = this.props).onStop) || void 0 === _this$props$onStop ? void 0 : _this$props$onStop.call(_this$props5, this.getEventArgs())
    };
    _proto.handleStart = function(e) {
        var _this$props$onStart, _this$props6;
        this.eventForUserAction = e;
        this.eventHandler((function(scrollbar) {
            return scrollbar.startHandler()
        }));
        null === (_this$props$onStart = (_this$props6 = this.props).onStart) || void 0 === _this$props$onStart ? void 0 : _this$props$onStart.call(_this$props6, this.getEventArgs())
    };
    _proto.handleMove = function(e) {
        if (this.isLocked()) {
            e.cancel = true;
            return
        }
        e.preventDefault();
        this.adjustDistance(e, "delta");
        this.eventForUserAction = e;
        this.eventHandler((function(scrollbar) {
            return scrollbar.moveHandler(e.delta)
        }))
    };
    _proto.handleEnd = function(e) {
        var _this$props$onEnd, _this$props7;
        this.adjustDistance(e, "velocity");
        this.eventForUserAction = e;
        this.eventHandler((function(scrollbar) {
            return scrollbar.endHandler(e.velocity)
        }));
        null === (_this$props$onEnd = (_this$props7 = this.props).onEnd) || void 0 === _this$props$onEnd ? void 0 : _this$props$onEnd.call(_this$props7, this.getEventArgs())
    };
    _proto.handleStop = function() {
        this.eventHandler((function(scrollbar) {
            return scrollbar.stopHandler()
        }))
    };
    _proto.handleCancel = function(e) {
        this.eventForUserAction = e;
        this.eventHandler((function(scrollbar) {
            return scrollbar.endHandler({
                x: 0,
                y: 0
            })
        }))
    };
    _proto.applyScaleRatio = function(targetLocation) {
        var currentTargetLocation = targetLocation;
        if (this.direction.isVertical && (0, _type.isDefined)(targetLocation.top)) {
            currentTargetLocation.top *= this.scaleRatioHeight
        }
        if (this.direction.isHorizontal && (0, _type.isDefined)(targetLocation.left)) {
            currentTargetLocation.left *= this.scaleRatioWidth
        }
        return currentTargetLocation
    };
    _proto.isCrossThumbScrolling = function(e) {
        var target = e.originalEvent.target;
        var verticalScrolling;
        var horizontalScrolling;
        if (this.direction.isVertical) {
            verticalScrolling = this.props.scrollByThumb && this.verticalScrollbarRef.current.isThumb(target)
        }
        if (this.direction.isHorizontal) {
            horizontalScrolling = this.props.scrollByThumb && this.horizontalScrollbarRef.current.isThumb(target)
        }
        return verticalScrolling || horizontalScrolling
    };
    _proto.adjustDistance = function(e, property) {
        var distance = e[property];
        distance.x *= this.validDirections[_consts.DIRECTION_HORIZONTAL] ? 1 : 0;
        distance.y *= this.validDirections[_consts.DIRECTION_VERTICAL] ? 1 : 0;
        var devicePixelRatio = this.tryGetDevicePixelRatio();
        if (devicePixelRatio && (0, _index.isDxMouseWheelEvent)(e.originalEvent)) {
            distance.x = Math.round(distance.x / devicePixelRatio * 100) / 100;
            distance.y = Math.round(distance.y / devicePixelRatio * 100) / 100
        }
    };
    _proto.suppressDirections = function(e) {
        if ((0, _index.isDxMouseWheelEvent)(e.originalEvent)) {
            this.prepareDirections(true);
            return
        }
        this.prepareDirections(false);
        if (this.direction.isVertical && (0, _type.isDefined)(this.verticalScrollbarRef.current)) {
            var isValid = this.validateEvent(e, this.verticalScrollbarRef.current);
            this.validDirections[_consts.DIRECTION_VERTICAL] = isValid
        }
        if (this.direction.isHorizontal && (0, _type.isDefined)(this.horizontalScrollbarRef.current)) {
            var _isValid = this.validateEvent(e, this.horizontalScrollbarRef.current);
            this.validDirections[_consts.DIRECTION_HORIZONTAL] = _isValid
        }
    };
    _proto.validateEvent = function(e, scrollbarRef) {
        var _this$props8 = this.props,
            scrollByContent = _this$props8.scrollByContent,
            scrollByThumb = _this$props8.scrollByThumb;
        return scrollByThumb && scrollbarRef.validateEvent(e) || scrollByContent && this.isContent(e.originalEvent.target)
    };
    _proto.prepareDirections = function(value) {
        this.validDirections[_consts.DIRECTION_HORIZONTAL] = value;
        this.validDirections[_consts.DIRECTION_VERTICAL] = value
    };
    _proto.isContent = function(element) {
        var closest = element.closest(".dx-scrollable-simulated");
        if ((0, _type.isDefined)(closest)) {
            return closest === this.scrollableRef.current
        }
        return false
    };
    _proto.eventHandler = function(handler) {
        if (this.direction.isVertical) {
            handler(this.verticalScrollbarRef.current)
        }
        if (this.direction.isHorizontal) {
            handler(this.horizontalScrollbarRef.current)
        }
    };
    _proto.getDirection = function(e) {
        return (0, _index.isDxMouseWheelEvent)(e) ? this.wheelDirection(e) : this.allowedDirection()
    };
    _proto.allowedDirection = function() {
        return (0, _scrollable_utils.updateAllowedDirection)(this.allowedDirections, this.props.direction)
    };
    _proto.isLocked = function() {
        return this.locked
    };
    _proto.validateWheel = function(e) {
        var scrollbar = "horizontal" === this.wheelDirection(e) ? this.horizontalScrollbarRef.current : this.verticalScrollbarRef.current;
        var reachedMin = scrollbar.reachedMin();
        var reachedMax = scrollbar.reachedMax();
        var contentGreaterThanContainer = !reachedMin || !reachedMax;
        var locatedNotAtBound = !reachedMin && !reachedMax;
        var delta = e.delta;
        var scrollFromMin = reachedMin && delta > 0;
        var scrollFromMax = reachedMax && delta < 0;
        var validated = contentGreaterThanContainer && (locatedNotAtBound || scrollFromMin || scrollFromMax);
        validated = validated || void 0 !== this.validateWheelTimer;
        if (validated) {
            this.clearWheelValidationTimer();
            this.validateWheelTimer = setTimeout(this.clearWheelValidationTimer, _consts.VALIDATE_WHEEL_TIMEOUT)
        }
        return validated
    };
    _proto.clearWheelValidationTimer = function() {
        clearTimeout(this.validateWheelTimer);
        this.validateWheelTimer = void 0
    };
    _proto.validateMove = function(e) {
        if (!this.props.scrollByContent && !(0, _type.isDefined)(e.target.closest(".".concat(_consts.SCROLLABLE_SCROLLBAR_CLASS)))) {
            return false
        }
        return (0, _type.isDefined)(this.allowedDirection())
    };
    _proto.onWidgetKeyDown = function(options) {
        var onKeyDown = this.props.onKeyDown;
        var originalEvent = options.originalEvent;
        var result = null === onKeyDown || void 0 === onKeyDown ? void 0 : onKeyDown(options);
        if (null !== result && void 0 !== result && result.cancel) {
            return result
        }
        this.keyDownHandler(originalEvent);
        return
    };
    _proto.keyDownHandler = function(e) {
        var handled = true;
        switch ((0, _index.normalizeKeyName)(e)) {
            case _consts.KEY_CODES.DOWN:
                this.scrollByLine({
                    y: 1
                });
                break;
            case _consts.KEY_CODES.UP:
                this.scrollByLine({
                    y: -1
                });
                break;
            case _consts.KEY_CODES.RIGHT:
                this.scrollByLine({
                    x: 1
                });
                break;
            case _consts.KEY_CODES.LEFT:
                this.scrollByLine({
                    x: -1
                });
                break;
            case _consts.KEY_CODES.PAGE_DOWN:
                this.scrollByPage(1);
                break;
            case _consts.KEY_CODES.PAGE_UP:
                this.scrollByPage(-1);
                break;
            case _consts.KEY_CODES.HOME:
                this.scrollToHome();
                break;
            case _consts.KEY_CODES.END:
                this.scrollToEnd();
                break;
            default:
                handled = false
        }
        if (handled) {
            e.stopPropagation();
            e.preventDefault()
        }
    };
    _proto.scrollByLine = function(lines) {
        var devicePixelRatio = this.tryGetDevicePixelRatio();
        var scrollOffset = _consts.SCROLL_LINE_HEIGHT;
        if (devicePixelRatio) {
            scrollOffset = Math.abs(scrollOffset / devicePixelRatio * 100) / 100
        }
        this.scrollBy({
            top: (lines.y || 0) * scrollOffset,
            left: (lines.x || 0) * scrollOffset
        })
    };
    _proto.tryGetDevicePixelRatio = function() {
        if ((0, _window.hasWindow)()) {
            return (0, _window.getWindow)().devicePixelRatio
        }
        return
    };
    _proto.scrollByPage = function(page) {
        var _ScrollDirection = new _scroll_direction.ScrollDirection(this.wheelDirection()),
            isVertical = _ScrollDirection.isVertical;
        var distance = {};
        if (isVertical) {
            distance.top = page * this.containerRef.current.clientHeight
        } else {
            distance.left = page * this.containerRef.current.clientWidth
        }
        this.scrollBy(distance)
    };
    _proto.wheelDirection = function(e) {
        switch (this.props.direction) {
            case _consts.DIRECTION_HORIZONTAL:
                return _consts.DIRECTION_HORIZONTAL;
            case _consts.DIRECTION_VERTICAL:
                return _consts.DIRECTION_VERTICAL;
            default:
                return null !== e && void 0 !== e && e.shiftKey ? _consts.DIRECTION_HORIZONTAL : _consts.DIRECTION_VERTICAL
        }
    };
    _proto.scrollToHome = function() {
        var distance = _defineProperty({}, this.direction.isVertical ? "top" : "left", 0);
        this.scrollTo(distance)
    };
    _proto.scrollToEnd = function() {
        var _ScrollDirection2 = new _scroll_direction.ScrollDirection(this.wheelDirection()),
            isVertical = _ScrollDirection2.isVertical;
        var distance = {};
        if (isVertical) {
            distance.top = this.contentRef.current.clientHeight - this.containerRef.current.clientHeight
        } else {
            distance.left = this.contentRef.current.clientWidth - this.containerRef.current.clientWidth
        }
        this.scrollTo(distance)
    };
    _proto.lock = function() {
        this.locked = true
    };
    _proto.unlock = function() {
        if (!this.props.disabled) {
            this.locked = false
        }
    };
    _proto.windowResizeHandler = function() {
        var _this$props$onUpdated2, _this$props9;
        this.updateSizes();
        null === (_this$props$onUpdated2 = (_this$props9 = this.props).onUpdated) || void 0 === _this$props$onUpdated2 ? void 0 : _this$props$onUpdated2.call(_this$props9, this.getEventArgs())
    };
    _proto.updateSizes = function() {
        var containerEl = this.containerRef.current;
        var contentEl = this.contentRef.current;
        if ((0, _type.isDefined)(containerEl)) {
            this.set_containerClientWidth((function() {
                return containerEl.clientWidth
            }));
            this.set_containerClientHeight((function() {
                return containerEl.clientHeight
            }));
            this.set_containerOffsetWidth((function() {
                return containerEl.offsetWidth
            }));
            this.set_containerOffsetHeight((function() {
                return containerEl.offsetHeight
            }))
        }
        if ((0, _type.isDefined)(contentEl)) {
            this.set_contentClientWidth((function() {
                return contentEl.clientWidth
            }));
            this.set_contentClientHeight((function() {
                return contentEl.clientHeight
            }));
            this.set_contentScrollWidth((function() {
                return contentEl.scrollWidth
            }));
            this.set_contentScrollHeight((function() {
                return contentEl.scrollHeight
            }));
            this.set_contentOffsetWidth((function() {
                return contentEl.offsetWidth
            }));
            this.set_contentOffsetHeight((function() {
                return contentEl.offsetHeight
            }))
        }
        if (this.props.forceGeneratePockets) {
            if (this.props.pullDownEnabled) {
                var topPocketEl = this.topPocketRef.current;
                if ((0, _type.isDefined)(topPocketEl)) {
                    this.set_topPocketClientHeight((function() {
                        return topPocketEl.clientHeight
                    }))
                }
            }
            if (this.props.reachBottomEnabled) {
                var bottomPocketEl = this.bottomPocketRef.current;
                if ((0, _type.isDefined)(bottomPocketEl)) {
                    this.set_bottomPocketClientHeight((function() {
                        return bottomPocketEl.clientHeight
                    }))
                }
            }
        }
    };
    _proto.content = function() {
        return this.contentRef.current
    };
    _proto.update = function() {
        var contentEl = this.contentRef.current;
        if ((0, _type.isDefined)(contentEl)) {
            var _this$props$onUpdated3, _this$props10;
            this.updateSizes();
            null === (_this$props$onUpdated3 = (_this$props10 = this.props).onUpdated) || void 0 === _this$props$onUpdated3 ? void 0 : _this$props$onUpdated3.call(_this$props10, this.getEventArgs())
        }
    };
    _proto.refresh = function() {
        var _this$props$onPullDow2, _this$props11;
        this.set_topPocketState((function() {
            return _consts.TopPocketState.STATE_READY
        }));
        this.startLoading();
        null === (_this$props$onPullDow2 = (_this$props11 = this.props).onPullDown) || void 0 === _this$props$onPullDow2 ? void 0 : _this$props$onPullDow2.call(_this$props11, {})
    };
    _proto.release = function() {
        this.eventHandler((function(scrollbar) {
            return scrollbar.releaseHandler()
        }))
    };
    _proto.scrollBy = function(distance) {
        var _this$props$onStart2, _this$props12, _this$props$onEnd2, _this$props13;
        var location = (0, _scrollable_utils.normalizeLocation)(distance, this.props.direction);
        if (this.direction.isVertical) {
            var scrollbar = this.verticalScrollbarRef.current;
            location.top = scrollbar.boundLocation(location.top + scrollbar.getScrollLocation()) - scrollbar.getScrollLocation()
        }
        if (this.direction.isHorizontal) {
            var _scrollbar = this.horizontalScrollbarRef.current;
            location.left = _scrollbar.boundLocation(location.left + _scrollbar.getScrollLocation()) - _scrollbar.getScrollLocation()
        }
        this.prepareDirections(true);
        null === (_this$props$onStart2 = (_this$props12 = this.props).onStart) || void 0 === _this$props$onStart2 ? void 0 : _this$props$onStart2.call(_this$props12, this.getEventArgs());
        this.eventHandler((function(scrollbar) {
            return scrollbar.scrollByHandler({
                x: location.left || 0,
                y: location.top || 0
            })
        }));
        null === (_this$props$onEnd2 = (_this$props13 = this.props).onEnd) || void 0 === _this$props$onEnd2 ? void 0 : _this$props$onEnd2.call(_this$props13, this.getEventArgs())
    };
    _proto.scrollTo = function(targetLocation) {
        var location = (0, _scrollable_utils.normalizeLocation)(targetLocation, this.props.direction);
        var containerPosition = this.scrollOffset();
        location = this.applyScaleRatio(location);
        containerPosition = this.applyScaleRatio(containerPosition);
        var distance = {
            top: -containerPosition.top - (0, _common.ensureDefined)(location.top, -containerPosition.top),
            left: -containerPosition.left - (0, _common.ensureDefined)(location.left, -containerPosition.left)
        };
        if (!distance.top && !distance.left) {
            return
        }
        this.scrollBy(distance)
    };
    _proto.scrollToElement = function(element, offset) {
        if (void 0 === element || null === element) {
            return
        }
        if (element.closest(".".concat(_consts.SCROLLABLE_CONTENT_CLASS))) {
            var scrollOffset = _extends({
                top: 0,
                left: 0,
                right: 0,
                bottom: 0
            }, offset);
            this.scrollTo({
                top: (0, _scrollable_utils.getElementLocation)(element, scrollOffset, _consts.DIRECTION_VERTICAL, this.containerRef.current, this.props.rtlEnabled),
                left: (0, _scrollable_utils.getElementLocation)(element, scrollOffset, _consts.DIRECTION_HORIZONTAL, this.containerRef.current, this.props.rtlEnabled)
            })
        }
    };
    _proto.scrollHeight = function() {
        return this.content().offsetHeight
    };
    _proto.scrollWidth = function() {
        return this.content().offsetWidth
    };
    _proto.scrollOffset = function() {
        return {
            top: this.containerRef.current.scrollTop,
            left: this.containerRef.current.scrollLeft
        }
    };
    _proto.scrollTop = function() {
        return this.scrollOffset().top
    };
    _proto.scrollLeft = function() {
        return this.scrollOffset().left
    };
    _proto.clientHeight = function() {
        return this.containerRef.current.clientHeight
    };
    _proto.clientWidth = function() {
        return this.containerRef.current.clientWidth
    };
    _proto.validate = function(e) {
        if (this.isLocked() || this.props.disabled || (0, _index.isDxMouseWheelEvent)(e) && (0, _index.isCommandKeyPressed)({
                ctrlKey: e.ctrlKey,
                metaKey: e.metaKey
            })) {
            return false
        }
        if (this.props.bounceEnabled) {
            return true
        }
        return (0, _index.isDxMouseWheelEvent)(e) ? this.validateWheel(e) : this.validateMove(e)
    };
    _proto.render = function() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props, {
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            isHovered: this.isHovered,
            scrollableOffsetLeft: this.scrollableOffsetLeft,
            scrollableOffsetTop: this.scrollableOffsetTop,
            containerClientWidth: this.containerClientWidth,
            containerClientHeight: this.containerClientHeight,
            contentScrollWidth: this.contentScrollWidth,
            contentScrollHeight: this.contentScrollHeight,
            contentClientWidth: this.contentClientWidth,
            contentClientHeight: this.contentClientHeight,
            containerOffsetWidth: this.containerOffsetWidth,
            containerOffsetHeight: this.containerOffsetHeight,
            contentOffsetWidth: this.contentOffsetWidth,
            contentOffsetHeight: this.contentOffsetHeight,
            topPocketClientHeight: this.topPocketClientHeight,
            bottomPocketClientHeight: this.bottomPocketClientHeight,
            topPocketState: this.topPocketState,
            isLoadPanelVisible: this.isLoadPanelVisible,
            scrollableRef: this.scrollableRef,
            wrapperRef: this.wrapperRef,
            contentRef: this.contentRef,
            containerRef: this.containerRef,
            verticalScrollbarRef: this.verticalScrollbarRef,
            horizontalScrollbarRef: this.horizontalScrollbarRef,
            topPocketRef: this.topPocketRef,
            bottomPocketRef: this.bottomPocketRef,
            startLoading: this.startLoading,
            finishLoading: this.finishLoading,
            getEventArgs: this.getEventArgs,
            getInitEventData: this.getInitEventData,
            onBounce: this.onBounce,
            onPullDown: this.onPullDown,
            onRelease: this.onRelease,
            onReachBottom: this.onReachBottom,
            pocketStateChange: this.pocketStateChange,
            contentPositionChange: this.contentPositionChange,
            triggerScrollEvent: this.triggerScrollEvent,
            contentTranslateOffsetChange: this.contentTranslateOffsetChange,
            cursorEnterHandler: this.cursorEnterHandler,
            cursorLeaveHandler: this.cursorLeaveHandler,
            handleInit: this.handleInit,
            handleStart: this.handleStart,
            handleMove: this.handleMove,
            handleEnd: this.handleEnd,
            handleStop: this.handleStop,
            handleCancel: this.handleCancel,
            applyScaleRatio: this.applyScaleRatio,
            isCrossThumbScrolling: this.isCrossThumbScrolling,
            adjustDistance: this.adjustDistance,
            suppressDirections: this.suppressDirections,
            validateEvent: this.validateEvent,
            prepareDirections: this.prepareDirections,
            isContent: this.isContent,
            eventHandler: this.eventHandler,
            getDirection: this.getDirection,
            allowedDirection: this.allowedDirection,
            allowedDirections: this.allowedDirections,
            isLocked: this.isLocked,
            validateWheel: this.validateWheel,
            clearWheelValidationTimer: this.clearWheelValidationTimer,
            validateMove: this.validateMove,
            onWidgetKeyDown: this.onWidgetKeyDown,
            keyDownHandler: this.keyDownHandler,
            scrollByLine: this.scrollByLine,
            tryGetDevicePixelRatio: this.tryGetDevicePixelRatio,
            scrollByPage: this.scrollByPage,
            wheelDirection: this.wheelDirection,
            scrollToHome: this.scrollToHome,
            scrollToEnd: this.scrollToEnd,
            lock: this.lock,
            unlock: this.unlock,
            fullScrollInactiveProp: this.fullScrollInactiveProp,
            windowResizeHandler: this.windowResizeHandler,
            updateSizes: this.updateSizes,
            baseContentWidth: this.baseContentWidth,
            baseContainerWidth: this.baseContainerWidth,
            baseContentHeight: this.baseContentHeight,
            baseContainerHeight: this.baseContainerHeight,
            scaleRatioWidth: this.scaleRatioWidth,
            scaleRatioHeight: this.scaleRatioHeight,
            contentWidth: this.contentWidth,
            contentHeight: this.contentHeight,
            scrollableOffset: this.scrollableOffset,
            contentStyles: this.contentStyles,
            containerStyles: this.containerStyles,
            cssClasses: this.cssClasses,
            direction: this.direction,
            restAttributes: this.restAttributes
        })
    };
    _createClass(ScrollableSimulated, [{
        key: "isHovered",
        get: function() {
            var state = this._currentState || this.state;
            return state.isHovered
        }
    }, {
        key: "scrollableOffsetLeft",
        get: function() {
            var state = this._currentState || this.state;
            return state.scrollableOffsetLeft
        }
    }, {
        key: "scrollableOffsetTop",
        get: function() {
            var state = this._currentState || this.state;
            return state.scrollableOffsetTop
        }
    }, {
        key: "containerClientWidth",
        get: function() {
            var state = this._currentState || this.state;
            return state.containerClientWidth
        }
    }, {
        key: "containerClientHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.containerClientHeight
        }
    }, {
        key: "contentScrollWidth",
        get: function() {
            var state = this._currentState || this.state;
            return state.contentScrollWidth
        }
    }, {
        key: "contentScrollHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.contentScrollHeight
        }
    }, {
        key: "contentClientWidth",
        get: function() {
            var state = this._currentState || this.state;
            return state.contentClientWidth
        }
    }, {
        key: "contentClientHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.contentClientHeight
        }
    }, {
        key: "containerOffsetWidth",
        get: function() {
            var state = this._currentState || this.state;
            return state.containerOffsetWidth
        }
    }, {
        key: "containerOffsetHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.containerOffsetHeight
        }
    }, {
        key: "contentOffsetWidth",
        get: function() {
            var state = this._currentState || this.state;
            return state.contentOffsetWidth
        }
    }, {
        key: "contentOffsetHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.contentOffsetHeight
        }
    }, {
        key: "topPocketClientHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.topPocketClientHeight
        }
    }, {
        key: "bottomPocketClientHeight",
        get: function() {
            var state = this._currentState || this.state;
            return state.bottomPocketClientHeight
        }
    }, {
        key: "topPocketState",
        get: function() {
            var state = this._currentState || this.state;
            return state.topPocketState
        }
    }, {
        key: "isLoadPanelVisible",
        get: function() {
            var state = this._currentState || this.state;
            return state.isLoadPanelVisible
        }
    }, {
        key: "__state_contentTranslateOffset",
        get: function() {
            var state = this._currentState || this.state;
            return void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : state.contentTranslateOffset
        }
    }, {
        key: "allowedDirections",
        get: function() {
            return {
                vertical: this.direction.isVertical && (Math.round(-Math.max(this.contentHeight - this.containerClientHeight, 0)) < 0 || this.props.bounceEnabled),
                horizontal: this.direction.isHorizontal && (Math.round(-Math.max(this.contentWidth - this.containerClientWidth, 0)) < 0 || this.props.bounceEnabled)
            }
        }
    }, {
        key: "fullScrollInactiveProp",
        get: function() {
            return this.props.direction === _consts.DIRECTION_HORIZONTAL ? "scrollTop" : "scrollLeft"
        }
    }, {
        key: "baseContentWidth",
        get: function() {
            return this.contentOffsetWidth
        }
    }, {
        key: "baseContainerWidth",
        get: function() {
            return this.containerOffsetWidth
        }
    }, {
        key: "baseContentHeight",
        get: function() {
            return this.contentOffsetHeight
        }
    }, {
        key: "baseContainerHeight",
        get: function() {
            return this.containerOffsetHeight
        }
    }, {
        key: "scaleRatioWidth",
        get: function() {
            var _this$containerRef;
            if (!(0, _type.isDefined)(null === (_this$containerRef = this.containerRef) || void 0 === _this$containerRef ? void 0 : _this$containerRef.current)) {
                return 1
            }
            var scaleRatio = 1;
            if ((0, _window.hasWindow)()) {
                var realDimension = this.containerRef.current.clientWidth;
                var baseDimension = this.containerRef.current.offsetWidth;
                scaleRatio = Math.round(realDimension / baseDimension * 100) / 100
            }
            return scaleRatio
        }
    }, {
        key: "scaleRatioHeight",
        get: function() {
            var _this$containerRef2;
            if (!(0, _type.isDefined)(null === (_this$containerRef2 = this.containerRef) || void 0 === _this$containerRef2 ? void 0 : _this$containerRef2.current)) {
                return 1
            }
            var scaleRatio = 1;
            if ((0, _window.hasWindow)()) {
                var realDimension = this.containerRef.current.clientHeight;
                var baseDimension = this.containerRef.current.offsetHeight;
                scaleRatio = Math.round(realDimension / baseDimension * 100) / 100
            }
            return scaleRatio
        }
    }, {
        key: "contentWidth",
        get: function() {
            if (!(0, _type.isDefined)(this.contentRef) || !(0, _type.isDefined)(this.contentRef.current)) {
                return 0
            }
            var isOverflowHidden = "hidden" === (0, _get_element_style.getElementStyle)("overflowX", this.contentRef.current);
            if (!isOverflowHidden) {
                var containerScrollSize = this.contentScrollWidth * this.scaleRatioWidth;
                return Math.max(containerScrollSize, this.contentClientWidth)
            }
            return this.contentClientWidth
        }
    }, {
        key: "contentHeight",
        get: function() {
            if (!(0, _type.isDefined)(this.contentRef) || !(0, _type.isDefined)(this.contentRef.current)) {
                return 0
            }
            var isOverflowHidden = "hidden" === (0, _get_element_style.getElementStyle)("overflowY", this.contentRef.current);
            if (!isOverflowHidden) {
                var containerScrollSize = this.contentScrollHeight * this.scaleRatioHeight;
                return Math.max(containerScrollSize, this.contentClientHeight)
            }
            return this.contentClientHeight
        }
    }, {
        key: "scrollableOffset",
        get: function() {
            return (0, _get_element_offset.getElementOffset)(this.scrollableRef.current)
        }
    }, {
        key: "contentStyles",
        get: function() {
            var _left$top$this$__stat = _extends({}, {
                    left: 0,
                    top: 0
                }, this.__state_contentTranslateOffset),
                left = _left$top$this$__stat.left,
                top = _left$top$this$__stat.top;
            return {
                transform: "translate(".concat(left, "px, ").concat(top, "px)")
            }
        }
    }, {
        key: "containerStyles",
        get: function() {
            var touchDirection = this.allowedDirections.vertical ? "pan-x" : "";
            touchDirection = this.allowedDirections.horizontal ? "pan-y" : touchDirection;
            touchDirection = this.allowedDirections.vertical && this.allowedDirections.horizontal ? "none" : touchDirection;
            return {
                touchAction: touchDirection
            }
        }
    }, {
        key: "cssClasses",
        get: function() {
            var _classesMap;
            var _this$props14 = this.props,
                classes = _this$props14.classes,
                direction = _this$props14.direction,
                disabled = _this$props14.disabled,
                showScrollbar = _this$props14.showScrollbar;
            var classesMap = (_classesMap = {
                "dx-scrollable dx-scrollable-renovated": true
            }, _defineProperty(_classesMap, _consts.SCROLLABLE_SIMULATED_CLASS, true), _defineProperty(_classesMap, "dx-scrollable-".concat(direction), true), _defineProperty(_classesMap, _consts.SCROLLABLE_DISABLED_CLASS, !!disabled), _defineProperty(_classesMap, _consts.SCROLLABLE_SCROLLBARS_ALWAYSVISIBLE, "always" === showScrollbar), _defineProperty(_classesMap, _consts.SCROLLABLE_SCROLLBARS_HIDDEN, !showScrollbar), _defineProperty(_classesMap, "".concat(classes), !!classes), _classesMap);
            return (0, _combine_classes.combineClasses)(classesMap)
        }
    }, {
        key: "direction",
        get: function() {
            return new _scroll_direction.ScrollDirection(this.props.direction)
        }
    }, {
        key: "restAttributes",
        get: function() {
            var _this$props$contentTr = _extends({}, this.props, {
                    contentTranslateOffset: this.__state_contentTranslateOffset
                }),
                restProps = (_this$props$contentTr.bounceEnabled, _this$props$contentTr.children, _this$props$contentTr.classes, _this$props$contentTr.contentPositionChange, _this$props$contentTr.contentTranslateOffset, _this$props$contentTr.contentTranslateOffsetChange, _this$props$contentTr.defaultContentTranslateOffset, _this$props$contentTr.direction, _this$props$contentTr.disabled, _this$props$contentTr.forceGeneratePockets, _this$props$contentTr.height, _this$props$contentTr.inertiaEnabled, _this$props$contentTr.needScrollViewContentWrapper, _this$props$contentTr.needScrollViewLoadPanel, _this$props$contentTr.onBounce, _this$props$contentTr.onEnd, _this$props$contentTr.onKeyDown, _this$props$contentTr.onPullDown, _this$props$contentTr.onReachBottom, _this$props$contentTr.onScroll, _this$props$contentTr.onStart, _this$props$contentTr.onStop, _this$props$contentTr.onUpdated, _this$props$contentTr.pullDownEnabled, _this$props$contentTr.pulledDownText, _this$props$contentTr.pullingDownText, _this$props$contentTr.reachBottomEnabled, _this$props$contentTr.reachBottomText, _this$props$contentTr.refreshingText, _this$props$contentTr.rtlEnabled, _this$props$contentTr.scrollByContent, _this$props$contentTr.scrollByThumb, _this$props$contentTr.showScrollbar, _this$props$contentTr.useKeyboard, _this$props$contentTr.useNative, _this$props$contentTr.visible, _this$props$contentTr.width, _objectWithoutProperties(_this$props$contentTr, ["bounceEnabled", "children", "classes", "contentPositionChange", "contentTranslateOffset", "contentTranslateOffsetChange", "defaultContentTranslateOffset", "direction", "disabled", "forceGeneratePockets", "height", "inertiaEnabled", "needScrollViewContentWrapper", "needScrollViewLoadPanel", "onBounce", "onEnd", "onKeyDown", "onPullDown", "onReachBottom", "onScroll", "onStart", "onStop", "onUpdated", "pullDownEnabled", "pulledDownText", "pullingDownText", "reachBottomEnabled", "reachBottomText", "refreshingText", "rtlEnabled", "scrollByContent", "scrollByThumb", "showScrollbar", "useKeyboard", "useNative", "visible", "width"]));
            return restProps
        }
    }]);
    return ScrollableSimulated
}(_vdom.InfernoComponent);
exports.ScrollableSimulated = ScrollableSimulated;
ScrollableSimulated.defaultProps = _extends({}, _scrollable_simulated_props.ScrollableSimulatedPropsType);
