/**
 * DevExtreme (cjs/renovation/ui/scroll_view/scrollbar.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.Scrollbar = exports.ScrollbarPropsType = exports.viewFunction = void 0;
var _inferno = require("inferno");
var _vdom = require("@devextreme/vdom");
var _widget = require("../common/widget");
var _combine_classes = require("../../utils/combine_classes");
var _dom_adapter = _interopRequireDefault(require("../../../core/dom_adapter"));
var _type = require("../../../core/utils/type");
var _index = require("../../../events/utils/index");
var _scrollbar_props = require("./scrollbar_props");
var _consts = require("./common/consts");
var _short = require("../../../events/short");
var _scrollable_simulated_props = require("./scrollable_simulated_props");
var _scrollable_props = require("./scrollable_props");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _objectWithoutProperties(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for (i = 0; i < sourceSymbolKeys.length; i++) {
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) {
                continue
            }
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) {
                continue
            }
            target[key] = source[key]
        }
    }
    return target
}

function _objectWithoutPropertiesLoose(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) {
            continue
        }
        target[key] = source[key]
    }
    return target
}

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        })
    } else {
        obj[key] = value
    }
    return obj
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}

function _assertThisInitialized(self) {
    if (void 0 === self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return self
}

function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass)
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}
var OUT_BOUNDS_ACCELERATION = .5;
var THUMB_MIN_SIZE = 15;
var viewFunction = function(viewModel) {
    var cssClasses = viewModel.cssClasses,
        hoverStateEnabled = viewModel.hoverStateEnabled,
        isVisible = viewModel.isVisible,
        onHoverEnd = viewModel.onHoverEnd,
        onHoverStart = viewModel.onHoverStart,
        activeStateEnabled = viewModel.props.activeStateEnabled,
        restAttributes = viewModel.restAttributes,
        scrollRef = viewModel.scrollRef,
        scrollStyles = viewModel.scrollStyles,
        scrollbarRef = viewModel.scrollbarRef;
    return (0, _inferno.normalizeProps)((0, _inferno.createComponentVNode)(2, _widget.Widget, _extends({
        rootElementRef: scrollbarRef,
        classes: cssClasses,
        activeStateEnabled: activeStateEnabled,
        hoverStateEnabled: hoverStateEnabled,
        visible: isVisible,
        onHoverStart: onHoverStart,
        onHoverEnd: onHoverEnd
    }, restAttributes, {
        children: (0, _inferno.createVNode)(1, "div", viewModel.scrollClasses, (0, _inferno.createVNode)(1, "div", _consts.SCROLLABLE_SCROLL_CONTENT_CLASS), 2, {
            style: (0, _vdom.normalizeStyles)(scrollStyles)
        }, null, scrollRef)
    })))
};
exports.viewFunction = viewFunction;
var ScrollbarPropsType = {
    activeStateEnabled: _scrollbar_props.ScrollbarProps.activeStateEnabled,
    expandable: _scrollbar_props.ScrollbarProps.expandable,
    direction: _scrollbar_props.ScrollbarProps.direction,
    scaleRatio: _scrollbar_props.ScrollbarProps.scaleRatio,
    containerSize: _scrollbar_props.ScrollbarProps.containerSize,
    contentSize: _scrollbar_props.ScrollbarProps.contentSize,
    baseContainerSize: _scrollbar_props.ScrollbarProps.baseContainerSize,
    baseContentSize: _scrollbar_props.ScrollbarProps.baseContentSize,
    topPocketSize: _scrollbar_props.ScrollbarProps.topPocketSize,
    bottomPocketSize: _scrollbar_props.ScrollbarProps.bottomPocketSize,
    scrollableOffset: _scrollbar_props.ScrollbarProps.scrollableOffset,
    scrollByThumb: _scrollbar_props.ScrollbarProps.scrollByThumb,
    isScrollableHovered: _scrollbar_props.ScrollbarProps.isScrollableHovered,
    bounceEnabled: _scrollbar_props.ScrollbarProps.bounceEnabled,
    forceVisibility: _scrollbar_props.ScrollbarProps.forceVisibility,
    defaultPocketState: _scrollbar_props.ScrollbarProps.defaultPocketState,
    showScrollbar: _scrollbar_props.ScrollbarProps.showScrollbar,
    scrollByContent: _scrollbar_props.ScrollbarProps.scrollByContent,
    useKeyboard: _scrollbar_props.ScrollbarProps.useKeyboard,
    pullDownEnabled: _scrollable_props.ScrollableProps.pullDownEnabled,
    reachBottomEnabled: _scrollable_props.ScrollableProps.reachBottomEnabled,
    forceGeneratePockets: _scrollable_simulated_props.ScrollableSimulatedProps.forceGeneratePockets,
    needScrollViewContentWrapper: _scrollbar_props.ScrollbarProps.needScrollViewContentWrapper,
    needScrollViewLoadPanel: _scrollbar_props.ScrollbarProps.needScrollViewLoadPanel,
    defaultContentTranslateOffset: _scrollable_simulated_props.ScrollableSimulatedProps.contentTranslateOffset
};
exports.ScrollbarPropsType = ScrollbarPropsType;
var Scrollbar = function(_InfernoComponent) {
    _inheritsLoose(Scrollbar, _InfernoComponent);

    function Scrollbar(props) {
        var _this;
        _this = _InfernoComponent.call(this, props) || this;
        _this._currentState = null;
        _this.location = 0;
        _this.thumbScrolling = false;
        _this.crossThumbScrolling = false;
        _this.scrollbarRef = (0, _inferno.createRef)();
        _this.scrollRef = (0, _inferno.createRef)();
        _this.state = {
            showOnScrollByWheel: void 0,
            hovered: false,
            expanded: false,
            visibility: false,
            scrollLocation: 0,
            boundaryOffset: 0,
            maxOffset: 0,
            minOffset: 0,
            minLimit: 0,
            pocketState: void 0 !== _this.props.pocketState ? _this.props.pocketState : _this.props.defaultPocketState,
            contentTranslateOffset: void 0 !== _this.props.contentTranslateOffset ? _this.props.contentTranslateOffset : _this.props.defaultContentTranslateOffset
        };
        _this.updateBoundaryOffset = _this.updateBoundaryOffset.bind(_assertThisInitialized(_this));
        _this.updateMinOffset = _this.updateMinOffset.bind(_assertThisInitialized(_this));
        _this.pointerDownEffect = _this.pointerDownEffect.bind(_assertThisInitialized(_this));
        _this.pointerUpEffect = _this.pointerUpEffect.bind(_assertThisInitialized(_this));
        _this.isThumb = _this.isThumb.bind(_assertThisInitialized(_this));
        _this.isScrollbar = _this.isScrollbar.bind(_assertThisInitialized(_this));
        _this.validateEvent = _this.validateEvent.bind(_assertThisInitialized(_this));
        _this.reachedMin = _this.reachedMin.bind(_assertThisInitialized(_this));
        _this.reachedMax = _this.reachedMax.bind(_assertThisInitialized(_this));
        _this.getLocation = _this.getLocation.bind(_assertThisInitialized(_this));
        _this.getScrollLocation = _this.getScrollLocation.bind(_assertThisInitialized(_this));
        _this.setLocation = _this.setLocation.bind(_assertThisInitialized(_this));
        _this.inBounds = _this.inBounds.bind(_assertThisInitialized(_this));
        _this.boundLocation = _this.boundLocation.bind(_assertThisInitialized(_this));
        _this.getMinOffset = _this.getMinOffset.bind(_assertThisInitialized(_this));
        _this.getMaxOffset = _this.getMaxOffset.bind(_assertThisInitialized(_this));
        _this.initHandler = _this.initHandler.bind(_assertThisInitialized(_this));
        _this.startHandler = _this.startHandler.bind(_assertThisInitialized(_this));
        _this.moveHandler = _this.moveHandler.bind(_assertThisInitialized(_this));
        _this.hide = _this.hide.bind(_assertThisInitialized(_this));
        _this.endHandler = _this.endHandler.bind(_assertThisInitialized(_this));
        _this.stopHandler = _this.stopHandler.bind(_assertThisInitialized(_this));
        _this.scrollByHandler = _this.scrollByHandler.bind(_assertThisInitialized(_this));
        _this.scrollComplete = _this.scrollComplete.bind(_assertThisInitialized(_this));
        _this.pullDownRefreshing = _this.pullDownRefreshing.bind(_assertThisInitialized(_this));
        _this.reachBottomLoading = _this.reachBottomLoading.bind(_assertThisInitialized(_this));
        _this.executePullDown = _this.executePullDown.bind(_assertThisInitialized(_this));
        _this.executeReachBottom = _this.executeReachBottom.bind(_assertThisInitialized(_this));
        _this.scrollToBounds = _this.scrollToBounds.bind(_assertThisInitialized(_this));
        _this.stopComplete = _this.stopComplete.bind(_assertThisInitialized(_this));
        _this.resetThumbScrolling = _this.resetThumbScrolling.bind(_assertThisInitialized(_this));
        _this.scrollBy = _this.scrollBy.bind(_assertThisInitialized(_this));
        _this.stopScrolling = _this.stopScrolling.bind(_assertThisInitialized(_this));
        _this.prepareThumbScrolling = _this.prepareThumbScrolling.bind(_assertThisInitialized(_this));
        _this.moveToMouseLocation = _this.moveToMouseLocation.bind(_assertThisInitialized(_this));
        _this.scrollStep = _this.scrollStep.bind(_assertThisInitialized(_this));
        _this.suppressBounce = _this.suppressBounce.bind(_assertThisInitialized(_this));
        _this.moveScrollbar = _this.moveScrollbar.bind(_assertThisInitialized(_this));
        _this.updateContent = _this.updateContent.bind(_assertThisInitialized(_this));
        _this.releaseHandler = _this.releaseHandler.bind(_assertThisInitialized(_this));
        _this.release = _this.release.bind(_assertThisInitialized(_this));
        _this.stateReleased = _this.stateReleased.bind(_assertThisInitialized(_this));
        _this.setPocketState = _this.setPocketState.bind(_assertThisInitialized(_this));
        _this.isPullDown = _this.isPullDown.bind(_assertThisInitialized(_this));
        _this.isReachBottom = _this.isReachBottom.bind(_assertThisInitialized(_this));
        _this.containerToContentRatio = _this.containerToContentRatio.bind(_assertThisInitialized(_this));
        _this.baseContainerToContentRatio = _this.baseContainerToContentRatio.bind(_assertThisInitialized(_this));
        _this.expand = _this.expand.bind(_assertThisInitialized(_this));
        _this.collapse = _this.collapse.bind(_assertThisInitialized(_this));
        _this.onHoverStart = _this.onHoverStart.bind(_assertThisInitialized(_this));
        _this.onHoverEnd = _this.onHoverEnd.bind(_assertThisInitialized(_this));
        return _this
    }
    var _proto = Scrollbar.prototype;
    _proto.createEffects = function() {
        return [new _vdom.InfernoEffect(this.updateBoundaryOffset, [this.props.forceGeneratePockets, this.scrollLocation, this.props.pullDownEnabled, this.props.topPocketSize, this.boundaryOffset]), new _vdom.InfernoEffect(this.updateMinOffset, [this.props.forceGeneratePockets, this.props.contentSize, this.props.reachBottomEnabled, this.props.bottomPocketSize, this.props.pullDownEnabled, this.props.topPocketSize, this.props.containerSize]), new _vdom.InfernoEffect(this.pointerDownEffect, []), new _vdom.InfernoEffect(this.pointerUpEffect, []), new _vdom.InfernoEffect(this.updateContent, [this.scrollLocation, this.minOffset, this.props.forceGeneratePockets, this.props.pullDownEnabled, this.props.topPocketSize, this.props.contentPositionChange, this.props.direction, this.props.scaleRatio, this.props.contentTranslateOffsetChange])]
    };
    _proto.updateEffects = function() {
        var _this$_effects$, _this$_effects$2, _this$_effects$3, _this$_effects$4, _this$_effects$5;
        null === (_this$_effects$ = this._effects[0]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([this.props.forceGeneratePockets, this.scrollLocation, this.props.pullDownEnabled, this.props.topPocketSize, this.boundaryOffset]);
        null === (_this$_effects$2 = this._effects[1]) || void 0 === _this$_effects$2 ? void 0 : _this$_effects$2.update([this.props.forceGeneratePockets, this.props.contentSize, this.props.reachBottomEnabled, this.props.bottomPocketSize, this.props.pullDownEnabled, this.props.topPocketSize, this.props.containerSize]);
        null === (_this$_effects$3 = this._effects[2]) || void 0 === _this$_effects$3 ? void 0 : _this$_effects$3.update([]);
        null === (_this$_effects$4 = this._effects[3]) || void 0 === _this$_effects$4 ? void 0 : _this$_effects$4.update([]);
        null === (_this$_effects$5 = this._effects[4]) || void 0 === _this$_effects$5 ? void 0 : _this$_effects$5.update([this.scrollLocation, this.minOffset, this.props.forceGeneratePockets, this.props.pullDownEnabled, this.props.topPocketSize, this.props.contentPositionChange, this.props.direction, this.props.scaleRatio, this.props.contentTranslateOffsetChange])
    };
    _proto.set_showOnScrollByWheel = function(value) {
        var _this2 = this;
        this.setState((function(state) {
            _this2._currentState = state;
            var newValue = value();
            _this2._currentState = null;
            return {
                showOnScrollByWheel: newValue
            }
        }))
    };
    _proto.set_hovered = function(value) {
        var _this3 = this;
        this.setState((function(state) {
            _this3._currentState = state;
            var newValue = value();
            _this3._currentState = null;
            return {
                hovered: newValue
            }
        }))
    };
    _proto.set_expanded = function(value) {
        var _this4 = this;
        this.setState((function(state) {
            _this4._currentState = state;
            var newValue = value();
            _this4._currentState = null;
            return {
                expanded: newValue
            }
        }))
    };
    _proto.set_visibility = function(value) {
        var _this5 = this;
        this.setState((function(state) {
            _this5._currentState = state;
            var newValue = value();
            _this5._currentState = null;
            return {
                visibility: newValue
            }
        }))
    };
    _proto.set_scrollLocation = function(value) {
        var _this6 = this;
        this.setState((function(state) {
            _this6._currentState = state;
            var newValue = value();
            _this6._currentState = null;
            return {
                scrollLocation: newValue
            }
        }))
    };
    _proto.set_boundaryOffset = function(value) {
        var _this7 = this;
        this.setState((function(state) {
            _this7._currentState = state;
            var newValue = value();
            _this7._currentState = null;
            return {
                boundaryOffset: newValue
            }
        }))
    };
    _proto.set_maxOffset = function(value) {
        var _this8 = this;
        this.setState((function(state) {
            _this8._currentState = state;
            var newValue = value();
            _this8._currentState = null;
            return {
                maxOffset: newValue
            }
        }))
    };
    _proto.set_minOffset = function(value) {
        var _this9 = this;
        this.setState((function(state) {
            _this9._currentState = state;
            var newValue = value();
            _this9._currentState = null;
            return {
                minOffset: newValue
            }
        }))
    };
    _proto.set_minLimit = function(value) {
        var _this10 = this;
        this.setState((function(state) {
            _this10._currentState = state;
            var newValue = value();
            _this10._currentState = null;
            return {
                minLimit: newValue
            }
        }))
    };
    _proto.set_pocketState = function(value) {
        var _this11 = this;
        this.setState((function(state) {
            var _this11$props$pocketS, _this11$props;
            _this11._currentState = state;
            var newValue = value();
            null === (_this11$props$pocketS = (_this11$props = _this11.props).pocketStateChange) || void 0 === _this11$props$pocketS ? void 0 : _this11$props$pocketS.call(_this11$props, newValue);
            _this11._currentState = null;
            return {
                pocketState: newValue
            }
        }))
    };
    _proto.set_contentTranslateOffset = function(value) {
        var _this12 = this;
        this.setState((function(state) {
            var _this12$props$content, _this12$props;
            _this12._currentState = state;
            var newValue = value();
            null === (_this12$props$content = (_this12$props = _this12.props).contentTranslateOffsetChange) || void 0 === _this12$props$content ? void 0 : _this12$props$content.call(_this12$props, newValue);
            _this12._currentState = null;
            return {
                contentTranslateOffset: newValue
            }
        }))
    };
    _proto.updateBoundaryOffset = function() {
        var _this13 = this;
        if (this.props.forceGeneratePockets) {
            this.set_boundaryOffset((function() {
                return _this13.scrollLocation - _this13.topPocketSize
            }));
            this.set_maxOffset((function() {
                return _this13.boundaryOffset > 0 ? _this13.topPocketSize : 0
            }))
        }
    };
    _proto.updateMinOffset = function() {
        var _this14 = this;
        if (this.props.forceGeneratePockets) {
            this.set_minOffset((function() {
                return -Math.max(_this14.bottomBoundaryOffset + _this14.bottomPocketSize, 0)
            }))
        } else {
            this.set_minOffset((function() {
                return -Math.max(_this14.bottomBoundaryOffset, 0)
            }))
        }
    };
    _proto.pointerDownEffect = function() {
        var _this15 = this;
        _short.dxPointerDown.on(this.scrollRef.current, (function() {
            _this15.expand()
        }), {
            namespace: "dxScrollbar"
        });
        return function() {
            return _short.dxPointerDown.off(_this15.scrollRef.current, {
                namespace: "dxScrollbar"
            })
        }
    };
    _proto.pointerUpEffect = function() {
        var _this16 = this;
        _short.dxPointerUp.on(_dom_adapter.default.getDocument(), (function() {
            _this16.collapse()
        }), {
            namespace: "dxScrollbar"
        });
        return function() {
            return _short.dxPointerUp.off(_this16.scrollRef.current, {
                namespace: "dxScrollbar"
            })
        }
    };
    _proto.updateContent = function() {
        var _this$props$contentPo, _this$props, _this$props$contentTr, _this$props2;
        var location = this.scrollLocation;
        var currentTranslateOffset;
        if (location > 0) {
            currentTranslateOffset = location
        } else if (location <= this.minOffset) {
            currentTranslateOffset = location - this.minOffset
        } else {
            currentTranslateOffset = location % 1
        }
        if (this.props.forceGeneratePockets && this.props.pullDownEnabled) {
            currentTranslateOffset -= this.props.topPocketSize
        }
        null === (_this$props$contentPo = (_this$props = this.props).contentPositionChange) || void 0 === _this$props$contentPo ? void 0 : _this$props$contentPo.call(_this$props, this.fullScrollProp, location, this.props.scaleRatio);
        null === (_this$props$contentTr = (_this$props2 = this.props).contentTranslateOffsetChange) || void 0 === _this$props$contentTr ? void 0 : _this$props$contentTr.call(_this$props2, _defineProperty({}, this.scrollProp, currentTranslateOffset))
    };
    _proto.hide = function() {
        var _this17 = this;
        this.set_visibility((function() {
            return false
        }));
        if ((0, _type.isDefined)(this.showOnScrollByWheel) && "onScroll" === this.props.showScrollbar) {
            setTimeout((function() {
                _this17.set_showOnScrollByWheel((function() {
                    return
                }))
            }), _consts.HIDE_SCROLLBAR_TIMEOUT)
        }
    };
    _proto.pullDownRefreshing = function() {
        this.setPocketState(_consts.TopPocketState.STATE_REFRESHING);
        this.executePullDown()
    };
    _proto.reachBottomLoading = function() {
        this.executeReachBottom()
    };
    _proto.executePullDown = function() {
        var _this$props$onPullDow, _this$props3;
        null === (_this$props$onPullDow = (_this$props3 = this.props).onPullDown) || void 0 === _this$props$onPullDow ? void 0 : _this$props$onPullDow.call(_this$props3)
    };
    _proto.executeReachBottom = function() {
        var _this$props$onReachBo, _this$props4;
        null === (_this$props$onReachBo = (_this$props4 = this.props).onReachBottom) || void 0 === _this$props$onReachBo ? void 0 : _this$props$onReachBo.call(_this$props4)
    };
    _proto.scrollToBounds = function() {
        var _this$props$onAnimato, _this$props5;
        if (this.inBounds()) {
            this.hide();
            return
        }
        null === (_this$props$onAnimato = (_this$props5 = this.props).onAnimatorStart) || void 0 === _this$props$onAnimato ? void 0 : _this$props$onAnimato.call(_this$props5, "bounce")
    };
    _proto.resetThumbScrolling = function() {
        this.thumbScrolling = false;
        this.crossThumbScrolling = false
    };
    _proto.scrollBy = function(delta) {
        var distance = delta[this.axis];
        if (!this.inBounds()) {
            distance *= OUT_BOUNDS_ACCELERATION
        }
        this.scrollStep(distance)
    };
    _proto.stopScrolling = function() {
        var _this$props$onAnimato2, _this$props6;
        this.hide();
        null === (_this$props$onAnimato2 = (_this$props6 = this.props).onAnimatorCancel) || void 0 === _this$props$onAnimato2 ? void 0 : _this$props$onAnimato2.call(_this$props6)
    };
    _proto.prepareThumbScrolling = function(e, currentCrossThumbScrolling) {
        if ((0, _index.isDxMouseWheelEvent)(e.originalEvent)) {
            if ("onScroll" === this.props.showScrollbar) {
                this.set_showOnScrollByWheel((function() {
                    return true
                }))
            }
            return
        }
        var target = e.originalEvent.target;
        var scrollbarClicked = this.props.scrollByThumb && this.isScrollbar(target);
        if (scrollbarClicked) {
            this.moveToMouseLocation(e)
        }
        var currentThumbScrolling = scrollbarClicked || this.props.scrollByThumb && this.isThumb(target);
        this.thumbScrolling = currentThumbScrolling;
        this.crossThumbScrolling = !currentThumbScrolling && currentCrossThumbScrolling;
        if (currentThumbScrolling) {
            this.expand()
        }
    };
    _proto.moveToMouseLocation = function(e) {
        var mouseLocation = e["page".concat(this.axis.toUpperCase())] - this.props.scrollableOffset;
        var location = this.scrollLocation + mouseLocation / this.containerToContentRatio() - this.props.containerSize / 2;
        this.scrollStep(-Math.round(location))
    };
    _proto.suppressBounce = function() {
        if (this.props.bounceEnabled || this.inBounds()) {
            return
        }
        this.setLocation(this.boundLocation())
    };
    _proto.release = function() {
        this.stateReleased();
        this.scrollComplete()
    };
    _proto.stateReleased = function() {
        var _this$props$onRelease, _this$props7;
        this.setPocketState(_consts.TopPocketState.STATE_RELEASED);
        null === (_this$props$onRelease = (_this$props7 = this.props).onRelease) || void 0 === _this$props$onRelease ? void 0 : _this$props$onRelease.call(_this$props7)
    };
    _proto.setPocketState = function(state) {
        var _this$props$pocketSta, _this$props8;
        null === (_this$props$pocketSta = (_this$props8 = this.props).pocketStateChange) || void 0 === _this$props$pocketSta ? void 0 : _this$props$pocketSta.call(_this$props8, state)
    };
    _proto.isPullDown = function() {
        return this.props.pullDownEnabled && this.props.bounceEnabled && this.boundaryOffset >= 0
    };
    _proto.isReachBottom = function() {
        return this.props.reachBottomEnabled && this.scrollLocation - this.minOffset - this.bottomPocketSize <= .5
    };
    _proto.containerToContentRatio = function() {
        return this.contentSize ? this.props.containerSize / this.contentSize : this.props.containerSize
    };
    _proto.baseContainerToContentRatio = function() {
        var _this$props9 = this.props,
            baseContainerSize = _this$props9.baseContainerSize,
            baseContentSize = _this$props9.baseContentSize;
        return baseContentSize ? baseContainerSize / baseContentSize : baseContainerSize
    };
    _proto.expand = function() {
        this.set_expanded((function() {
            return true
        }))
    };
    _proto.collapse = function() {
        this.set_expanded((function() {
            return false
        }))
    };
    _proto.onHoverStart = function() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_hovered((function() {
                return true
            }))
        }
    };
    _proto.onHoverEnd = function() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_hovered((function() {
                return false
            }))
        }
    };
    _proto.isThumb = function(element) {
        var _this$scrollbarRef$cu, _this$scrollbarRef$cu2;
        return (null === (_this$scrollbarRef$cu = this.scrollbarRef.current) || void 0 === _this$scrollbarRef$cu ? void 0 : _this$scrollbarRef$cu.querySelector(".".concat(_consts.SCROLLABLE_SCROLL_CLASS))) === element || (null === (_this$scrollbarRef$cu2 = this.scrollbarRef.current) || void 0 === _this$scrollbarRef$cu2 ? void 0 : _this$scrollbarRef$cu2.querySelector(".".concat(_consts.SCROLLABLE_SCROLL_CONTENT_CLASS))) === element
    };
    _proto.isScrollbar = function(element) {
        return element === this.scrollbarRef.current
    };
    _proto.validateEvent = function(e) {
        var target = e.originalEvent.target;
        return this.isThumb(target) || this.isScrollbar(target)
    };
    _proto.reachedMin = function() {
        return this.getLocation() <= this.minOffset
    };
    _proto.reachedMax = function() {
        return this.getLocation() >= this.maxOffset
    };
    _proto.getLocation = function() {
        return this.location
    };
    _proto.getScrollLocation = function() {
        return this.scrollLocation
    };
    _proto.setLocation = function(value) {
        this.location = value
    };
    _proto.inBounds = function() {
        return this.boundLocation() === this.getLocation()
    };
    _proto.boundLocation = function(value) {
        var currentLocation = (0, _type.isDefined)(value) ? value : this.getLocation();
        return Math.max(Math.min(currentLocation, this.maxOffset), this.minOffset)
    };
    _proto.getMinOffset = function() {
        return this.minOffset
    };
    _proto.getMaxOffset = function() {
        return this.maxOffset
    };
    _proto.initHandler = function(e, crossThumbScrolling) {
        this.stopScrolling();
        this.prepareThumbScrolling(e, crossThumbScrolling)
    };
    _proto.startHandler = function() {
        this.set_visibility((function() {
            return true
        }))
    };
    _proto.moveHandler = function(delta) {
        if (this.crossThumbScrolling) {
            return
        }
        var distance = delta;
        if (this.thumbScrolling) {
            distance[this.axis] = -Math.round(distance[this.axis] / this.containerToContentRatio())
        }
        this.scrollBy(distance)
    };
    _proto.endHandler = function(velocity) {
        var _this$props$onAnimato3, _this$props10;
        null === (_this$props$onAnimato3 = (_this$props10 = this.props).onAnimatorStart) || void 0 === _this$props$onAnimato3 ? void 0 : _this$props$onAnimato3.call(_this$props10, "inertia", velocity[this.axis], this.thumbScrolling, this.crossThumbScrolling);
        this.resetThumbScrolling()
    };
    _proto.stopHandler = function() {
        this.hide();
        if (this.thumbScrolling) {
            this.scrollComplete()
        } else {
            this.scrollToBounds()
        }
        this.resetThumbScrolling()
    };
    _proto.scrollByHandler = function(delta) {
        this.scrollBy(delta);
        this.scrollComplete()
    };
    _proto.scrollComplete = function() {
        if (this.props.forceGeneratePockets) {
            if (this.inBounds()) {
                if (this.__state_pocketState === _consts.TopPocketState.STATE_READY) {
                    this.pullDownRefreshing();
                    return
                }
                if (this.__state_pocketState === _consts.TopPocketState.STATE_LOADING) {
                    this.reachBottomLoading();
                    return
                }
            }
        }
        this.scrollToBounds()
    };
    _proto.stopComplete = function() {};
    _proto.scrollStep = function(delta) {
        this.setLocation(this.scrollLocation + delta);
        this.suppressBounce();
        this.moveScrollbar()
    };
    _proto.moveScrollbar = function(location) {
        var currentLocation = (0, _type.isDefined)(location) ? location * this.props.scaleRatio : this.getLocation();
        this.setLocation(currentLocation);
        this.set_scrollLocation((function() {
            return currentLocation
        }));
        if (this.props.forceGeneratePockets) {
            if (this.isPullDown()) {
                if (this.__state_pocketState !== _consts.TopPocketState.STATE_READY) {
                    this.setPocketState(_consts.TopPocketState.STATE_READY)
                }
            } else if (this.isReachBottom()) {
                if (this.__state_pocketState !== _consts.TopPocketState.STATE_LOADING) {
                    this.setPocketState(_consts.TopPocketState.STATE_LOADING)
                }
            } else if (this.__state_pocketState !== _consts.TopPocketState.STATE_RELEASED) {
                this.stateReleased()
            }
        }
    };
    _proto.releaseHandler = function() {
        this.release()
    };
    _proto.render = function() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props, {
                pocketState: this.__state_pocketState,
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            showOnScrollByWheel: this.showOnScrollByWheel,
            hovered: this.hovered,
            expanded: this.expanded,
            visibility: this.visibility,
            scrollLocation: this.scrollLocation,
            boundaryOffset: this.boundaryOffset,
            maxOffset: this.maxOffset,
            minOffset: this.minOffset,
            minLimit: this.minLimit,
            scrollbarRef: this.scrollbarRef,
            scrollRef: this.scrollRef,
            scrollTranslateOffset: this.scrollTranslateOffset,
            scrollPosition: this.scrollPosition,
            bottomBoundaryOffset: this.bottomBoundaryOffset,
            axis: this.axis,
            scrollProp: this.scrollProp,
            fullScrollProp: this.fullScrollProp,
            hide: this.hide,
            pullDownRefreshing: this.pullDownRefreshing,
            reachBottomLoading: this.reachBottomLoading,
            executePullDown: this.executePullDown,
            executeReachBottom: this.executeReachBottom,
            scrollToBounds: this.scrollToBounds,
            resetThumbScrolling: this.resetThumbScrolling,
            scrollBy: this.scrollBy,
            stopScrolling: this.stopScrolling,
            prepareThumbScrolling: this.prepareThumbScrolling,
            moveToMouseLocation: this.moveToMouseLocation,
            suppressBounce: this.suppressBounce,
            release: this.release,
            stateReleased: this.stateReleased,
            setPocketState: this.setPocketState,
            isPullDown: this.isPullDown,
            isReachBottom: this.isReachBottom,
            thumbSize: this.thumbSize,
            thumbRatio: this.thumbRatio,
            contentSize: this.contentSize,
            containerToContentRatio: this.containerToContentRatio,
            baseContainerToContentRatio: this.baseContainerToContentRatio,
            dimension: this.dimension,
            expand: this.expand,
            collapse: this.collapse,
            onHoverStart: this.onHoverStart,
            onHoverEnd: this.onHoverEnd,
            topPocketSize: this.topPocketSize,
            bottomPocketSize: this.bottomPocketSize,
            cssClasses: this.cssClasses,
            scrollStyles: this.scrollStyles,
            scrollClasses: this.scrollClasses,
            isVisible: this.isVisible,
            visible: this.visible,
            hoverStateEnabled: this.hoverStateEnabled,
            restAttributes: this.restAttributes
        })
    };
    _createClass(Scrollbar, [{
        key: "showOnScrollByWheel",
        get: function() {
            var state = this._currentState || this.state;
            return state.showOnScrollByWheel
        }
    }, {
        key: "hovered",
        get: function() {
            var state = this._currentState || this.state;
            return state.hovered
        }
    }, {
        key: "expanded",
        get: function() {
            var state = this._currentState || this.state;
            return state.expanded
        }
    }, {
        key: "visibility",
        get: function() {
            var state = this._currentState || this.state;
            return state.visibility
        }
    }, {
        key: "scrollLocation",
        get: function() {
            var state = this._currentState || this.state;
            return state.scrollLocation
        }
    }, {
        key: "boundaryOffset",
        get: function() {
            var state = this._currentState || this.state;
            return state.boundaryOffset
        }
    }, {
        key: "maxOffset",
        get: function() {
            var state = this._currentState || this.state;
            return state.maxOffset
        }
    }, {
        key: "minOffset",
        get: function() {
            var state = this._currentState || this.state;
            return state.minOffset
        }
    }, {
        key: "minLimit",
        get: function() {
            var state = this._currentState || this.state;
            return state.minLimit
        }
    }, {
        key: "__state_pocketState",
        get: function() {
            var state = this._currentState || this.state;
            return void 0 !== this.props.pocketState ? this.props.pocketState : state.pocketState
        }
    }, {
        key: "__state_contentTranslateOffset",
        get: function() {
            var state = this._currentState || this.state;
            return void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : state.contentTranslateOffset
        }
    }, {
        key: "scrollTranslateOffset",
        get: function() {
            return _extends({}, {
                left: 0,
                top: 0
            }, _defineProperty({}, this.scrollProp, this.scrollPosition))
        }
    }, {
        key: "scrollPosition",
        get: function() {
            return -this.scrollLocation * this.thumbRatio
        }
    }, {
        key: "bottomBoundaryOffset",
        get: function() {
            return this.contentSize - this.props.containerSize
        }
    }, {
        key: "axis",
        get: function() {
            return this.props.direction === _consts.DIRECTION_HORIZONTAL ? "x" : "y"
        }
    }, {
        key: "scrollProp",
        get: function() {
            return this.props.direction === _consts.DIRECTION_HORIZONTAL ? "left" : "top"
        }
    }, {
        key: "fullScrollProp",
        get: function() {
            return this.props.direction === _consts.DIRECTION_HORIZONTAL ? "scrollLeft" : "scrollTop"
        }
    }, {
        key: "thumbSize",
        get: function() {
            var _this$props11 = this.props,
                containerSize = _this$props11.containerSize,
                scaleRatio = _this$props11.scaleRatio;
            var size = Math.round(Math.max(Math.round(containerSize * this.containerToContentRatio()), THUMB_MIN_SIZE));
            return size / scaleRatio
        }
    }, {
        key: "thumbRatio",
        get: function() {
            var _this$props12 = this.props,
                containerSize = _this$props12.containerSize,
                scaleRatio = _this$props12.scaleRatio;
            if (this.contentSize) {
                return (containerSize - this.thumbSize) / (scaleRatio * (this.contentSize - containerSize))
            }
            return 1
        }
    }, {
        key: "contentSize",
        get: function() {
            if (this.props.contentSize) {
                return this.props.contentSize - this.bottomPocketSize - this.topPocketSize
            }
            return 0
        }
    }, {
        key: "dimension",
        get: function() {
            return this.props.direction === _consts.DIRECTION_HORIZONTAL ? "width" : "height"
        }
    }, {
        key: "topPocketSize",
        get: function() {
            if (this.props.pullDownEnabled) {
                return this.props.topPocketSize
            }
            return 0
        }
    }, {
        key: "bottomPocketSize",
        get: function() {
            if (this.props.reachBottomEnabled) {
                return this.props.bottomPocketSize
            }
            return 0
        }
    }, {
        key: "cssClasses",
        get: function() {
            var _classesMap;
            var direction = this.props.direction;
            var classesMap = (_classesMap = {}, _defineProperty(_classesMap, _consts.SCROLLABLE_SCROLLBAR_CLASS, true), _defineProperty(_classesMap, "dx-scrollbar-".concat(direction), true), _defineProperty(_classesMap, _consts.SCROLLABLE_SCROLLBAR_ACTIVE_CLASS, !!this.expanded), _defineProperty(_classesMap, _consts.HOVER_ENABLED_STATE, !!this.hoverStateEnabled), _classesMap);
            return (0, _combine_classes.combineClasses)(classesMap)
        }
    }, {
        key: "scrollStyles",
        get: function() {
            var _ref;
            return _ref = {}, _defineProperty(_ref, this.dimension, "".concat(this.thumbSize || THUMB_MIN_SIZE, "px")), _defineProperty(_ref, "transform", "never" === this.props.showScrollbar ? "none" : "translate(".concat(this.scrollTranslateOffset.left, "px, ").concat(this.scrollTranslateOffset.top, "px)")), _ref
        }
    }, {
        key: "scrollClasses",
        get: function() {
            var _combineClasses;
            return (0, _combine_classes.combineClasses)((_combineClasses = {}, _defineProperty(_combineClasses, _consts.SCROLLABLE_SCROLL_CLASS, true), _defineProperty(_combineClasses, "dx-state-invisible", !this.visible), _combineClasses))
        }
    }, {
        key: "isVisible",
        get: function() {
            return "never" !== this.props.showScrollbar && this.baseContainerToContentRatio() < 1
        }
    }, {
        key: "visible",
        get: function() {
            var _this$props13 = this.props,
                forceVisibility = _this$props13.forceVisibility,
                showScrollbar = _this$props13.showScrollbar;
            if (!this.isVisible) {
                return false
            }
            if ("onHover" === showScrollbar) {
                return this.visibility || this.props.isScrollableHovered || this.hovered
            }
            if ("always" === showScrollbar) {
                return true
            }
            return forceVisibility || this.visibility || !!this.showOnScrollByWheel
        }
    }, {
        key: "hoverStateEnabled",
        get: function() {
            var _this$props14 = this.props,
                scrollByThumb = _this$props14.scrollByThumb,
                showScrollbar = _this$props14.showScrollbar;
            return ("onHover" === showScrollbar || "always" === showScrollbar) && scrollByThumb
        }
    }, {
        key: "restAttributes",
        get: function() {
            var _this$props$pocketSta2 = _extends({}, this.props, {
                    pocketState: this.__state_pocketState,
                    contentTranslateOffset: this.__state_contentTranslateOffset
                }),
                restProps = (_this$props$pocketSta2.activeStateEnabled, _this$props$pocketSta2.baseContainerSize, _this$props$pocketSta2.baseContentSize, _this$props$pocketSta2.bottomPocketSize, _this$props$pocketSta2.bounceEnabled, _this$props$pocketSta2.children, _this$props$pocketSta2.classes, _this$props$pocketSta2.containerSize, _this$props$pocketSta2.contentPositionChange, _this$props$pocketSta2.contentSize, _this$props$pocketSta2.contentTranslateOffset, _this$props$pocketSta2.contentTranslateOffsetChange, _this$props$pocketSta2.defaultContentTranslateOffset, _this$props$pocketSta2.defaultPocketState, _this$props$pocketSta2.direction, _this$props$pocketSta2.expandable, _this$props$pocketSta2.forceGeneratePockets, _this$props$pocketSta2.forceVisibility, _this$props$pocketSta2.hoverStateEnabled, _this$props$pocketSta2.isScrollableHovered, _this$props$pocketSta2.needScrollViewContentWrapper, _this$props$pocketSta2.needScrollViewLoadPanel, _this$props$pocketSta2.onAnimatorCancel, _this$props$pocketSta2.onAnimatorStart, _this$props$pocketSta2.onPullDown, _this$props$pocketSta2.onReachBottom, _this$props$pocketSta2.onRelease, _this$props$pocketSta2.onScroll, _this$props$pocketSta2.onUpdated, _this$props$pocketSta2.pocketState, _this$props$pocketSta2.pocketStateChange, _this$props$pocketSta2.pullDownEnabled, _this$props$pocketSta2.reachBottomEnabled, _this$props$pocketSta2.scaleRatio, _this$props$pocketSta2.scrollByContent, _this$props$pocketSta2.scrollByThumb, _this$props$pocketSta2.scrollableOffset, _this$props$pocketSta2.showScrollbar, _this$props$pocketSta2.topPocketSize, _this$props$pocketSta2.useKeyboard, _objectWithoutProperties(_this$props$pocketSta2, ["activeStateEnabled", "baseContainerSize", "baseContentSize", "bottomPocketSize", "bounceEnabled", "children", "classes", "containerSize", "contentPositionChange", "contentSize", "contentTranslateOffset", "contentTranslateOffsetChange", "defaultContentTranslateOffset", "defaultPocketState", "direction", "expandable", "forceGeneratePockets", "forceVisibility", "hoverStateEnabled", "isScrollableHovered", "needScrollViewContentWrapper", "needScrollViewLoadPanel", "onAnimatorCancel", "onAnimatorStart", "onPullDown", "onReachBottom", "onRelease", "onScroll", "onUpdated", "pocketState", "pocketStateChange", "pullDownEnabled", "reachBottomEnabled", "scaleRatio", "scrollByContent", "scrollByThumb", "scrollableOffset", "showScrollbar", "topPocketSize", "useKeyboard"]));
            return restProps
        }
    }]);
    return Scrollbar
}(_vdom.InfernoComponent);
exports.Scrollbar = Scrollbar;
Scrollbar.defaultProps = _extends({}, ScrollbarPropsType);
