/**
 * DevExtreme (esm/exporter/jspdf/export_data_grid_2.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    isDefined
} from "../../core/utils/type";
import {
    PdfGrid
} from "./pdf_grid";

function exportDataGrid(doc, dataGrid, options) {
    if (!isDefined(options.rect)) {
        throw "options.rect is required"
    }
    var dataProvider = dataGrid.getDataProvider();
    return new Promise(resolve => {
        dataProvider.ready().done(() => {
            var columns = dataProvider.getColumns();
            var pdfGrid = new PdfGrid(options.splitToTablesByColumns);
            pdfGrid.startNewTable(options.drawTableBorder, options.rect);
            var dataRowsCount = dataProvider.getRowsCount();
            for (var rowIndex = 0; rowIndex < dataRowsCount; rowIndex++) {
                var currentRow = [];
                for (var cellIndex = 0; cellIndex < columns.length; cellIndex++) {
                    var cellData = dataProvider.getCellData(rowIndex, cellIndex, true);
                    var pdfCell = {
                        text: cellData.value
                    };
                    if (options.onCellExporting) {
                        options.onCellExporting({
                            gridCell: {
                                value: cellData.value
                            },
                            pdfCell: pdfCell
                        })
                    }
                    currentRow.push(pdfCell)
                }
                if (options.onRowExporting) {
                    ! function() {
                        var args = {
                            drawNewTableFromThisRow: {},
                            rowCells: currentRow
                        };
                        options.onRowExporting(args);
                        var {
                            startNewTable: startNewTable,
                            addPage: addPage,
                            tableRect: tableRect,
                            splitToTablesByColumns: splitToTablesByColumns
                        } = args.drawNewTableFromThisRow;
                        if (true === startNewTable) {
                            pdfGrid.startNewTable(options.drawTableBorder, tableRect, true === addPage, splitToTablesByColumns)
                        }
                        if (isDefined(args.rowHeight)) {
                            currentRow.forEach(cell => {
                                if (!cell.skip && !cell.rect.h) {
                                    cell.rect.h = args.rowHeight
                                }
                            })
                        }
                    }()
                }
                pdfGrid.addRow(currentRow)
            }
            pdfGrid.drawTo(doc);
            resolve()
        })
    })
}
export {
    exportDataGrid
};
