/**
 * DevExtreme (esm/exporter/jspdf/pdf_grid.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    isDefined
} from "../../core/utils/type";
import {
    PdfTable
} from "./pdf_table";
export class PdfGrid {
    constructor(splitByColumns) {
        this._splitByColumns = null !== splitByColumns && void 0 !== splitByColumns ? splitByColumns : [];
        this._newPageTables = [];
        this._tables = [];
        this._currentHorizontalTables = null
    }
    _addLastTableToNewPages() {
        this._newPageTables.push(this._currentHorizontalTables[this._currentHorizontalTables.length - 1])
    }
    startNewTable(drawTableBorder, firstTableRect, firstTableOnNewPage, splitByColumns) {
        this._currentHorizontalTables = [new PdfTable(drawTableBorder, firstTableRect)];
        if (firstTableOnNewPage) {
            this._addLastTableToNewPages()
        }
        if (isDefined(splitByColumns)) {
            this._splitByColumns = splitByColumns
        }
        if (isDefined(this._splitByColumns)) {
            this._splitByColumns.forEach(splitByColumn => {
                this._currentHorizontalTables.push(new PdfTable(drawTableBorder, splitByColumn.tableRect));
                if (splitByColumn.drawOnNewPage) {
                    this._addLastTableToNewPages()
                }
            })
        }
        this._tables.push(...this._currentHorizontalTables)
    }
    addRow(cells) {
        var _this = this;
        var currentTableIndex = 0;
        var currentTableCells = [];
        var _loop = function(cellIndex) {
            var isNewTableColumn = _this._splitByColumns.filter(splitByColumn => splitByColumn.columnIndex === cellIndex)[0];
            if (isNewTableColumn) {
                _this._currentHorizontalTables[currentTableIndex].addRow(currentTableCells);
                currentTableIndex++;
                currentTableCells = []
            }
            currentTableCells.push(cells[cellIndex])
        };
        for (var cellIndex = 0; cellIndex < cells.length; cellIndex++) {
            _loop(cellIndex)
        }
        this._currentHorizontalTables[currentTableIndex].addRow(currentTableCells)
    }
    drawTo(doc) {
        this._tables.forEach(table => {
            if (-1 !== this._newPageTables.indexOf(table)) {
                doc.addPage()
            }
            table.drawTo(doc)
        })
    }
}
