/**
 * DevExtreme (esm/renovation/ui/grids/data_grid/data_grid.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import {
    createComponentVNode,
    normalizeProps
} from "inferno";
import {
    InfernoEffect,
    InfernoComponent
} from "@devextreme/vdom";
import {
    DataGridProps
} from "./common/data_grid_props";
import "../../../../ui/data_grid/ui.data_grid";
import {
    Widget
} from "../../common/widget";
import {
    DataGridComponent
} from "./datagrid_component";
import {
    DataGridViews
} from "./data_grid_views";
import {
    getUpdatedOptions
} from "./utils/get_updated_options";
var aria = {
    role: "presentation"
};
export var viewFunction = _ref => {
    var {
        instance: instance,
        props: {
            accessKey: accessKey,
            activeStateEnabled: activeStateEnabled,
            disabled: disabled,
            focusStateEnabled: focusStateEnabled,
            height: height,
            hint: hint,
            hoverStateEnabled: hoverStateEnabled,
            onContentReady: onContentReady,
            rtlEnabled: rtlEnabled,
            tabIndex: tabIndex,
            visible: visible,
            width: width
        },
        restAttributes: restAttributes
    } = _ref;
    return normalizeProps(createComponentVNode(2, Widget, _extends({
        accessKey: accessKey,
        activeStateEnabled: activeStateEnabled,
        aria: aria,
        disabled: disabled,
        focusStateEnabled: focusStateEnabled,
        height: height,
        hint: hint,
        hoverStateEnabled: hoverStateEnabled,
        onContentReady: onContentReady,
        rtlEnabled: rtlEnabled,
        tabIndex: tabIndex,
        visible: visible,
        width: width
    }, restAttributes, {
        children: createComponentVNode(2, DataGridViews, {
            instance: instance
        })
    })))
};
var getTemplate = TemplateProp => TemplateProp && (TemplateProp.defaultProps ? props => normalizeProps(createComponentVNode(2, TemplateProp, _extends({}, props))) : TemplateProp);
export class DataGrid extends InfernoComponent {
    constructor(props) {
        super(props);
        this._currentState = null;
        this.state = {
            filterValue: void 0 !== this.props.filterValue ? this.props.filterValue : this.props.defaultFilterValue,
            focusedColumnIndex: void 0 !== this.props.focusedColumnIndex ? this.props.focusedColumnIndex : this.props.defaultFocusedColumnIndex,
            focusedRowIndex: void 0 !== this.props.focusedRowIndex ? this.props.focusedRowIndex : this.props.defaultFocusedRowIndex,
            focusedRowKey: void 0 !== this.props.focusedRowKey ? this.props.focusedRowKey : this.props.defaultFocusedRowKey,
            selectedRowKeys: void 0 !== this.props.selectedRowKeys ? this.props.selectedRowKeys : this.props.defaultSelectedRowKeys,
            selectionFilter: void 0 !== this.props.selectionFilter ? this.props.selectionFilter : this.props.defaultSelectionFilter
        };
        this.getComponentInstance = this.getComponentInstance.bind(this);
        this.beginCustomLoading = this.beginCustomLoading.bind(this);
        this.byKey = this.byKey.bind(this);
        this.cancelEditData = this.cancelEditData.bind(this);
        this.cellValue = this.cellValue.bind(this);
        this.clearFilter = this.clearFilter.bind(this);
        this.clearSelection = this.clearSelection.bind(this);
        this.clearSorting = this.clearSorting.bind(this);
        this.closeEditCell = this.closeEditCell.bind(this);
        this.collapseAdaptiveDetailRow = this.collapseAdaptiveDetailRow.bind(this);
        this.columnCount = this.columnCount.bind(this);
        this.columnOption = this.columnOption.bind(this);
        this.deleteColumn = this.deleteColumn.bind(this);
        this.deleteRow = this.deleteRow.bind(this);
        this.deselectAll = this.deselectAll.bind(this);
        this.deselectRows = this.deselectRows.bind(this);
        this.editCell = this.editCell.bind(this);
        this.editRow = this.editRow.bind(this);
        this.endCustomLoading = this.endCustomLoading.bind(this);
        this.expandAdaptiveDetailRow = this.expandAdaptiveDetailRow.bind(this);
        this.filter = this.filter.bind(this);
        this.focus = this.focus.bind(this);
        this.getCellElement = this.getCellElement.bind(this);
        this.getCombinedFilter = this.getCombinedFilter.bind(this);
        this.getDataSource = this.getDataSource.bind(this);
        this.getKeyByRowIndex = this.getKeyByRowIndex.bind(this);
        this.getRowElement = this.getRowElement.bind(this);
        this.getRowIndexByKey = this.getRowIndexByKey.bind(this);
        this.getScrollable = this.getScrollable.bind(this);
        this.getVisibleColumnIndex = this.getVisibleColumnIndex.bind(this);
        this.hasEditData = this.hasEditData.bind(this);
        this.hideColumnChooser = this.hideColumnChooser.bind(this);
        this.isAdaptiveDetailRowExpanded = this.isAdaptiveDetailRowExpanded.bind(this);
        this.isRowFocused = this.isRowFocused.bind(this);
        this.isRowSelected = this.isRowSelected.bind(this);
        this.keyOf = this.keyOf.bind(this);
        this.navigateToRow = this.navigateToRow.bind(this);
        this.pageCount = this.pageCount.bind(this);
        this.pageIndex = this.pageIndex.bind(this);
        this.pageSize = this.pageSize.bind(this);
        this.refresh = this.refresh.bind(this);
        this.repaintRows = this.repaintRows.bind(this);
        this.saveEditData = this.saveEditData.bind(this);
        this.searchByText = this.searchByText.bind(this);
        this.selectAll = this.selectAll.bind(this);
        this.selectRows = this.selectRows.bind(this);
        this.selectRowsByIndexes = this.selectRowsByIndexes.bind(this);
        this.showColumnChooser = this.showColumnChooser.bind(this);
        this.undeleteRow = this.undeleteRow.bind(this);
        this.updateDimensions = this.updateDimensions.bind(this);
        this.addColumn = this.addColumn.bind(this);
        this.addRow = this.addRow.bind(this);
        this.clearGrouping = this.clearGrouping.bind(this);
        this.collapseAll = this.collapseAll.bind(this);
        this.collapseRow = this.collapseRow.bind(this);
        this.expandAll = this.expandAll.bind(this);
        this.expandRow = this.expandRow.bind(this);
        this.exportToExcel = this.exportToExcel.bind(this);
        this.getSelectedRowKeys = this.getSelectedRowKeys.bind(this);
        this.getSelectedRowsData = this.getSelectedRowsData.bind(this);
        this.getTotalSummaryValue = this.getTotalSummaryValue.bind(this);
        this.getVisibleColumns = this.getVisibleColumns.bind(this);
        this.getVisibleRows = this.getVisibleRows.bind(this);
        this.isRowExpanded = this.isRowExpanded.bind(this);
        this.totalCount = this.totalCount.bind(this);
        this.getController = this.getController.bind(this);
        this.updateOptions = this.updateOptions.bind(this);
        this.dispose = this.dispose.bind(this);
        this.normalizeProps = this.normalizeProps.bind(this);
        this.createInstance = this.createInstance.bind(this)
    }
    createEffects() {
        return [new InfernoEffect(this.updateOptions, [this.props, this.__state_filterValue, this.__state_focusedColumnIndex, this.__state_focusedRowIndex, this.__state_focusedRowKey, this.__state_selectedRowKeys, this.__state_selectionFilter]), new InfernoEffect(this.dispose, [])]
    }
    updateEffects() {
        var _this$_effects$;
        null === (_this$_effects$ = this._effects[0]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([this.props, this.__state_filterValue, this.__state_focusedColumnIndex, this.__state_focusedRowIndex, this.__state_focusedRowKey, this.__state_selectedRowKeys, this.__state_selectionFilter])
    }
    get __state_filterValue() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.filterValue ? this.props.filterValue : state.filterValue
    }
    set_filterValue(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this.props.filterValueChange(newValue);
            this._currentState = null;
            return {
                filterValue: newValue
            }
        })
    }
    get __state_focusedColumnIndex() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.focusedColumnIndex ? this.props.focusedColumnIndex : state.focusedColumnIndex
    }
    set_focusedColumnIndex(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this.props.focusedColumnIndexChange(newValue);
            this._currentState = null;
            return {
                focusedColumnIndex: newValue
            }
        })
    }
    get __state_focusedRowIndex() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.focusedRowIndex ? this.props.focusedRowIndex : state.focusedRowIndex
    }
    set_focusedRowIndex(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this.props.focusedRowIndexChange(newValue);
            this._currentState = null;
            return {
                focusedRowIndex: newValue
            }
        })
    }
    get __state_focusedRowKey() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.focusedRowKey ? this.props.focusedRowKey : state.focusedRowKey
    }
    set_focusedRowKey(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this.props.focusedRowKeyChange(newValue);
            this._currentState = null;
            return {
                focusedRowKey: newValue
            }
        })
    }
    get __state_selectedRowKeys() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.selectedRowKeys ? this.props.selectedRowKeys : state.selectedRowKeys
    }
    set_selectedRowKeys(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this.props.selectedRowKeysChange(newValue);
            this._currentState = null;
            return {
                selectedRowKeys: newValue
            }
        })
    }
    get __state_selectionFilter() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.selectionFilter ? this.props.selectionFilter : state.selectionFilter
    }
    set_selectionFilter(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this.props.selectionFilterChange(newValue);
            this._currentState = null;
            return {
                selectionFilter: newValue
            }
        })
    }
    updateOptions() {
        if (this.instance && this.prevProps) {
            var updatedOptions = getUpdatedOptions(this.prevProps, _extends({}, this.props, {
                filterValue: this.__state_filterValue,
                focusedColumnIndex: this.__state_focusedColumnIndex,
                focusedRowIndex: this.__state_focusedRowIndex,
                focusedRowKey: this.__state_focusedRowKey,
                selectedRowKeys: this.__state_selectedRowKeys,
                selectionFilter: this.__state_selectionFilter
            }));
            this.instance.beginUpdate();
            updatedOptions.forEach(_ref2 => {
                var {
                    path: path,
                    value: value
                } = _ref2;
                return this.instance.option(path, value)
            });
            this.instance.endUpdate()
        }
        this.prevProps = _extends({}, this.props, {
            filterValue: this.__state_filterValue,
            focusedColumnIndex: this.__state_focusedColumnIndex,
            focusedRowIndex: this.__state_focusedRowIndex,
            focusedRowKey: this.__state_focusedRowKey,
            selectedRowKeys: this.__state_selectedRowKeys,
            selectionFilter: this.__state_selectionFilter
        })
    }
    dispose() {
        return () => {
            this.instance.dispose()
        }
    }
    get instance() {
        if (!this.componentInstance) {
            this.componentInstance = this.createInstance()
        }
        return this.componentInstance
    }
    normalizeProps() {
        var result = {};
        Object.keys(_extends({}, this.props, {
            filterValue: this.__state_filterValue,
            focusedColumnIndex: this.__state_focusedColumnIndex,
            focusedRowIndex: this.__state_focusedRowIndex,
            focusedRowKey: this.__state_focusedRowKey,
            selectedRowKeys: this.__state_selectedRowKeys,
            selectionFilter: this.__state_selectionFilter
        })).forEach(key => {
            if (void 0 !== _extends({}, this.props, {
                    filterValue: this.__state_filterValue,
                    focusedColumnIndex: this.__state_focusedColumnIndex,
                    focusedRowIndex: this.__state_focusedRowIndex,
                    focusedRowKey: this.__state_focusedRowKey,
                    selectedRowKeys: this.__state_selectedRowKeys,
                    selectionFilter: this.__state_selectionFilter
                })[key]) {
                result[key] = _extends({}, this.props, {
                    filterValue: this.__state_filterValue,
                    focusedColumnIndex: this.__state_focusedColumnIndex,
                    focusedRowIndex: this.__state_focusedRowIndex,
                    focusedRowKey: this.__state_focusedRowKey,
                    selectedRowKeys: this.__state_selectedRowKeys,
                    selectionFilter: this.__state_selectionFilter
                })[key]
            }
        });
        return result
    }
    createInstance() {
        var instance = new DataGridComponent(this.normalizeProps());
        return instance
    }
    get restAttributes() {
        var _this$props$filterVal = _extends({}, this.props, {
                filterValue: this.__state_filterValue,
                focusedColumnIndex: this.__state_focusedColumnIndex,
                focusedRowIndex: this.__state_focusedRowIndex,
                focusedRowKey: this.__state_focusedRowKey,
                selectedRowKeys: this.__state_selectedRowKeys,
                selectionFilter: this.__state_selectionFilter
            }),
            restProps = _objectWithoutPropertiesLoose(_this$props$filterVal, ["accessKey", "activeStateEnabled", "allowColumnReordering", "allowColumnResizing", "autoNavigateToFocusedRow", "cacheEnabled", "cellHintEnabled", "columnAutoWidth", "columnChooser", "columnFixing", "columnHidingEnabled", "columnMinWidth", "columnResizingMode", "columnWidth", "columns", "customizeColumns", "customizeExportData", "dataSource", "dateSerializationFormat", "defaultFilterValue", "defaultFocusedColumnIndex", "defaultFocusedRowIndex", "defaultFocusedRowKey", "defaultSelectedRowKeys", "defaultSelectionFilter", "disabled", "editing", "errorRowEnabled", "export", "filterBuilder", "filterBuilderPopup", "filterPanel", "filterRow", "filterSyncEnabled", "filterValue", "filterValueChange", "focusStateEnabled", "focusedColumnIndex", "focusedColumnIndexChange", "focusedRowEnabled", "focusedRowIndex", "focusedRowIndexChange", "focusedRowKey", "focusedRowKeyChange", "groupPanel", "grouping", "headerFilter", "height", "highlightChanges", "hint", "hoverStateEnabled", "keyExpr", "keyboardNavigation", "loadPanel", "masterDetail", "noDataText", "onAdaptiveDetailRowPreparing", "onCellClick", "onCellDblClick", "onCellHoverChanged", "onCellPrepared", "onClick", "onContentReady", "onContextMenuPreparing", "onDataErrorOccurred", "onEditingStart", "onEditorPrepared", "onEditorPreparing", "onExported", "onExporting", "onFileSaving", "onFocusedCellChanged", "onFocusedCellChanging", "onFocusedRowChanged", "onFocusedRowChanging", "onInitNewRow", "onKeyDown", "onRowClick", "onRowCollapsed", "onRowCollapsing", "onRowDblClick", "onRowExpanded", "onRowExpanding", "onRowInserted", "onRowInserting", "onRowPrepared", "onRowRemoved", "onRowRemoving", "onRowUpdated", "onRowUpdating", "onRowValidating", "onSelectionChanged", "onToolbarPreparing", "pager", "paging", "remoteOperations", "renderAsync", "repaintChangesOnly", "rowAlternationEnabled", "rowDragging", "rowTemplate", "rtlEnabled", "scrolling", "searchPanel", "selectedRowKeys", "selectedRowKeysChange", "selection", "selectionFilter", "selectionFilterChange", "showBorders", "showColumnHeaders", "showColumnLines", "showRowLines", "sortByGroupSummaryInfo", "sorting", "stateStoring", "summary", "tabIndex", "twoWayBindingEnabled", "visible", "width", "wordWrapEnabled"]);
        return restProps
    }
    getComponentInstance() {
        return this.instance
    }
    beginCustomLoading(messageText) {
        var _this$instance;
        return null === (_this$instance = this.instance) || void 0 === _this$instance ? void 0 : _this$instance.beginCustomLoading(messageText)
    }
    byKey(key) {
        var _this$instance2;
        return null === (_this$instance2 = this.instance) || void 0 === _this$instance2 ? void 0 : _this$instance2.byKey(key)
    }
    cancelEditData() {
        var _this$instance3;
        return null === (_this$instance3 = this.instance) || void 0 === _this$instance3 ? void 0 : _this$instance3.cancelEditData()
    }
    cellValue(rowIndex, dataField, value) {
        var _this$instance4;
        return null === (_this$instance4 = this.instance) || void 0 === _this$instance4 ? void 0 : _this$instance4.cellValue(rowIndex, dataField, value)
    }
    clearFilter(filterName) {
        var _this$instance5;
        return null === (_this$instance5 = this.instance) || void 0 === _this$instance5 ? void 0 : _this$instance5.clearFilter(filterName)
    }
    clearSelection() {
        var _this$instance6;
        return null === (_this$instance6 = this.instance) || void 0 === _this$instance6 ? void 0 : _this$instance6.clearSelection()
    }
    clearSorting() {
        var _this$instance7;
        return null === (_this$instance7 = this.instance) || void 0 === _this$instance7 ? void 0 : _this$instance7.clearSorting()
    }
    closeEditCell() {
        var _this$instance8;
        return null === (_this$instance8 = this.instance) || void 0 === _this$instance8 ? void 0 : _this$instance8.closeEditCell()
    }
    collapseAdaptiveDetailRow() {
        var _this$instance9;
        return null === (_this$instance9 = this.instance) || void 0 === _this$instance9 ? void 0 : _this$instance9.collapseAdaptiveDetailRow()
    }
    columnCount() {
        var _this$instance10;
        return null === (_this$instance10 = this.instance) || void 0 === _this$instance10 ? void 0 : _this$instance10.columnCount()
    }
    columnOption(id, optionName, optionValue) {
        var _this$instance11;
        return null === (_this$instance11 = this.instance) || void 0 === _this$instance11 ? void 0 : _this$instance11.columnOption(id, optionName, optionValue)
    }
    deleteColumn(id) {
        var _this$instance12;
        return null === (_this$instance12 = this.instance) || void 0 === _this$instance12 ? void 0 : _this$instance12.deleteColumn(id)
    }
    deleteRow(rowIndex) {
        var _this$instance13;
        return null === (_this$instance13 = this.instance) || void 0 === _this$instance13 ? void 0 : _this$instance13.deleteRow(rowIndex)
    }
    deselectAll() {
        var _this$instance14;
        return null === (_this$instance14 = this.instance) || void 0 === _this$instance14 ? void 0 : _this$instance14.deselectAll()
    }
    deselectRows(keys) {
        var _this$instance15;
        return null === (_this$instance15 = this.instance) || void 0 === _this$instance15 ? void 0 : _this$instance15.deselectRows(keys)
    }
    editCell(rowIndex, dataFieldColumnIndex) {
        var _this$instance16;
        return null === (_this$instance16 = this.instance) || void 0 === _this$instance16 ? void 0 : _this$instance16.editCell(rowIndex, dataFieldColumnIndex)
    }
    editRow(rowIndex) {
        var _this$instance17;
        return null === (_this$instance17 = this.instance) || void 0 === _this$instance17 ? void 0 : _this$instance17.editRow(rowIndex)
    }
    endCustomLoading() {
        var _this$instance18;
        return null === (_this$instance18 = this.instance) || void 0 === _this$instance18 ? void 0 : _this$instance18.endCustomLoading()
    }
    expandAdaptiveDetailRow(key) {
        var _this$instance19;
        return null === (_this$instance19 = this.instance) || void 0 === _this$instance19 ? void 0 : _this$instance19.expandAdaptiveDetailRow(key)
    }
    filter(filterExpr) {
        var _this$instance20;
        return null === (_this$instance20 = this.instance) || void 0 === _this$instance20 ? void 0 : _this$instance20.filter(filterExpr)
    }
    focus(element) {
        var _this$instance21;
        return null === (_this$instance21 = this.instance) || void 0 === _this$instance21 ? void 0 : _this$instance21.focus(element)
    }
    getCellElement(rowIndex, dataField) {
        var _this$instance22;
        return null === (_this$instance22 = this.instance) || void 0 === _this$instance22 ? void 0 : _this$instance22.getCellElement(rowIndex, dataField)
    }
    getCombinedFilter(returnDataField) {
        var _this$instance23;
        return null === (_this$instance23 = this.instance) || void 0 === _this$instance23 ? void 0 : _this$instance23.getCombinedFilter(returnDataField)
    }
    getDataSource() {
        var _this$instance24;
        return null === (_this$instance24 = this.instance) || void 0 === _this$instance24 ? void 0 : _this$instance24.getDataSource()
    }
    getKeyByRowIndex(rowIndex) {
        var _this$instance25;
        return null === (_this$instance25 = this.instance) || void 0 === _this$instance25 ? void 0 : _this$instance25.getKeyByRowIndex(rowIndex)
    }
    getRowElement(rowIndex) {
        var _this$instance26;
        return null === (_this$instance26 = this.instance) || void 0 === _this$instance26 ? void 0 : _this$instance26.getRowElement(rowIndex)
    }
    getRowIndexByKey(key) {
        var _this$instance27;
        return null === (_this$instance27 = this.instance) || void 0 === _this$instance27 ? void 0 : _this$instance27.getRowIndexByKey(key)
    }
    getScrollable() {
        var _this$instance28;
        return null === (_this$instance28 = this.instance) || void 0 === _this$instance28 ? void 0 : _this$instance28.getScrollable()
    }
    getVisibleColumnIndex(id) {
        var _this$instance29;
        return null === (_this$instance29 = this.instance) || void 0 === _this$instance29 ? void 0 : _this$instance29.getVisibleColumnIndex(id)
    }
    hasEditData() {
        var _this$instance30;
        return null === (_this$instance30 = this.instance) || void 0 === _this$instance30 ? void 0 : _this$instance30.hasEditData()
    }
    hideColumnChooser() {
        var _this$instance31;
        return null === (_this$instance31 = this.instance) || void 0 === _this$instance31 ? void 0 : _this$instance31.hideColumnChooser()
    }
    isAdaptiveDetailRowExpanded(key) {
        var _this$instance32;
        return null === (_this$instance32 = this.instance) || void 0 === _this$instance32 ? void 0 : _this$instance32.isAdaptiveDetailRowExpanded(key)
    }
    isRowFocused(key) {
        var _this$instance33;
        return null === (_this$instance33 = this.instance) || void 0 === _this$instance33 ? void 0 : _this$instance33.isRowFocused(key)
    }
    isRowSelected(key) {
        var _this$instance34;
        return null === (_this$instance34 = this.instance) || void 0 === _this$instance34 ? void 0 : _this$instance34.isRowSelected(key)
    }
    keyOf(obj) {
        var _this$instance35;
        return null === (_this$instance35 = this.instance) || void 0 === _this$instance35 ? void 0 : _this$instance35.keyOf(obj)
    }
    navigateToRow(key) {
        var _this$instance36;
        return null === (_this$instance36 = this.instance) || void 0 === _this$instance36 ? void 0 : _this$instance36.navigateToRow(key)
    }
    pageCount() {
        var _this$instance37;
        return null === (_this$instance37 = this.instance) || void 0 === _this$instance37 ? void 0 : _this$instance37.pageCount()
    }
    pageIndex(newIndex) {
        var _this$instance38;
        return null === (_this$instance38 = this.instance) || void 0 === _this$instance38 ? void 0 : _this$instance38.pageIndex(newIndex)
    }
    pageSize(value) {
        var _this$instance39;
        return null === (_this$instance39 = this.instance) || void 0 === _this$instance39 ? void 0 : _this$instance39.pageSize(value)
    }
    refresh(changesOnly) {
        var _this$instance40;
        return null === (_this$instance40 = this.instance) || void 0 === _this$instance40 ? void 0 : _this$instance40.refresh(changesOnly)
    }
    repaintRows(rowIndexes) {
        var _this$instance41;
        return null === (_this$instance41 = this.instance) || void 0 === _this$instance41 ? void 0 : _this$instance41.repaintRows(rowIndexes)
    }
    saveEditData() {
        var _this$instance42;
        return null === (_this$instance42 = this.instance) || void 0 === _this$instance42 ? void 0 : _this$instance42.saveEditData()
    }
    searchByText(text) {
        var _this$instance43;
        return null === (_this$instance43 = this.instance) || void 0 === _this$instance43 ? void 0 : _this$instance43.searchByText(text)
    }
    selectAll() {
        var _this$instance44;
        return null === (_this$instance44 = this.instance) || void 0 === _this$instance44 ? void 0 : _this$instance44.selectAll()
    }
    selectRows(keys, preserve) {
        var _this$instance45;
        return null === (_this$instance45 = this.instance) || void 0 === _this$instance45 ? void 0 : _this$instance45.selectRows(keys, preserve)
    }
    selectRowsByIndexes(indexes) {
        var _this$instance46;
        return null === (_this$instance46 = this.instance) || void 0 === _this$instance46 ? void 0 : _this$instance46.selectRowsByIndexes(indexes)
    }
    showColumnChooser() {
        var _this$instance47;
        return null === (_this$instance47 = this.instance) || void 0 === _this$instance47 ? void 0 : _this$instance47.showColumnChooser()
    }
    undeleteRow(rowIndex) {
        var _this$instance48;
        return null === (_this$instance48 = this.instance) || void 0 === _this$instance48 ? void 0 : _this$instance48.undeleteRow(rowIndex)
    }
    updateDimensions() {
        var _this$instance49;
        return null === (_this$instance49 = this.instance) || void 0 === _this$instance49 ? void 0 : _this$instance49.updateDimensions()
    }
    addColumn(columnOptions) {
        var _this$instance50;
        return null === (_this$instance50 = this.instance) || void 0 === _this$instance50 ? void 0 : _this$instance50.addColumn(columnOptions)
    }
    addRow() {
        var _this$instance51;
        return null === (_this$instance51 = this.instance) || void 0 === _this$instance51 ? void 0 : _this$instance51.addRow()
    }
    clearGrouping() {
        var _this$instance52;
        return null === (_this$instance52 = this.instance) || void 0 === _this$instance52 ? void 0 : _this$instance52.clearGrouping()
    }
    collapseAll(groupIndex) {
        var _this$instance53;
        return null === (_this$instance53 = this.instance) || void 0 === _this$instance53 ? void 0 : _this$instance53.collapseAll(groupIndex)
    }
    collapseRow(key) {
        var _this$instance54;
        return null === (_this$instance54 = this.instance) || void 0 === _this$instance54 ? void 0 : _this$instance54.collapseRow(key)
    }
    expandAll(groupIndex) {
        var _this$instance55;
        return null === (_this$instance55 = this.instance) || void 0 === _this$instance55 ? void 0 : _this$instance55.expandAll(groupIndex)
    }
    expandRow(key) {
        var _this$instance56;
        return null === (_this$instance56 = this.instance) || void 0 === _this$instance56 ? void 0 : _this$instance56.expandRow(key)
    }
    exportToExcel(selectionOnly) {
        var _this$instance57;
        return null === (_this$instance57 = this.instance) || void 0 === _this$instance57 ? void 0 : _this$instance57.exportToExcel(selectionOnly)
    }
    getSelectedRowKeys() {
        var _this$instance58;
        return null === (_this$instance58 = this.instance) || void 0 === _this$instance58 ? void 0 : _this$instance58.getSelectedRowKeys()
    }
    getSelectedRowsData() {
        var _this$instance59;
        return null === (_this$instance59 = this.instance) || void 0 === _this$instance59 ? void 0 : _this$instance59.getSelectedRowsData()
    }
    getTotalSummaryValue(summaryItemName) {
        var _this$instance60;
        return null === (_this$instance60 = this.instance) || void 0 === _this$instance60 ? void 0 : _this$instance60.getTotalSummaryValue(summaryItemName)
    }
    getVisibleColumns(headerLevel) {
        var _this$instance61;
        return null === (_this$instance61 = this.instance) || void 0 === _this$instance61 ? void 0 : _this$instance61.getVisibleColumns(headerLevel)
    }
    getVisibleRows() {
        var _this$instance62;
        return null === (_this$instance62 = this.instance) || void 0 === _this$instance62 ? void 0 : _this$instance62.getVisibleRows()
    }
    isRowExpanded(key) {
        var _this$instance63;
        return null === (_this$instance63 = this.instance) || void 0 === _this$instance63 ? void 0 : _this$instance63.isRowExpanded(key)
    }
    totalCount() {
        var _this$instance64;
        return null === (_this$instance64 = this.instance) || void 0 === _this$instance64 ? void 0 : _this$instance64.totalCount()
    }
    getController(name) {
        var _this$instance65;
        return null === (_this$instance65 = this.instance) || void 0 === _this$instance65 ? void 0 : _this$instance65.getController(name)
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props, {
                filterValue: this.__state_filterValue,
                focusedColumnIndex: this.__state_focusedColumnIndex,
                focusedRowIndex: this.__state_focusedRowIndex,
                focusedRowKey: this.__state_focusedRowKey,
                selectedRowKeys: this.__state_selectedRowKeys,
                selectionFilter: this.__state_selectionFilter,
                rowTemplate: getTemplate(props.rowTemplate)
            }),
            instance: this.instance,
            normalizeProps: this.normalizeProps,
            createInstance: this.createInstance,
            restAttributes: this.restAttributes
        })
    }
}
DataGrid.defaultProps = _extends({}, DataGridProps);
