/**
 * DevExtreme (esm/renovation/ui/grids/data_grid/utils/get_updated_options.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    type
} from "../../../../../core/utils/type";

function getDiffItem(key, value) {
    return {
        path: key,
        value: value
    }
}

function compare(resultPaths, item1, item2, key) {
    var type1 = type(item1);
    var type2 = type(item2);
    if (item1 === item2) {
        return
    }
    if (type1 !== type2) {
        resultPaths.push(getDiffItem(key, item2))
    } else if ("object" === type1) {
        var diffPaths = objectDiffs(item1, item2);
        resultPaths.push(...diffPaths.map(item => ({
            path: "".concat(key, ".").concat(item.path),
            value: item.value
        })))
    } else if ("array" === type1) {
        if (item1.length !== item2.length) {
            resultPaths.push(getDiffItem(key, item2))
        } else {
            var _diffPaths = objectDiffs(item1, item2);
            [].push.apply(resultPaths, _diffPaths.map(item => ({
                path: "".concat(key).concat(item.path),
                value: item.value
            })))
        }
    } else {
        resultPaths.push(getDiffItem(key, item2))
    }
}
var objectDiffsFiltered = propsEnumerator => (oldProps, props) => {
    if (!props) {
        return []
    }
    var resultPaths = [];
    var processItem = !Array.isArray(oldProps) ? propName => compare(resultPaths, oldProps[propName], props[propName], propName) : propName => compare(resultPaths, oldProps[propName], props[propName], "[".concat(propName, "]"));
    propsEnumerator(oldProps).forEach(processItem);
    Object.keys(props).filter(propName => !oldProps[propName] && oldProps[propName] !== props[propName]).forEach(propName => {
        resultPaths.push({
            path: propName,
            value: props[propName]
        })
    });
    return resultPaths
};
var objectDiffs = objectDiffsFiltered(oldProps => Object.keys(oldProps));
var reactProps = {
    key: true,
    ref: true,
    children: true
};
var objectDiffsWithoutReactProps = objectDiffsFiltered(prop => Object.keys(prop).filter(p => !reactProps[p]));
export function getUpdatedOptions(oldProps, props) {
    return objectDiffsWithoutReactProps(oldProps, props)
}
