/**
 * DevExtreme (esm/renovation/ui/scroll_view/scroll_view.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import {
    createComponentVNode,
    normalizeProps
} from "inferno";
import {
    BaseInfernoComponent
} from "@devextreme/vdom";
import {
    current,
    isMaterial
} from "../../../ui/themes";
import {
    isDefined
} from "../../../core/utils/type";
import {
    Scrollable,
    defaultOptionRules
} from "./scrollable";
import {
    combineClasses
} from "../../utils/combine_classes";
import {
    ScrollViewPropsType
} from "./scroll_view_props";
import {
    ScrollableSimulated
} from "./scrollable_simulated";
export var viewFunction = viewModel => {
    var {
        cssClasses: cssClasses,
        props: props,
        pulledDownText: pulledDownText,
        pullingDownText: pullingDownText,
        reachBottomText: reachBottomText,
        refreshingText: refreshingText,
        restAttributes: restAttributes,
        scrollViewSimulatedRef: scrollViewSimulatedRef,
        scrollableNativeRef: scrollableNativeRef
    } = viewModel;
    return viewModel.props.useNative ? normalizeProps(createComponentVNode(2, Scrollable, _extends({
        classes: cssClasses
    }, props, restAttributes, {
        pulledDownText: pulledDownText,
        pullingDownText: pullingDownText,
        refreshingText: refreshingText,
        reachBottomText: reachBottomText,
        forceGeneratePockets: true,
        needScrollViewContentWrapper: true,
        needScrollViewLoadPanel: true
    }), null, scrollableNativeRef)) : normalizeProps(createComponentVNode(2, ScrollableSimulated, _extends({
        classes: cssClasses
    }, props, restAttributes, {
        pulledDownText: pulledDownText,
        pullingDownText: pullingDownText,
        refreshingText: refreshingText,
        reachBottomText: reachBottomText,
        forceGeneratePockets: true,
        needScrollViewContentWrapper: true,
        needScrollViewLoadPanel: true
    }), null, scrollViewSimulatedRef))
};
import {
    convertRulesToOptions
} from "../../../core/options/utils";
import {
    createRef as infernoCreateRef
} from "inferno";
export class ScrollView extends BaseInfernoComponent {
    constructor(props) {
        super(props);
        this.state = {};
        this.scrollableNativeRef = infernoCreateRef();
        this.scrollViewSimulatedRef = infernoCreateRef();
        this.update = this.update.bind(this);
        this.release = this.release.bind(this);
        this.refresh = this.refresh.bind(this);
        this.content = this.content.bind(this);
        this.scrollBy = this.scrollBy.bind(this);
        this.scrollTo = this.scrollTo.bind(this);
        this.scrollToElement = this.scrollToElement.bind(this);
        this.scrollHeight = this.scrollHeight.bind(this);
        this.scrollWidth = this.scrollWidth.bind(this);
        this.scrollOffset = this.scrollOffset.bind(this);
        this.scrollTop = this.scrollTop.bind(this);
        this.scrollLeft = this.scrollLeft.bind(this);
        this.clientHeight = this.clientHeight.bind(this);
        this.clientWidth = this.clientWidth.bind(this)
    }
    get cssClasses() {
        return combineClasses({
            "dx-scrollview": true
        })
    }
    get pullingDownText() {
        var {
            pullingDownText: pullingDownText
        } = this.props;
        if (isDefined(pullingDownText)) {
            return pullingDownText
        }
        return isMaterial(current()) ? "" : void 0
    }
    get pulledDownText() {
        var {
            pulledDownText: pulledDownText
        } = this.props;
        if (isDefined(pulledDownText)) {
            return pulledDownText
        }
        return isMaterial(current()) ? "" : void 0
    }
    get refreshingText() {
        var {
            refreshingText: refreshingText
        } = this.props;
        if (isDefined(refreshingText)) {
            return refreshingText
        }
        return isMaterial(current()) ? "" : void 0
    }
    get reachBottomText() {
        var {
            reachBottomText: reachBottomText
        } = this.props;
        if (isDefined(reachBottomText)) {
            return reachBottomText
        }
        return isMaterial(current()) ? "" : void 0
    }
    get scrollableRef() {
        if (this.props.useNative) {
            return this.scrollableNativeRef.current
        }
        return this.scrollViewSimulatedRef.current
    }
    get restAttributes() {
        var _this$props = this.props,
            restProps = _objectWithoutPropertiesLoose(_this$props, ["bounceEnabled", "children", "classes", "direction", "disabled", "forceGeneratePockets", "height", "needScrollViewContentWrapper", "needScrollViewLoadPanel", "onPullDown", "onReachBottom", "onScroll", "onUpdated", "pullDownEnabled", "pulledDownText", "pullingDownText", "reachBottomEnabled", "reachBottomText", "refreshingText", "rtlEnabled", "scrollByContent", "scrollByThumb", "showScrollbar", "useKeyboard", "useNative", "width"]);
        return restProps
    }
    update() {
        this.scrollableRef.update()
    }
    release() {
        this.scrollableRef.release()
    }
    refresh() {
        if (this.props.pullDownEnabled) {
            this.scrollableRef.refresh()
        }
    }
    content() {
        return this.scrollableRef.content()
    }
    scrollBy(distance) {
        this.scrollableRef.scrollBy(distance)
    }
    scrollTo(targetLocation) {
        this.scrollableRef.scrollTo(targetLocation)
    }
    scrollToElement(element, offset) {
        this.scrollableRef.scrollToElement(element, offset)
    }
    scrollHeight() {
        return this.scrollableRef.scrollHeight()
    }
    scrollWidth() {
        return this.scrollableRef.scrollWidth()
    }
    scrollOffset() {
        return this.scrollableRef.scrollOffset()
    }
    scrollTop() {
        return this.scrollableRef.scrollTop()
    }
    scrollLeft() {
        return this.scrollableRef.scrollLeft()
    }
    clientHeight() {
        return this.scrollableRef.clientHeight()
    }
    clientWidth() {
        return this.scrollableRef.clientWidth()
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props),
            scrollableNativeRef: this.scrollableNativeRef,
            scrollViewSimulatedRef: this.scrollViewSimulatedRef,
            cssClasses: this.cssClasses,
            pullingDownText: this.pullingDownText,
            pulledDownText: this.pulledDownText,
            refreshingText: this.refreshingText,
            reachBottomText: this.reachBottomText,
            scrollableRef: this.scrollableRef,
            restAttributes: this.restAttributes
        })
    }
}

function __createDefaultProps() {
    return _extends({}, ScrollViewPropsType, convertRulesToOptions(defaultOptionRules))
}
ScrollView.defaultProps = __createDefaultProps();
var __defaultOptionRules = [];
export function defaultOptions(rule) {
    __defaultOptionRules.push(rule);
    ScrollView.defaultProps = _extends({}, __createDefaultProps(), convertRulesToOptions(__defaultOptionRules))
}
