/**
 * DevExtreme (esm/renovation/ui/scroll_view/scrollable.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import {
    createComponentVNode,
    normalizeProps
} from "inferno";
import {
    BaseInfernoComponent
} from "@devextreme/vdom";
import {
    BaseWidgetProps
} from "../common/base_props";
import {
    ScrollableProps
} from "./scrollable_props";
import {
    ScrollableNative
} from "./scrollable_native";
import {
    ScrollableSimulated
} from "./scrollable_simulated";
import {
    createDefaultOptionRules,
    convertRulesToOptions
} from "../../../core/options/utils";
import devices from "../../../core/devices";
import {
    nativeScrolling,
    touch
} from "../../../core/utils/support";
export var viewFunction = viewModel => {
    var {
        props: {
            pulledDownText: pulledDownText,
            pullingDownText: pullingDownText,
            reachBottomText: reachBottomText,
            refreshingText: refreshingText,
            useNative: useNative
        },
        restAttributes: restAttributes,
        scrollableNativeRef: scrollableNativeRef,
        scrollableSimulatedRef: scrollableSimulatedRef
    } = viewModel, scrollableProps = _objectWithoutPropertiesLoose(viewModel.props, ["pulledDownText", "pullingDownText", "reachBottomText", "refreshingText", "useNative"]);
    return useNative ? normalizeProps(createComponentVNode(2, ScrollableNative, _extends({}, scrollableProps, restAttributes, {
        pulledDownText: pulledDownText,
        pullingDownText: pullingDownText,
        refreshingText: refreshingText,
        reachBottomText: reachBottomText
    }), null, scrollableNativeRef)) : normalizeProps(createComponentVNode(2, ScrollableSimulated, _extends({}, scrollableProps, restAttributes, {
        pulledDownText: pulledDownText,
        pullingDownText: pullingDownText,
        refreshingText: refreshingText,
        reachBottomText: reachBottomText
    }), null, scrollableSimulatedRef))
};
var ScrollablePropsType = {
    useNative: ScrollableProps.useNative,
    direction: ScrollableProps.direction,
    showScrollbar: ScrollableProps.showScrollbar,
    scrollByThumb: ScrollableProps.scrollByThumb,
    bounceEnabled: ScrollableProps.bounceEnabled,
    scrollByContent: ScrollableProps.scrollByContent,
    useKeyboard: ScrollableProps.useKeyboard,
    pullDownEnabled: ScrollableProps.pullDownEnabled,
    reachBottomEnabled: ScrollableProps.reachBottomEnabled,
    forceGeneratePockets: ScrollableProps.forceGeneratePockets,
    needScrollViewContentWrapper: ScrollableProps.needScrollViewContentWrapper,
    needScrollViewLoadPanel: ScrollableProps.needScrollViewLoadPanel,
    disabled: BaseWidgetProps.disabled
};
export var defaultOptionRules = createDefaultOptionRules([{
    device: _device => !devices.isSimulator() && "desktop" === devices.real().deviceType && "generic" === _device.platform,
    options: {
        bounceEnabled: false,
        scrollByContent: touch,
        scrollByThumb: true,
        showScrollbar: "onHover"
    }
}, {
    device: () => !nativeScrolling,
    options: {
        useNative: false
    }
}]);
import {
    createRef as infernoCreateRef
} from "inferno";
export class Scrollable extends BaseInfernoComponent {
    constructor(props) {
        super(props);
        this.state = {};
        this.scrollableNativeRef = infernoCreateRef();
        this.scrollableSimulatedRef = infernoCreateRef();
        this.content = this.content.bind(this);
        this.scrollBy = this.scrollBy.bind(this);
        this.update = this.update.bind(this);
        this.release = this.release.bind(this);
        this.refresh = this.refresh.bind(this);
        this.scrollTo = this.scrollTo.bind(this);
        this.scrollToElement = this.scrollToElement.bind(this);
        this.scrollHeight = this.scrollHeight.bind(this);
        this.scrollWidth = this.scrollWidth.bind(this);
        this.scrollOffset = this.scrollOffset.bind(this);
        this.scrollTop = this.scrollTop.bind(this);
        this.scrollLeft = this.scrollLeft.bind(this);
        this.clientHeight = this.clientHeight.bind(this);
        this.clientWidth = this.clientWidth.bind(this);
        this.validate = this.validate.bind(this)
    }
    get scrollableRef() {
        if (this.props.useNative) {
            return this.scrollableNativeRef.current
        }
        return this.scrollableSimulatedRef.current
    }
    get restAttributes() {
        var _this$props = this.props,
            restProps = _objectWithoutPropertiesLoose(_this$props, ["bounceEnabled", "children", "classes", "direction", "disabled", "forceGeneratePockets", "height", "needScrollViewContentWrapper", "needScrollViewLoadPanel", "onPullDown", "onReachBottom", "onScroll", "onUpdated", "pullDownEnabled", "pulledDownText", "pullingDownText", "reachBottomEnabled", "reachBottomText", "refreshingText", "rtlEnabled", "scrollByContent", "scrollByThumb", "showScrollbar", "useKeyboard", "useNative", "width"]);
        return restProps
    }
    content() {
        return this.scrollableRef.content()
    }
    scrollBy(distance) {
        this.scrollableRef.scrollBy(distance)
    }
    update() {
        this.scrollableRef.update()
    }
    release() {
        return this.scrollableRef.release()
    }
    refresh() {
        this.scrollableRef.refresh()
    }
    scrollTo(targetLocation) {
        this.scrollableRef.scrollTo(targetLocation)
    }
    scrollToElement(element, offset) {
        this.scrollableRef.scrollToElement(element, offset)
    }
    scrollHeight() {
        return this.scrollableRef.scrollHeight()
    }
    scrollWidth() {
        return this.scrollableRef.scrollWidth()
    }
    scrollOffset() {
        return this.scrollableRef.scrollOffset()
    }
    scrollTop() {
        return this.scrollableRef.scrollTop()
    }
    scrollLeft() {
        return this.scrollableRef.scrollLeft()
    }
    clientHeight() {
        return this.scrollableRef.clientHeight()
    }
    clientWidth() {
        return this.scrollableRef.clientWidth()
    }
    validate(e) {
        return this.scrollableRef.validate(e)
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props),
            scrollableNativeRef: this.scrollableNativeRef,
            scrollableSimulatedRef: this.scrollableSimulatedRef,
            scrollableRef: this.scrollableRef,
            restAttributes: this.restAttributes
        })
    }
}

function __createDefaultProps() {
    return _extends({}, ScrollablePropsType, convertRulesToOptions(defaultOptionRules))
}
Scrollable.defaultProps = __createDefaultProps();
var __defaultOptionRules = [];
export function defaultOptions(rule) {
    __defaultOptionRules.push(rule);
    Scrollable.defaultProps = _extends({}, __createDefaultProps(), convertRulesToOptions(__defaultOptionRules))
}
