/**
 * DevExtreme (esm/renovation/ui/scroll_view/scrollable_native.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import {
    createVNode,
    createComponentVNode,
    normalizeProps
} from "inferno";
import {
    InfernoEffect,
    InfernoComponent
} from "@devextreme/vdom";
import {
    subscribeToScrollEvent
} from "../../utils/subscribe_to_event";
import {
    Widget
} from "../common/widget";
import {
    ScrollViewLoadPanel
} from "./load_panel";
import {
    combineClasses
} from "../../utils/combine_classes";
import devices from "../../../core/devices";
import {
    isDefined
} from "../../../core/utils/type";
import {
    BaseWidgetProps
} from "../common/base_props";
import {
    ScrollableProps
} from "./scrollable_props";
import {
    TopPocket
} from "./top_pocket";
import {
    BottomPocket
} from "./bottom_pocket";
import browser from "../../../core/utils/browser";
import {
    nativeScrolling
} from "../../../core/utils/support";
import "../../../events/gesture/emitter.gesture.scroll";
import {
    isDxMouseWheelEvent
} from "../../../events/utils/index";
import {
    ensureLocation,
    normalizeCoordinate,
    getMaxScrollOffset,
    getElementLocation,
    getPublicCoordinate,
    getBoundaryProps
} from "./scrollable_utils";
import {
    ScrollDirection
} from "./utils/scroll_direction";
import {
    DIRECTION_VERTICAL,
    DIRECTION_HORIZONTAL,
    DIRECTION_BOTH,
    SCROLLABLE_CONTAINER_CLASS,
    SCROLLABLE_CONTENT_CLASS,
    SCROLLABLE_WRAPPER_CLASS,
    SCROLLVIEW_CONTENT_CLASS,
    SCROLLABLE_DISABLED_CLASS,
    SCROLLABLE_SCROLLBAR_SIMULATED,
    SCROLLABLE_SCROLLBARS_HIDDEN,
    TopPocketState
} from "./common/consts";
import {
    Scrollbar
} from "./scrollbar";
import {
    dxScrollInit,
    dxScrollMove
} from "../../../events/short";
var HIDE_SCROLLBAR_TIMEOUT = 500;
export var viewFunction = viewModel => {
    var {
        containerClientHeight: containerClientHeight,
        containerClientWidth: containerClientWidth,
        containerRef: containerRef,
        contentClientHeight: contentClientHeight,
        contentClientWidth: contentClientWidth,
        contentRef: contentRef,
        cssClasses: cssClasses,
        direction: direction,
        horizontalScrollbarRef: horizontalScrollbarRef,
        isLoadPanelVisible: isLoadPanelVisible,
        needForceScrollbarsVisibility: needForceScrollbarsVisibility,
        props: {
            children: children,
            disabled: disabled,
            forceGeneratePockets: forceGeneratePockets,
            height: height,
            needScrollViewContentWrapper: needScrollViewContentWrapper,
            needScrollViewLoadPanel: needScrollViewLoadPanel,
            pullDownEnabled: pullDownEnabled,
            pulledDownText: pulledDownText,
            pullingDownText: pullingDownText,
            reachBottomEnabled: reachBottomEnabled,
            reachBottomText: reachBottomText,
            refreshingText: refreshingText,
            rtlEnabled: rtlEnabled,
            scrollByThumb: scrollByThumb,
            showScrollbar: showScrollbar,
            visible: visible,
            width: width
        },
        restAttributes: restAttributes,
        scrollableRef: scrollableRef,
        topPocketState: topPocketState,
        useSimulatedScrollbar: useSimulatedScrollbar,
        verticalScrollbarRef: verticalScrollbarRef,
        windowResizeHandler: windowResizeHandler,
        wrapperRef: wrapperRef
    } = viewModel;
    return normalizeProps(createComponentVNode(2, Widget, _extends({
        rootElementRef: scrollableRef,
        classes: cssClasses,
        disabled: disabled,
        rtlEnabled: rtlEnabled,
        height: height,
        width: width,
        visible: visible,
        onDimensionChanged: windowResizeHandler
    }, restAttributes, {
        children: [createVNode(1, "div", SCROLLABLE_WRAPPER_CLASS, createVNode(1, "div", SCROLLABLE_CONTAINER_CLASS, createVNode(1, "div", SCROLLABLE_CONTENT_CLASS, [forceGeneratePockets && createComponentVNode(2, TopPocket, {
            pullingDownText: pullingDownText,
            pulledDownText: pulledDownText,
            refreshingText: refreshingText,
            pocketState: topPocketState,
            useNative: true,
            visible: !!pullDownEnabled
        }), needScrollViewContentWrapper ? createVNode(1, "div", SCROLLVIEW_CONTENT_CLASS, children, 0) : createVNode(1, "div", null, children, 0), forceGeneratePockets && createComponentVNode(2, BottomPocket, {
            reachBottomText: reachBottomText,
            visible: !!reachBottomEnabled
        })], 0, null, null, contentRef), 2, null, null, containerRef), 2, null, null, wrapperRef), needScrollViewLoadPanel && createComponentVNode(2, ScrollViewLoadPanel, {
            targetElement: scrollableRef,
            refreshingText: refreshingText,
            visible: isLoadPanelVisible
        }), showScrollbar && useSimulatedScrollbar && direction.isHorizontal && createComponentVNode(2, Scrollbar, {
            direction: "horizontal",
            scrollByThumb: scrollByThumb,
            contentSize: contentClientWidth,
            containerSize: containerClientWidth,
            forceVisibility: needForceScrollbarsVisibility
        }, null, horizontalScrollbarRef), showScrollbar && useSimulatedScrollbar && direction.isVertical && createComponentVNode(2, Scrollbar, {
            direction: "vertical",
            scrollByThumb: scrollByThumb,
            contentSize: contentClientHeight,
            containerSize: containerClientHeight,
            forceVisibility: needForceScrollbarsVisibility
        }, null, verticalScrollbarRef)]
    })))
};
export var ScrollableNativeProps = _extends({}, ScrollableProps);
var ScrollableNativePropsType = {
    useNative: ScrollableNativeProps.useNative,
    direction: ScrollableNativeProps.direction,
    showScrollbar: ScrollableNativeProps.showScrollbar,
    scrollByThumb: ScrollableNativeProps.scrollByThumb,
    bounceEnabled: ScrollableNativeProps.bounceEnabled,
    scrollByContent: ScrollableNativeProps.scrollByContent,
    useKeyboard: ScrollableNativeProps.useKeyboard,
    pullDownEnabled: ScrollableNativeProps.pullDownEnabled,
    reachBottomEnabled: ScrollableNativeProps.reachBottomEnabled,
    forceGeneratePockets: ScrollableNativeProps.forceGeneratePockets,
    needScrollViewContentWrapper: ScrollableNativeProps.needScrollViewContentWrapper,
    needScrollViewLoadPanel: ScrollableNativeProps.needScrollViewLoadPanel,
    disabled: BaseWidgetProps.disabled,
    visible: BaseWidgetProps.visible
};
import {
    createRef as infernoCreateRef
} from "inferno";
export class ScrollableNative extends InfernoComponent {
    constructor(props) {
        super(props);
        this._currentState = null;
        this.scrollableRef = infernoCreateRef();
        this.wrapperRef = infernoCreateRef();
        this.contentRef = infernoCreateRef();
        this.containerRef = infernoCreateRef();
        this.verticalScrollbarRef = infernoCreateRef();
        this.horizontalScrollbarRef = infernoCreateRef();
        this.locked = false;
        this.loadingIndicatorEnabled = true;
        this.lastLocation = {
            top: 0,
            left: 0
        };
        this.state = {
            containerClientWidth: 0,
            containerClientHeight: 0,
            contentClientWidth: 0,
            contentClientHeight: 0,
            needForceScrollbarsVisibility: false,
            topPocketState: TopPocketState.STATE_RELEASED,
            isLoadPanelVisible: false
        };
        this.content = this.content.bind(this);
        this.update = this.update.bind(this);
        this.release = this.release.bind(this);
        this.scrollBy = this.scrollBy.bind(this);
        this.scrollTo = this.scrollTo.bind(this);
        this.scrollToElement = this.scrollToElement.bind(this);
        this.scrollHeight = this.scrollHeight.bind(this);
        this.scrollWidth = this.scrollWidth.bind(this);
        this.scrollOffset = this.scrollOffset.bind(this);
        this.scrollTop = this.scrollTop.bind(this);
        this.scrollLeft = this.scrollLeft.bind(this);
        this.clientHeight = this.clientHeight.bind(this);
        this.clientWidth = this.clientWidth.bind(this);
        this.scrollEffect = this.scrollEffect.bind(this);
        this.getEventArgs = this.getEventArgs.bind(this);
        this.effectDisabledState = this.effectDisabledState.bind(this);
        this.lock = this.lock.bind(this);
        this.unlock = this.unlock.bind(this);
        this.effectResetInactiveState = this.effectResetInactiveState.bind(this);
        this.updateScrollbarSize = this.updateScrollbarSize.bind(this);
        this.windowResizeHandler = this.windowResizeHandler.bind(this);
        this.updateSizes = this.updateSizes.bind(this);
        this.moveScrollbars = this.moveScrollbars.bind(this);
        this.disposeHideScrollbarTimeout = this.disposeHideScrollbarTimeout.bind(this);
        this.clearHideScrollbarTimeout = this.clearHideScrollbarTimeout.bind(this);
        this.location = this.location.bind(this);
        this.initEffect = this.initEffect.bind(this);
        this.getInitEventData = this.getInitEventData.bind(this);
        this.moveEffect = this.moveEffect.bind(this);
        this.handleInit = this.handleInit.bind(this);
        this.handleMove = this.handleMove.bind(this);
        this.tryGetAllowedDirection = this.tryGetAllowedDirection.bind(this);
        this.validate = this.validate.bind(this);
        this.isLocked = this.isLocked.bind(this);
        this.isScrollingOutOfBound = this.isScrollingOutOfBound.bind(this)
    }
    createEffects() {
        return [new InfernoEffect(this.scrollEffect, [this.props.useSimulatedScrollbar, this.props.direction, this.props.onScroll, this.props.rtlEnabled]), new InfernoEffect(this.effectDisabledState, [this.props.disabled]), new InfernoEffect(this.effectResetInactiveState, [this.props.direction]), new InfernoEffect(this.updateScrollbarSize, [this.containerClientWidth, this.containerClientHeight, this.contentClientWidth, this.contentClientHeight, this.needForceScrollbarsVisibility, this.topPocketState, this.isLoadPanelVisible, this.props.useSimulatedScrollbar, this.props.useNative, this.props.pullingDownText, this.props.pulledDownText, this.props.refreshingText, this.props.reachBottomText, this.props.children, this.props.direction, this.props.showScrollbar, this.props.scrollByThumb, this.props.bounceEnabled, this.props.scrollByContent, this.props.useKeyboard, this.props.classes, this.props.pullDownEnabled, this.props.reachBottomEnabled, this.props.forceGeneratePockets, this.props.needScrollViewContentWrapper, this.props.needScrollViewLoadPanel, this.props.onScroll, this.props.onUpdated, this.props.onPullDown, this.props.onReachBottom, this.props.disabled, this.props.height, this.props.onKeyDown, this.props.rtlEnabled, this.props.visible, this.props.width]), new InfernoEffect(this.disposeHideScrollbarTimeout, []), new InfernoEffect(this.initEffect, [this.props.direction, this.props.pullDownEnabled, this.props.disabled, this.props.rtlEnabled]), new InfernoEffect(this.moveEffect, [this.props.direction, this.props.pullDownEnabled])]
    }
    updateEffects() {
        var _this$_effects$, _this$_effects$2, _this$_effects$3, _this$_effects$4, _this$_effects$5, _this$_effects$6;
        null === (_this$_effects$ = this._effects[0]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([this.props.useSimulatedScrollbar, this.props.direction, this.props.onScroll, this.props.rtlEnabled]);
        null === (_this$_effects$2 = this._effects[1]) || void 0 === _this$_effects$2 ? void 0 : _this$_effects$2.update([this.props.disabled]);
        null === (_this$_effects$3 = this._effects[2]) || void 0 === _this$_effects$3 ? void 0 : _this$_effects$3.update([this.props.direction]);
        null === (_this$_effects$4 = this._effects[3]) || void 0 === _this$_effects$4 ? void 0 : _this$_effects$4.update([this.containerClientWidth, this.containerClientHeight, this.contentClientWidth, this.contentClientHeight, this.needForceScrollbarsVisibility, this.topPocketState, this.isLoadPanelVisible, this.props.useSimulatedScrollbar, this.props.useNative, this.props.pullingDownText, this.props.pulledDownText, this.props.refreshingText, this.props.reachBottomText, this.props.children, this.props.direction, this.props.showScrollbar, this.props.scrollByThumb, this.props.bounceEnabled, this.props.scrollByContent, this.props.useKeyboard, this.props.classes, this.props.pullDownEnabled, this.props.reachBottomEnabled, this.props.forceGeneratePockets, this.props.needScrollViewContentWrapper, this.props.needScrollViewLoadPanel, this.props.onScroll, this.props.onUpdated, this.props.onPullDown, this.props.onReachBottom, this.props.disabled, this.props.height, this.props.onKeyDown, this.props.rtlEnabled, this.props.visible, this.props.width]);
        null === (_this$_effects$5 = this._effects[5]) || void 0 === _this$_effects$5 ? void 0 : _this$_effects$5.update([this.props.direction, this.props.pullDownEnabled, this.props.disabled, this.props.rtlEnabled]);
        null === (_this$_effects$6 = this._effects[6]) || void 0 === _this$_effects$6 ? void 0 : _this$_effects$6.update([this.props.direction, this.props.pullDownEnabled])
    }
    get containerClientWidth() {
        var state = this._currentState || this.state;
        return state.containerClientWidth
    }
    set_containerClientWidth(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                containerClientWidth: newValue
            }
        })
    }
    get containerClientHeight() {
        var state = this._currentState || this.state;
        return state.containerClientHeight
    }
    set_containerClientHeight(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                containerClientHeight: newValue
            }
        })
    }
    get contentClientWidth() {
        var state = this._currentState || this.state;
        return state.contentClientWidth
    }
    set_contentClientWidth(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                contentClientWidth: newValue
            }
        })
    }
    get contentClientHeight() {
        var state = this._currentState || this.state;
        return state.contentClientHeight
    }
    set_contentClientHeight(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                contentClientHeight: newValue
            }
        })
    }
    get needForceScrollbarsVisibility() {
        var state = this._currentState || this.state;
        return state.needForceScrollbarsVisibility
    }
    set_needForceScrollbarsVisibility(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                needForceScrollbarsVisibility: newValue
            }
        })
    }
    get topPocketState() {
        var state = this._currentState || this.state;
        return state.topPocketState
    }
    set_topPocketState(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                topPocketState: newValue
            }
        })
    }
    get isLoadPanelVisible() {
        var state = this._currentState || this.state;
        return state.isLoadPanelVisible
    }
    set_isLoadPanelVisible(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                isLoadPanelVisible: newValue
            }
        })
    }
    scrollEffect() {
        return subscribeToScrollEvent(this.containerRef.current, e => {
            var _this$props$onScroll, _this$props;
            this.eventForUserAction = e;
            if (this.useSimulatedScrollbar) {
                this.moveScrollbars()
            }
            null === (_this$props$onScroll = (_this$props = this.props).onScroll) || void 0 === _this$props$onScroll ? void 0 : _this$props$onScroll.call(_this$props, this.getEventArgs());
            this.lastLocation = this.location()
        })
    }
    effectDisabledState() {
        if (this.props.disabled) {
            this.lock()
        } else {
            this.unlock()
        }
    }
    effectResetInactiveState() {
        var containerEl = this.containerRef.current;
        if (this.props.direction === DIRECTION_BOTH || !isDefined(containerEl)) {
            return
        }
        containerEl[this.fullScrollInactiveProp] = 0
    }
    updateScrollbarSize() {
        this.updateSizes()
    }
    disposeHideScrollbarTimeout() {
        return () => this.clearHideScrollbarTimeout()
    }
    initEffect() {
        dxScrollInit.on(this.wrapperRef.current, e => {
            this.handleInit(e)
        }, this.getInitEventData(), {
            namespace: "dxScrollable"
        });
        return () => dxScrollInit.off(this.wrapperRef.current, {
            namespace: "dxScrollable"
        })
    }
    moveEffect() {
        dxScrollMove.on(this.wrapperRef.current, e => {
            this.handleMove(e)
        }, {
            namespace: "dxScrollable"
        });
        return () => dxScrollMove.off(this.wrapperRef.current, {
            namespace: "dxScrollable"
        })
    }
    getEventArgs() {
        return _extends({
            event: this.eventForUserAction,
            scrollOffset: this.scrollOffset()
        }, getBoundaryProps(this.props.direction, this.scrollOffset(), this.containerRef.current, 0))
    }
    lock() {
        this.locked = true
    }
    unlock() {
        if (!this.props.disabled) {
            this.locked = false
        }
    }
    get fullScrollInactiveProp() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "scrollTop" : "scrollLeft"
    }
    windowResizeHandler() {
        var _this$props$onUpdated, _this$props2;
        this.updateSizes();
        null === (_this$props$onUpdated = (_this$props2 = this.props).onUpdated) || void 0 === _this$props$onUpdated ? void 0 : _this$props$onUpdated.call(_this$props2, this.getEventArgs())
    }
    updateSizes() {
        var containerEl = this.containerRef.current;
        var contentEl = this.contentRef.current;
        if (isDefined(containerEl)) {
            this.set_containerClientWidth(() => containerEl.clientWidth);
            this.set_containerClientHeight(() => containerEl.clientHeight)
        }
        if (isDefined(contentEl)) {
            this.set_contentClientWidth(() => contentEl.clientWidth);
            this.set_contentClientHeight(() => contentEl.clientHeight)
        }
    }
    moveScrollbars() {
        var {
            left: left,
            top: top
        } = this.location();
        if (this.direction.isHorizontal) {
            var scrollbarEl = this.horizontalScrollbarRef.current;
            if (isDefined(scrollbarEl)) {
                this.horizontalScrollbarRef.current.moveScrollbar(left)
            }
        }
        if (this.direction.isVertical) {
            var _scrollbarEl = this.verticalScrollbarRef.current;
            if (isDefined(_scrollbarEl)) {
                this.verticalScrollbarRef.current.moveScrollbar(top)
            }
        }
        this.set_needForceScrollbarsVisibility(() => true);
        this.clearHideScrollbarTimeout();
        this.hideScrollbarTimeout = setTimeout(() => {
            this.set_needForceScrollbarsVisibility(() => false)
        }, HIDE_SCROLLBAR_TIMEOUT)
    }
    clearHideScrollbarTimeout() {
        clearTimeout(this.hideScrollbarTimeout);
        this.hideScrollbarTimeout = void 0
    }
    location() {
        return {
            top: -this.containerRef.current.scrollTop,
            left: -this.containerRef.current.scrollLeft
        }
    }
    getInitEventData() {
        return {
            getDirection: this.tryGetAllowedDirection,
            validate: this.validate,
            isNative: true,
            scrollTarget: this.containerRef.current
        }
    }
    handleInit(e) {}
    handleMove(e) {
        if (this.locked) {
            e.cancel = true;
            return
        }
        if (isDefined(this.tryGetAllowedDirection())) {
            e.originalEvent.isScrollingEvent = true
        }
    }
    tryGetAllowedDirection() {
        var {
            isBoth: isBoth,
            isHorizontal: isHorizontal,
            isVertical: isVertical
        } = new ScrollDirection(this.props.direction);
        var contentEl = this.contentRef.current;
        var containerEl = this.containerRef.current;
        var isOverflowVertical = isVertical && contentEl.clientHeight > containerEl.clientHeight || this.props.pullDownEnabled;
        var isOverflowHorizontal = isHorizontal && contentEl.clientWidth > containerEl.clientWidth || this.props.pullDownEnabled;
        if (isBoth && isOverflowVertical && isOverflowHorizontal) {
            return DIRECTION_BOTH
        }
        if (isHorizontal && isOverflowHorizontal) {
            return DIRECTION_HORIZONTAL
        }
        if (isVertical && isOverflowVertical) {
            return DIRECTION_VERTICAL
        }
        return
    }
    isLocked() {
        return this.locked
    }
    isScrollingOutOfBound(e) {
        var {
            delta: delta,
            shiftKey: shiftKey
        } = e;
        var {
            clientHeight: clientHeight,
            clientWidth: clientWidth,
            scrollHeight: scrollHeight,
            scrollLeft: scrollLeft,
            scrollTop: scrollTop,
            scrollWidth: scrollWidth
        } = this.containerRef.current;
        if (delta > 0) {
            return shiftKey ? !scrollLeft : !scrollTop
        }
        return shiftKey ? clientWidth >= scrollWidth - scrollLeft : clientHeight >= scrollHeight - scrollTop
    }
    get cssClasses() {
        var {
            classes: classes,
            direction: direction,
            disabled: disabled,
            showScrollbar: showScrollbar
        } = this.props;
        var classesMap = {
            ["dx-scrollable dx-scrollable-native dx-scrollable-native-".concat(devices.real().platform, " dx-scrollable-renovated")]: true,
            ["dx-scrollable-".concat(direction)]: true,
            [SCROLLABLE_DISABLED_CLASS]: !!disabled,
            [SCROLLABLE_SCROLLBAR_SIMULATED]: showScrollbar && this.useSimulatedScrollbar,
            [SCROLLABLE_SCROLLBARS_HIDDEN]: !showScrollbar,
            ["".concat(classes)]: !!classes
        };
        return combineClasses(classesMap)
    }
    get direction() {
        return new ScrollDirection(this.props.direction)
    }
    get useSimulatedScrollbar() {
        if (!isDefined(this.props.useSimulatedScrollbar)) {
            return nativeScrolling && "android" === devices.real().platform && !browser.mozilla
        }
        return this.props.useSimulatedScrollbar
    }
    get restAttributes() {
        var _this$props3 = this.props,
            restProps = _objectWithoutPropertiesLoose(_this$props3, ["bounceEnabled", "children", "classes", "direction", "disabled", "forceGeneratePockets", "height", "needScrollViewContentWrapper", "needScrollViewLoadPanel", "onKeyDown", "onPullDown", "onReachBottom", "onScroll", "onUpdated", "pullDownEnabled", "pulledDownText", "pullingDownText", "reachBottomEnabled", "reachBottomText", "refreshingText", "rtlEnabled", "scrollByContent", "scrollByThumb", "showScrollbar", "useKeyboard", "useNative", "useSimulatedScrollbar", "visible", "width"]);
        return restProps
    }
    content() {
        return this.contentRef.current
    }
    update() {
        var contentEl = this.contentRef.current;
        if (isDefined(contentEl)) {
            var _this$props$onUpdated2, _this$props4;
            this.updateSizes();
            null === (_this$props$onUpdated2 = (_this$props4 = this.props).onUpdated) || void 0 === _this$props$onUpdated2 ? void 0 : _this$props$onUpdated2.call(_this$props4, this.getEventArgs())
        }
    }
    release() {}
    scrollBy(distance) {
        var location = ensureLocation(distance);
        if (this.direction.isVertical) {
            this.containerRef.current.scrollTop += Math.round(location.top)
        }
        if (this.direction.isHorizontal) {
            this.containerRef.current.scrollLeft += normalizeCoordinate("left", Math.round(location.left), this.props.rtlEnabled)
        }
    }
    scrollTo(targetLocation) {
        var location = ensureLocation(targetLocation);
        var containerPosition = this.scrollOffset();
        var top = location.top - containerPosition.top;
        var left = location.left - containerPosition.left;
        this.scrollBy({
            top: top,
            left: left
        })
    }
    scrollToElement(element, offset) {
        if (void 0 === element || null === element) {
            return
        }
        if (element.closest(".".concat(SCROLLABLE_CONTENT_CLASS))) {
            var scrollOffset = _extends({
                top: 0,
                left: 0,
                right: 0,
                bottom: 0
            }, offset);
            this.scrollTo({
                top: getElementLocation(element, scrollOffset, DIRECTION_VERTICAL, this.containerRef.current, this.props.rtlEnabled),
                left: getElementLocation(element, scrollOffset, DIRECTION_HORIZONTAL, this.containerRef.current, this.props.rtlEnabled)
            })
        }
    }
    scrollHeight() {
        return this.content().offsetHeight
    }
    scrollWidth() {
        return this.content().offsetWidth
    }
    scrollOffset() {
        var maxScrollLeftOffset = getMaxScrollOffset("width", this.containerRef.current);
        return {
            left: getPublicCoordinate("left", this.containerRef.current.scrollLeft, maxScrollLeftOffset, this.props.rtlEnabled),
            top: this.containerRef.current.scrollTop
        }
    }
    scrollTop() {
        return this.scrollOffset().top
    }
    scrollLeft() {
        return this.scrollOffset().left
    }
    clientHeight() {
        return this.containerRef.current.clientHeight
    }
    clientWidth() {
        return this.containerRef.current.clientWidth
    }
    validate(e) {
        var {
            disabled: disabled
        } = this.props;
        if (this.isLocked() || disabled || isDxMouseWheelEvent(e) && this.isScrollingOutOfBound(e)) {
            return false
        }
        return isDefined(this.tryGetAllowedDirection())
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props),
            containerClientWidth: this.containerClientWidth,
            containerClientHeight: this.containerClientHeight,
            contentClientWidth: this.contentClientWidth,
            contentClientHeight: this.contentClientHeight,
            needForceScrollbarsVisibility: this.needForceScrollbarsVisibility,
            topPocketState: this.topPocketState,
            isLoadPanelVisible: this.isLoadPanelVisible,
            scrollableRef: this.scrollableRef,
            wrapperRef: this.wrapperRef,
            contentRef: this.contentRef,
            containerRef: this.containerRef,
            verticalScrollbarRef: this.verticalScrollbarRef,
            horizontalScrollbarRef: this.horizontalScrollbarRef,
            getEventArgs: this.getEventArgs,
            lock: this.lock,
            unlock: this.unlock,
            fullScrollInactiveProp: this.fullScrollInactiveProp,
            windowResizeHandler: this.windowResizeHandler,
            updateSizes: this.updateSizes,
            moveScrollbars: this.moveScrollbars,
            clearHideScrollbarTimeout: this.clearHideScrollbarTimeout,
            location: this.location,
            getInitEventData: this.getInitEventData,
            handleInit: this.handleInit,
            handleMove: this.handleMove,
            tryGetAllowedDirection: this.tryGetAllowedDirection,
            isLocked: this.isLocked,
            isScrollingOutOfBound: this.isScrollingOutOfBound,
            cssClasses: this.cssClasses,
            direction: this.direction,
            useSimulatedScrollbar: this.useSimulatedScrollbar,
            restAttributes: this.restAttributes
        })
    }
}
ScrollableNative.defaultProps = _extends({}, ScrollableNativePropsType);
