/**
 * DevExtreme (esm/renovation/ui/scroll_view/scrollbar.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import {
    createVNode,
    createComponentVNode,
    normalizeProps
} from "inferno";
import {
    InfernoEffect,
    InfernoComponent,
    normalizeStyles
} from "@devextreme/vdom";
import {
    Widget
} from "../common/widget";
import {
    combineClasses
} from "../../utils/combine_classes";
import domAdapter from "../../../core/dom_adapter";
import {
    isDefined
} from "../../../core/utils/type";
import {
    isDxMouseWheelEvent
} from "../../../events/utils/index";
import {
    ScrollbarProps
} from "./scrollbar_props";
import {
    DIRECTION_HORIZONTAL,
    SCROLLABLE_SCROLLBAR_CLASS,
    TopPocketState,
    SCROLLABLE_SCROLL_CLASS,
    SCROLLABLE_SCROLL_CONTENT_CLASS,
    HIDE_SCROLLBAR_TIMEOUT,
    SCROLLABLE_SCROLLBAR_ACTIVE_CLASS,
    HOVER_ENABLED_STATE
} from "./common/consts";
import {
    dxPointerDown,
    dxPointerUp
} from "../../../events/short";
import {
    ScrollableSimulatedProps
} from "./scrollable_simulated_props";
import {
    ScrollableProps
} from "./scrollable_props";
var OUT_BOUNDS_ACCELERATION = .5;
var THUMB_MIN_SIZE = 15;
export var viewFunction = viewModel => {
    var {
        cssClasses: cssClasses,
        hoverStateEnabled: hoverStateEnabled,
        isVisible: isVisible,
        onHoverEnd: onHoverEnd,
        onHoverStart: onHoverStart,
        props: {
            activeStateEnabled: activeStateEnabled
        },
        restAttributes: restAttributes,
        scrollRef: scrollRef,
        scrollStyles: scrollStyles,
        scrollbarRef: scrollbarRef
    } = viewModel;
    return normalizeProps(createComponentVNode(2, Widget, _extends({
        rootElementRef: scrollbarRef,
        classes: cssClasses,
        activeStateEnabled: activeStateEnabled,
        hoverStateEnabled: hoverStateEnabled,
        visible: isVisible,
        onHoverStart: onHoverStart,
        onHoverEnd: onHoverEnd
    }, restAttributes, {
        children: createVNode(1, "div", viewModel.scrollClasses, createVNode(1, "div", SCROLLABLE_SCROLL_CONTENT_CLASS), 2, {
            style: normalizeStyles(scrollStyles)
        }, null, scrollRef)
    })))
};
export var ScrollbarPropsType = {
    activeStateEnabled: ScrollbarProps.activeStateEnabled,
    expandable: ScrollbarProps.expandable,
    direction: ScrollbarProps.direction,
    scaleRatio: ScrollbarProps.scaleRatio,
    containerSize: ScrollbarProps.containerSize,
    contentSize: ScrollbarProps.contentSize,
    baseContainerSize: ScrollbarProps.baseContainerSize,
    baseContentSize: ScrollbarProps.baseContentSize,
    topPocketSize: ScrollbarProps.topPocketSize,
    bottomPocketSize: ScrollbarProps.bottomPocketSize,
    scrollableOffset: ScrollbarProps.scrollableOffset,
    scrollByThumb: ScrollbarProps.scrollByThumb,
    isScrollableHovered: ScrollbarProps.isScrollableHovered,
    bounceEnabled: ScrollbarProps.bounceEnabled,
    forceVisibility: ScrollbarProps.forceVisibility,
    defaultPocketState: ScrollbarProps.defaultPocketState,
    showScrollbar: ScrollbarProps.showScrollbar,
    scrollByContent: ScrollbarProps.scrollByContent,
    useKeyboard: ScrollbarProps.useKeyboard,
    pullDownEnabled: ScrollableProps.pullDownEnabled,
    reachBottomEnabled: ScrollableProps.reachBottomEnabled,
    forceGeneratePockets: ScrollableSimulatedProps.forceGeneratePockets,
    needScrollViewContentWrapper: ScrollbarProps.needScrollViewContentWrapper,
    needScrollViewLoadPanel: ScrollbarProps.needScrollViewLoadPanel,
    defaultContentTranslateOffset: ScrollableSimulatedProps.contentTranslateOffset
};
import {
    createRef as infernoCreateRef
} from "inferno";
export class Scrollbar extends InfernoComponent {
    constructor(props) {
        super(props);
        this._currentState = null;
        this.location = 0;
        this.thumbScrolling = false;
        this.crossThumbScrolling = false;
        this.scrollbarRef = infernoCreateRef();
        this.scrollRef = infernoCreateRef();
        this.state = {
            showOnScrollByWheel: void 0,
            hovered: false,
            expanded: false,
            visibility: false,
            scrollLocation: 0,
            boundaryOffset: 0,
            maxOffset: 0,
            minOffset: 0,
            minLimit: 0,
            pocketState: void 0 !== this.props.pocketState ? this.props.pocketState : this.props.defaultPocketState,
            contentTranslateOffset: void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : this.props.defaultContentTranslateOffset
        };
        this.updateBoundaryOffset = this.updateBoundaryOffset.bind(this);
        this.updateMinOffset = this.updateMinOffset.bind(this);
        this.pointerDownEffect = this.pointerDownEffect.bind(this);
        this.pointerUpEffect = this.pointerUpEffect.bind(this);
        this.isThumb = this.isThumb.bind(this);
        this.isScrollbar = this.isScrollbar.bind(this);
        this.validateEvent = this.validateEvent.bind(this);
        this.reachedMin = this.reachedMin.bind(this);
        this.reachedMax = this.reachedMax.bind(this);
        this.getLocation = this.getLocation.bind(this);
        this.getScrollLocation = this.getScrollLocation.bind(this);
        this.setLocation = this.setLocation.bind(this);
        this.inBounds = this.inBounds.bind(this);
        this.boundLocation = this.boundLocation.bind(this);
        this.getMinOffset = this.getMinOffset.bind(this);
        this.getMaxOffset = this.getMaxOffset.bind(this);
        this.initHandler = this.initHandler.bind(this);
        this.startHandler = this.startHandler.bind(this);
        this.moveHandler = this.moveHandler.bind(this);
        this.hide = this.hide.bind(this);
        this.endHandler = this.endHandler.bind(this);
        this.stopHandler = this.stopHandler.bind(this);
        this.scrollByHandler = this.scrollByHandler.bind(this);
        this.scrollComplete = this.scrollComplete.bind(this);
        this.pullDownRefreshing = this.pullDownRefreshing.bind(this);
        this.reachBottomLoading = this.reachBottomLoading.bind(this);
        this.executePullDown = this.executePullDown.bind(this);
        this.executeReachBottom = this.executeReachBottom.bind(this);
        this.scrollToBounds = this.scrollToBounds.bind(this);
        this.stopComplete = this.stopComplete.bind(this);
        this.resetThumbScrolling = this.resetThumbScrolling.bind(this);
        this.scrollBy = this.scrollBy.bind(this);
        this.stopScrolling = this.stopScrolling.bind(this);
        this.prepareThumbScrolling = this.prepareThumbScrolling.bind(this);
        this.moveToMouseLocation = this.moveToMouseLocation.bind(this);
        this.scrollStep = this.scrollStep.bind(this);
        this.suppressBounce = this.suppressBounce.bind(this);
        this.moveScrollbar = this.moveScrollbar.bind(this);
        this.updateContent = this.updateContent.bind(this);
        this.releaseHandler = this.releaseHandler.bind(this);
        this.release = this.release.bind(this);
        this.stateReleased = this.stateReleased.bind(this);
        this.setPocketState = this.setPocketState.bind(this);
        this.isPullDown = this.isPullDown.bind(this);
        this.isReachBottom = this.isReachBottom.bind(this);
        this.containerToContentRatio = this.containerToContentRatio.bind(this);
        this.baseContainerToContentRatio = this.baseContainerToContentRatio.bind(this);
        this.expand = this.expand.bind(this);
        this.collapse = this.collapse.bind(this);
        this.onHoverStart = this.onHoverStart.bind(this);
        this.onHoverEnd = this.onHoverEnd.bind(this)
    }
    createEffects() {
        return [new InfernoEffect(this.updateBoundaryOffset, [this.props.forceGeneratePockets, this.scrollLocation, this.props.pullDownEnabled, this.props.topPocketSize, this.boundaryOffset]), new InfernoEffect(this.updateMinOffset, [this.props.forceGeneratePockets, this.props.contentSize, this.props.reachBottomEnabled, this.props.bottomPocketSize, this.props.pullDownEnabled, this.props.topPocketSize, this.props.containerSize]), new InfernoEffect(this.pointerDownEffect, []), new InfernoEffect(this.pointerUpEffect, []), new InfernoEffect(this.updateContent, [this.scrollLocation, this.minOffset, this.props.forceGeneratePockets, this.props.pullDownEnabled, this.props.topPocketSize, this.props.contentPositionChange, this.props.direction, this.props.scaleRatio, this.props.contentTranslateOffsetChange])]
    }
    updateEffects() {
        var _this$_effects$, _this$_effects$2, _this$_effects$3, _this$_effects$4, _this$_effects$5;
        null === (_this$_effects$ = this._effects[0]) || void 0 === _this$_effects$ ? void 0 : _this$_effects$.update([this.props.forceGeneratePockets, this.scrollLocation, this.props.pullDownEnabled, this.props.topPocketSize, this.boundaryOffset]);
        null === (_this$_effects$2 = this._effects[1]) || void 0 === _this$_effects$2 ? void 0 : _this$_effects$2.update([this.props.forceGeneratePockets, this.props.contentSize, this.props.reachBottomEnabled, this.props.bottomPocketSize, this.props.pullDownEnabled, this.props.topPocketSize, this.props.containerSize]);
        null === (_this$_effects$3 = this._effects[2]) || void 0 === _this$_effects$3 ? void 0 : _this$_effects$3.update([]);
        null === (_this$_effects$4 = this._effects[3]) || void 0 === _this$_effects$4 ? void 0 : _this$_effects$4.update([]);
        null === (_this$_effects$5 = this._effects[4]) || void 0 === _this$_effects$5 ? void 0 : _this$_effects$5.update([this.scrollLocation, this.minOffset, this.props.forceGeneratePockets, this.props.pullDownEnabled, this.props.topPocketSize, this.props.contentPositionChange, this.props.direction, this.props.scaleRatio, this.props.contentTranslateOffsetChange])
    }
    get showOnScrollByWheel() {
        var state = this._currentState || this.state;
        return state.showOnScrollByWheel
    }
    set_showOnScrollByWheel(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                showOnScrollByWheel: newValue
            }
        })
    }
    get hovered() {
        var state = this._currentState || this.state;
        return state.hovered
    }
    set_hovered(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                hovered: newValue
            }
        })
    }
    get expanded() {
        var state = this._currentState || this.state;
        return state.expanded
    }
    set_expanded(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                expanded: newValue
            }
        })
    }
    get visibility() {
        var state = this._currentState || this.state;
        return state.visibility
    }
    set_visibility(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                visibility: newValue
            }
        })
    }
    get scrollLocation() {
        var state = this._currentState || this.state;
        return state.scrollLocation
    }
    set_scrollLocation(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                scrollLocation: newValue
            }
        })
    }
    get boundaryOffset() {
        var state = this._currentState || this.state;
        return state.boundaryOffset
    }
    set_boundaryOffset(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                boundaryOffset: newValue
            }
        })
    }
    get maxOffset() {
        var state = this._currentState || this.state;
        return state.maxOffset
    }
    set_maxOffset(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                maxOffset: newValue
            }
        })
    }
    get minOffset() {
        var state = this._currentState || this.state;
        return state.minOffset
    }
    set_minOffset(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                minOffset: newValue
            }
        })
    }
    get minLimit() {
        var state = this._currentState || this.state;
        return state.minLimit
    }
    set_minLimit(value) {
        this.setState(state => {
            this._currentState = state;
            var newValue = value();
            this._currentState = null;
            return {
                minLimit: newValue
            }
        })
    }
    get __state_pocketState() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.pocketState ? this.props.pocketState : state.pocketState
    }
    set_pocketState(value) {
        this.setState(state => {
            var _this$props$pocketSta, _this$props;
            this._currentState = state;
            var newValue = value();
            null === (_this$props$pocketSta = (_this$props = this.props).pocketStateChange) || void 0 === _this$props$pocketSta ? void 0 : _this$props$pocketSta.call(_this$props, newValue);
            this._currentState = null;
            return {
                pocketState: newValue
            }
        })
    }
    get __state_contentTranslateOffset() {
        var state = this._currentState || this.state;
        return void 0 !== this.props.contentTranslateOffset ? this.props.contentTranslateOffset : state.contentTranslateOffset
    }
    set_contentTranslateOffset(value) {
        this.setState(state => {
            var _this$props$contentTr, _this$props2;
            this._currentState = state;
            var newValue = value();
            null === (_this$props$contentTr = (_this$props2 = this.props).contentTranslateOffsetChange) || void 0 === _this$props$contentTr ? void 0 : _this$props$contentTr.call(_this$props2, newValue);
            this._currentState = null;
            return {
                contentTranslateOffset: newValue
            }
        })
    }
    updateBoundaryOffset() {
        if (this.props.forceGeneratePockets) {
            this.set_boundaryOffset(() => this.scrollLocation - this.topPocketSize);
            this.set_maxOffset(() => this.boundaryOffset > 0 ? this.topPocketSize : 0)
        }
    }
    updateMinOffset() {
        if (this.props.forceGeneratePockets) {
            this.set_minOffset(() => -Math.max(this.bottomBoundaryOffset + this.bottomPocketSize, 0))
        } else {
            this.set_minOffset(() => -Math.max(this.bottomBoundaryOffset, 0))
        }
    }
    pointerDownEffect() {
        dxPointerDown.on(this.scrollRef.current, () => {
            this.expand()
        }, {
            namespace: "dxScrollbar"
        });
        return () => dxPointerDown.off(this.scrollRef.current, {
            namespace: "dxScrollbar"
        })
    }
    pointerUpEffect() {
        dxPointerUp.on(domAdapter.getDocument(), () => {
            this.collapse()
        }, {
            namespace: "dxScrollbar"
        });
        return () => dxPointerUp.off(this.scrollRef.current, {
            namespace: "dxScrollbar"
        })
    }
    updateContent() {
        var _this$props$contentPo, _this$props3, _this$props$contentTr2, _this$props4;
        var location = this.scrollLocation;
        var currentTranslateOffset;
        if (location > 0) {
            currentTranslateOffset = location
        } else if (location <= this.minOffset) {
            currentTranslateOffset = location - this.minOffset
        } else {
            currentTranslateOffset = location % 1
        }
        if (this.props.forceGeneratePockets && this.props.pullDownEnabled) {
            currentTranslateOffset -= this.props.topPocketSize
        }
        null === (_this$props$contentPo = (_this$props3 = this.props).contentPositionChange) || void 0 === _this$props$contentPo ? void 0 : _this$props$contentPo.call(_this$props3, this.fullScrollProp, location, this.props.scaleRatio);
        null === (_this$props$contentTr2 = (_this$props4 = this.props).contentTranslateOffsetChange) || void 0 === _this$props$contentTr2 ? void 0 : _this$props$contentTr2.call(_this$props4, {
            [this.scrollProp]: currentTranslateOffset
        })
    }
    get scrollTranslateOffset() {
        return _extends({}, {
            left: 0,
            top: 0
        }, {
            [this.scrollProp]: this.scrollPosition
        })
    }
    get scrollPosition() {
        return -this.scrollLocation * this.thumbRatio
    }
    get bottomBoundaryOffset() {
        return this.contentSize - this.props.containerSize
    }
    get axis() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "x" : "y"
    }
    get scrollProp() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "left" : "top"
    }
    get fullScrollProp() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "scrollLeft" : "scrollTop"
    }
    hide() {
        this.set_visibility(() => false);
        if (isDefined(this.showOnScrollByWheel) && "onScroll" === this.props.showScrollbar) {
            setTimeout(() => {
                this.set_showOnScrollByWheel(() => {})
            }, HIDE_SCROLLBAR_TIMEOUT)
        }
    }
    pullDownRefreshing() {
        this.setPocketState(TopPocketState.STATE_REFRESHING);
        this.executePullDown()
    }
    reachBottomLoading() {
        this.executeReachBottom()
    }
    executePullDown() {
        var _this$props$onPullDow, _this$props5;
        null === (_this$props$onPullDow = (_this$props5 = this.props).onPullDown) || void 0 === _this$props$onPullDow ? void 0 : _this$props$onPullDow.call(_this$props5)
    }
    executeReachBottom() {
        var _this$props$onReachBo, _this$props6;
        null === (_this$props$onReachBo = (_this$props6 = this.props).onReachBottom) || void 0 === _this$props$onReachBo ? void 0 : _this$props$onReachBo.call(_this$props6)
    }
    scrollToBounds() {
        var _this$props$onAnimato, _this$props7;
        if (this.inBounds()) {
            this.hide();
            return
        }
        null === (_this$props$onAnimato = (_this$props7 = this.props).onAnimatorStart) || void 0 === _this$props$onAnimato ? void 0 : _this$props$onAnimato.call(_this$props7, "bounce")
    }
    resetThumbScrolling() {
        this.thumbScrolling = false;
        this.crossThumbScrolling = false
    }
    scrollBy(delta) {
        var distance = delta[this.axis];
        if (!this.inBounds()) {
            distance *= OUT_BOUNDS_ACCELERATION
        }
        this.scrollStep(distance)
    }
    stopScrolling() {
        var _this$props$onAnimato2, _this$props8;
        this.hide();
        null === (_this$props$onAnimato2 = (_this$props8 = this.props).onAnimatorCancel) || void 0 === _this$props$onAnimato2 ? void 0 : _this$props$onAnimato2.call(_this$props8)
    }
    prepareThumbScrolling(e, currentCrossThumbScrolling) {
        if (isDxMouseWheelEvent(e.originalEvent)) {
            if ("onScroll" === this.props.showScrollbar) {
                this.set_showOnScrollByWheel(() => true)
            }
            return
        }
        var {
            target: target
        } = e.originalEvent;
        var scrollbarClicked = this.props.scrollByThumb && this.isScrollbar(target);
        if (scrollbarClicked) {
            this.moveToMouseLocation(e)
        }
        var currentThumbScrolling = scrollbarClicked || this.props.scrollByThumb && this.isThumb(target);
        this.thumbScrolling = currentThumbScrolling;
        this.crossThumbScrolling = !currentThumbScrolling && currentCrossThumbScrolling;
        if (currentThumbScrolling) {
            this.expand()
        }
    }
    moveToMouseLocation(e) {
        var mouseLocation = e["page".concat(this.axis.toUpperCase())] - this.props.scrollableOffset;
        var location = this.scrollLocation + mouseLocation / this.containerToContentRatio() - this.props.containerSize / 2;
        this.scrollStep(-Math.round(location))
    }
    suppressBounce() {
        if (this.props.bounceEnabled || this.inBounds()) {
            return
        }
        this.setLocation(this.boundLocation())
    }
    release() {
        this.stateReleased();
        this.scrollComplete()
    }
    stateReleased() {
        var _this$props$onRelease, _this$props9;
        this.setPocketState(TopPocketState.STATE_RELEASED);
        null === (_this$props$onRelease = (_this$props9 = this.props).onRelease) || void 0 === _this$props$onRelease ? void 0 : _this$props$onRelease.call(_this$props9)
    }
    setPocketState(state) {
        var _this$props$pocketSta2, _this$props10;
        null === (_this$props$pocketSta2 = (_this$props10 = this.props).pocketStateChange) || void 0 === _this$props$pocketSta2 ? void 0 : _this$props$pocketSta2.call(_this$props10, state)
    }
    isPullDown() {
        return this.props.pullDownEnabled && this.props.bounceEnabled && this.boundaryOffset >= 0
    }
    isReachBottom() {
        return this.props.reachBottomEnabled && this.scrollLocation - this.minOffset - this.bottomPocketSize <= .5
    }
    get thumbSize() {
        var {
            containerSize: containerSize,
            scaleRatio: scaleRatio
        } = this.props;
        var size = Math.round(Math.max(Math.round(containerSize * this.containerToContentRatio()), THUMB_MIN_SIZE));
        return size / scaleRatio
    }
    get thumbRatio() {
        var {
            containerSize: containerSize,
            scaleRatio: scaleRatio
        } = this.props;
        if (this.contentSize) {
            return (containerSize - this.thumbSize) / (scaleRatio * (this.contentSize - containerSize))
        }
        return 1
    }
    get contentSize() {
        if (this.props.contentSize) {
            return this.props.contentSize - this.bottomPocketSize - this.topPocketSize
        }
        return 0
    }
    containerToContentRatio() {
        return this.contentSize ? this.props.containerSize / this.contentSize : this.props.containerSize
    }
    baseContainerToContentRatio() {
        var {
            baseContainerSize: baseContainerSize,
            baseContentSize: baseContentSize
        } = this.props;
        return baseContentSize ? baseContainerSize / baseContentSize : baseContainerSize
    }
    get dimension() {
        return this.props.direction === DIRECTION_HORIZONTAL ? "width" : "height"
    }
    expand() {
        this.set_expanded(() => true)
    }
    collapse() {
        this.set_expanded(() => false)
    }
    onHoverStart() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_hovered(() => true)
        }
    }
    onHoverEnd() {
        if ("onHover" === this.props.showScrollbar) {
            this.set_hovered(() => false)
        }
    }
    get topPocketSize() {
        if (this.props.pullDownEnabled) {
            return this.props.topPocketSize
        }
        return 0
    }
    get bottomPocketSize() {
        if (this.props.reachBottomEnabled) {
            return this.props.bottomPocketSize
        }
        return 0
    }
    get cssClasses() {
        var {
            direction: direction
        } = this.props;
        var classesMap = {
            [SCROLLABLE_SCROLLBAR_CLASS]: true,
            ["dx-scrollbar-".concat(direction)]: true,
            [SCROLLABLE_SCROLLBAR_ACTIVE_CLASS]: !!this.expanded,
            [HOVER_ENABLED_STATE]: !!this.hoverStateEnabled
        };
        return combineClasses(classesMap)
    }
    get scrollStyles() {
        return {
            [this.dimension]: "".concat(this.thumbSize || THUMB_MIN_SIZE, "px"),
            transform: "never" === this.props.showScrollbar ? "none" : "translate(".concat(this.scrollTranslateOffset.left, "px, ").concat(this.scrollTranslateOffset.top, "px)")
        }
    }
    get scrollClasses() {
        return combineClasses({
            [SCROLLABLE_SCROLL_CLASS]: true,
            "dx-state-invisible": !this.visible
        })
    }
    get isVisible() {
        return "never" !== this.props.showScrollbar && this.baseContainerToContentRatio() < 1
    }
    get visible() {
        var {
            forceVisibility: forceVisibility,
            showScrollbar: showScrollbar
        } = this.props;
        if (!this.isVisible) {
            return false
        }
        if ("onHover" === showScrollbar) {
            return this.visibility || this.props.isScrollableHovered || this.hovered
        }
        if ("always" === showScrollbar) {
            return true
        }
        return forceVisibility || this.visibility || !!this.showOnScrollByWheel
    }
    get hoverStateEnabled() {
        var {
            scrollByThumb: scrollByThumb,
            showScrollbar: showScrollbar
        } = this.props;
        return ("onHover" === showScrollbar || "always" === showScrollbar) && scrollByThumb
    }
    get restAttributes() {
        var _this$props$pocketSta3 = _extends({}, this.props, {
                pocketState: this.__state_pocketState,
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            restProps = _objectWithoutPropertiesLoose(_this$props$pocketSta3, ["activeStateEnabled", "baseContainerSize", "baseContentSize", "bottomPocketSize", "bounceEnabled", "children", "classes", "containerSize", "contentPositionChange", "contentSize", "contentTranslateOffset", "contentTranslateOffsetChange", "defaultContentTranslateOffset", "defaultPocketState", "direction", "expandable", "forceGeneratePockets", "forceVisibility", "hoverStateEnabled", "isScrollableHovered", "needScrollViewContentWrapper", "needScrollViewLoadPanel", "onAnimatorCancel", "onAnimatorStart", "onPullDown", "onReachBottom", "onRelease", "onScroll", "onUpdated", "pocketState", "pocketStateChange", "pullDownEnabled", "reachBottomEnabled", "scaleRatio", "scrollByContent", "scrollByThumb", "scrollableOffset", "showScrollbar", "topPocketSize", "useKeyboard"]);
        return restProps
    }
    isThumb(element) {
        var _this$scrollbarRef$cu, _this$scrollbarRef$cu2;
        return (null === (_this$scrollbarRef$cu = this.scrollbarRef.current) || void 0 === _this$scrollbarRef$cu ? void 0 : _this$scrollbarRef$cu.querySelector(".".concat(SCROLLABLE_SCROLL_CLASS))) === element || (null === (_this$scrollbarRef$cu2 = this.scrollbarRef.current) || void 0 === _this$scrollbarRef$cu2 ? void 0 : _this$scrollbarRef$cu2.querySelector(".".concat(SCROLLABLE_SCROLL_CONTENT_CLASS))) === element
    }
    isScrollbar(element) {
        return element === this.scrollbarRef.current
    }
    validateEvent(e) {
        var {
            target: target
        } = e.originalEvent;
        return this.isThumb(target) || this.isScrollbar(target)
    }
    reachedMin() {
        return this.getLocation() <= this.minOffset
    }
    reachedMax() {
        return this.getLocation() >= this.maxOffset
    }
    getLocation() {
        return this.location
    }
    getScrollLocation() {
        return this.scrollLocation
    }
    setLocation(value) {
        this.location = value
    }
    inBounds() {
        return this.boundLocation() === this.getLocation()
    }
    boundLocation(value) {
        var currentLocation = isDefined(value) ? value : this.getLocation();
        return Math.max(Math.min(currentLocation, this.maxOffset), this.minOffset)
    }
    getMinOffset() {
        return this.minOffset
    }
    getMaxOffset() {
        return this.maxOffset
    }
    initHandler(e, crossThumbScrolling) {
        this.stopScrolling();
        this.prepareThumbScrolling(e, crossThumbScrolling)
    }
    startHandler() {
        this.set_visibility(() => true)
    }
    moveHandler(delta) {
        if (this.crossThumbScrolling) {
            return
        }
        var distance = delta;
        if (this.thumbScrolling) {
            distance[this.axis] = -Math.round(distance[this.axis] / this.containerToContentRatio())
        }
        this.scrollBy(distance)
    }
    endHandler(velocity) {
        var _this$props$onAnimato3, _this$props11;
        null === (_this$props$onAnimato3 = (_this$props11 = this.props).onAnimatorStart) || void 0 === _this$props$onAnimato3 ? void 0 : _this$props$onAnimato3.call(_this$props11, "inertia", velocity[this.axis], this.thumbScrolling, this.crossThumbScrolling);
        this.resetThumbScrolling()
    }
    stopHandler() {
        this.hide();
        if (this.thumbScrolling) {
            this.scrollComplete()
        } else {
            this.scrollToBounds()
        }
        this.resetThumbScrolling()
    }
    scrollByHandler(delta) {
        this.scrollBy(delta);
        this.scrollComplete()
    }
    scrollComplete() {
        if (this.props.forceGeneratePockets) {
            if (this.inBounds()) {
                if (this.__state_pocketState === TopPocketState.STATE_READY) {
                    this.pullDownRefreshing();
                    return
                }
                if (this.__state_pocketState === TopPocketState.STATE_LOADING) {
                    this.reachBottomLoading();
                    return
                }
            }
        }
        this.scrollToBounds()
    }
    stopComplete() {}
    scrollStep(delta) {
        this.setLocation(this.scrollLocation + delta);
        this.suppressBounce();
        this.moveScrollbar()
    }
    moveScrollbar(location) {
        var currentLocation = isDefined(location) ? location * this.props.scaleRatio : this.getLocation();
        this.setLocation(currentLocation);
        this.set_scrollLocation(() => currentLocation);
        if (this.props.forceGeneratePockets) {
            if (this.isPullDown()) {
                if (this.__state_pocketState !== TopPocketState.STATE_READY) {
                    this.setPocketState(TopPocketState.STATE_READY)
                }
            } else if (this.isReachBottom()) {
                if (this.__state_pocketState !== TopPocketState.STATE_LOADING) {
                    this.setPocketState(TopPocketState.STATE_LOADING)
                }
            } else if (this.__state_pocketState !== TopPocketState.STATE_RELEASED) {
                this.stateReleased()
            }
        }
    }
    releaseHandler() {
        this.release()
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props, {
                pocketState: this.__state_pocketState,
                contentTranslateOffset: this.__state_contentTranslateOffset
            }),
            showOnScrollByWheel: this.showOnScrollByWheel,
            hovered: this.hovered,
            expanded: this.expanded,
            visibility: this.visibility,
            scrollLocation: this.scrollLocation,
            boundaryOffset: this.boundaryOffset,
            maxOffset: this.maxOffset,
            minOffset: this.minOffset,
            minLimit: this.minLimit,
            scrollbarRef: this.scrollbarRef,
            scrollRef: this.scrollRef,
            scrollTranslateOffset: this.scrollTranslateOffset,
            scrollPosition: this.scrollPosition,
            bottomBoundaryOffset: this.bottomBoundaryOffset,
            axis: this.axis,
            scrollProp: this.scrollProp,
            fullScrollProp: this.fullScrollProp,
            hide: this.hide,
            pullDownRefreshing: this.pullDownRefreshing,
            reachBottomLoading: this.reachBottomLoading,
            executePullDown: this.executePullDown,
            executeReachBottom: this.executeReachBottom,
            scrollToBounds: this.scrollToBounds,
            resetThumbScrolling: this.resetThumbScrolling,
            scrollBy: this.scrollBy,
            stopScrolling: this.stopScrolling,
            prepareThumbScrolling: this.prepareThumbScrolling,
            moveToMouseLocation: this.moveToMouseLocation,
            suppressBounce: this.suppressBounce,
            release: this.release,
            stateReleased: this.stateReleased,
            setPocketState: this.setPocketState,
            isPullDown: this.isPullDown,
            isReachBottom: this.isReachBottom,
            thumbSize: this.thumbSize,
            thumbRatio: this.thumbRatio,
            contentSize: this.contentSize,
            containerToContentRatio: this.containerToContentRatio,
            baseContainerToContentRatio: this.baseContainerToContentRatio,
            dimension: this.dimension,
            expand: this.expand,
            collapse: this.collapse,
            onHoverStart: this.onHoverStart,
            onHoverEnd: this.onHoverEnd,
            topPocketSize: this.topPocketSize,
            bottomPocketSize: this.bottomPocketSize,
            cssClasses: this.cssClasses,
            scrollStyles: this.scrollStyles,
            scrollClasses: this.scrollClasses,
            isVisible: this.isVisible,
            visible: this.visible,
            hoverStateEnabled: this.hoverStateEnabled,
            restAttributes: this.restAttributes
        })
    }
}
Scrollbar.defaultProps = _extends({}, ScrollbarPropsType);
