/**
* DevExtreme (ui/action_sheet.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import DataSource, {
    DataSourceOptions
} from '../data/data_source';

import {
    TEvent,
    Cancelable,
    ComponentEvent,
    ComponentNativeEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo,
    ItemInfo
} from '../events/index';

import CollectionWidget, {
    CollectionWidgetItem,
    CollectionWidgetOptions
} from './collection/ui.collection_widget.base';

export type CancelClickEvent = ComponentEvent<dxActionSheet> & Cancelable;

export type ContentReadyEvent = ComponentEvent<dxActionSheet>;

export type DisposingEvent = ComponentEvent<dxActionSheet>;

export type InitializedEvent = ComponentInitializedEvent<dxActionSheet>;

export type ItemClickEvent = ComponentNativeEvent<dxActionSheet> & ItemInfo;

export type ItemContextMenuEvent = ComponentNativeEvent<dxActionSheet> & ItemInfo;

export type ItemHoldEvent = ComponentNativeEvent<dxActionSheet> & ItemInfo;

export type ItemRenderedEvent = ComponentNativeEvent<dxActionSheet> & ItemInfo;

export type OptionChangedEvent = ComponentEvent<dxActionSheet> & ChangedOptionInfo;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxActionSheetOptions extends CollectionWidgetOptions<dxActionSheet> {
    /**
     * The text displayed in the button that closes the action sheet.
     */
    cancelText?: string;
    /**
     * 
     */
    dataSource?: string | Array<string | dxActionSheetItem | any> | DataSource | DataSourceOptions;
    /**
     * 
     */
    items?: Array<string | dxActionSheetItem | any>;
    /**
     * A function that is executed when the Cancel button is clicked or tapped.
     */
    onCancelClick?: ((e: CancelClickEvent) => void) | string;
    /**
     * Specifies whether or not to display the Cancel button in action sheet.
     */
    showCancelButton?: boolean;
    /**
     * A Boolean value specifying whether or not the title of the action sheet is visible.
     */
    showTitle?: boolean;
    /**
     * Specifies the element the action sheet popover points at. Applies only if usePopover is true.
     */
    target?: string | TElement;
    /**
     * The title of the action sheet.
     */
    title?: string;
    /**
     * Specifies whether or not to show the action sheet within a Popover UI component.
     */
    usePopover?: boolean;
    /**
     * A Boolean value specifying whether or not the ActionSheet UI component is visible.
     */
    visible?: boolean;
}
/**
 * The ActionSheet UI component is a sheet containing a set of buttons located one under the other. These buttons usually represent several choices relating to a single task.
 */
export default class dxActionSheet extends CollectionWidget {
    constructor(element: TElement, options?: dxActionSheetOptions)
    /**
     * Hides the UI component.
     */
    hide(): TPromise<void>;
    /**
     * Shows the UI component.
     */
    show(): TPromise<void>;
    /**
     * Shows or hides the UI component depending on the argument.
     */
    toggle(showing: boolean): TPromise<void>;
}

/**
 * 
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxActionSheetItem extends CollectionWidgetItem {
    /**
     * Specifies the icon to be displayed on the action sheet button.
     */
    icon?: string;
    /**
     * A handler for the click event raised for the button representing the given action sheet button.
     */
    onClick?: ((e: { component?: dxActionSheet, element?: TElement, model?: any, event?: TEvent }) => void) | string;
    /**
     * Specifies the type of the button that represents an action sheet item.
     */
    type?: 'back' | 'danger' | 'default' | 'normal' | 'success';
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxActionSheetOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxActionSheetOptions;
