/**
* DevExtreme (ui/autocomplete.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    ComponentEvent,
    ComponentNativeEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo,
    ItemInfo
} from '../events/index';

import {
    SelectionChangedInfo,
} from './collection/ui.collection_widget.base';

import dxDropDownList, {
    dxDropDownListOptions
} from './drop_down_editor/ui.drop_down_list';

import {
    DropDownButtonTemplateDataModel
} from './drop_down_editor/ui.drop_down_editor';

import {
    ValueChangedInfo
} from './editor/editor';

export type ChangeEvent = ComponentNativeEvent<dxAutocomplete>;

export type ClosedEvent = ComponentEvent<dxAutocomplete>;

export type ContentReadyEvent = ComponentEvent<dxAutocomplete>;

export type CopyEvent = ComponentNativeEvent<dxAutocomplete>;

export type CutEvent = ComponentNativeEvent<dxAutocomplete>;

export type DisposingEvent = ComponentEvent<dxAutocomplete>;

export type EnterKeyEvent = ComponentNativeEvent<dxAutocomplete>;

export type FocusInEvent = ComponentNativeEvent<dxAutocomplete>;

export type FocusOutEvent = ComponentNativeEvent<dxAutocomplete>;

export type InitializedEvent = ComponentInitializedEvent<dxAutocomplete>;

export type InputEvent = ComponentNativeEvent<dxAutocomplete>;

export type ItemClickEvent = ComponentNativeEvent<dxAutocomplete> & ItemInfo;

export type KeyDownEvent = ComponentNativeEvent<dxAutocomplete>;

export type KeyPressEvent = ComponentNativeEvent<dxAutocomplete>;

export type KeyUpEvent = ComponentNativeEvent<dxAutocomplete>;

export type OpenedEvent = ComponentEvent<dxAutocomplete>;

export type OptionChangedEvent = ComponentEvent<dxAutocomplete> & ChangedOptionInfo;

export type PasteEvent = ComponentNativeEvent<dxAutocomplete>;

export type SelectionChangedEvent = ComponentEvent<dxAutocomplete> & SelectionChangedInfo;

export type ValueChangedEvent = ComponentNativeEvent<dxAutocomplete> & ValueChangedInfo;

export type DropDownButtonTemplateData = DropDownButtonTemplateDataModel;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxAutocompleteOptions extends dxDropDownListOptions<dxAutocomplete> {
    /**
     * Specifies the maximum count of items displayed by the UI component.
     */
    maxItemCount?: number;
    /**
     * The minimum number of characters that must be entered into the text box to begin a search.
     */
    minSearchLength?: number;
    /**
     * 
     */
    showDropDownButton?: boolean;
    /**
     * Specifies the current value displayed by the UI component.
     */
    value?: string;
}
/**
 * The Autocomplete UI component is a textbox that provides suggestions while a user types into it.
 */
export default class dxAutocomplete extends dxDropDownList {
    constructor(element: TElement, options?: dxAutocompleteOptions)
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxAutocompleteOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxAutocompleteOptions;
