/**
* DevExtreme (ui/color_box.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    ComponentEvent,
    ComponentNativeEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import {
    template
} from '../core/templates/template';

import dxDropDownEditor, {
    dxDropDownEditorOptions,
    DropDownButtonTemplateDataModel
} from './drop_down_editor/ui.drop_down_editor';

import {
    ValueChangedInfo
} from './editor/editor';

export type ChangeEvent = ComponentNativeEvent<dxColorBox>;

export type ClosedEvent = ComponentEvent<dxColorBox>;

export type CopyEvent = ComponentNativeEvent<dxColorBox>;

export type CutEvent = ComponentNativeEvent<dxColorBox>;

export type DisposingEvent = ComponentEvent<dxColorBox>;

export type EnterKeyEvent = ComponentNativeEvent<dxColorBox>;

export type FocusInEvent = ComponentNativeEvent<dxColorBox>;

export type FocusOutEvent = ComponentNativeEvent<dxColorBox>;

export type InitializedEvent = ComponentInitializedEvent<dxColorBox>;

export type InputEvent = ComponentNativeEvent<dxColorBox>;

export type KeyDownEvent = ComponentNativeEvent<dxColorBox>;

export type KeyPressEvent = ComponentNativeEvent<dxColorBox>;

export type KeyUpEvent = ComponentNativeEvent<dxColorBox>;

export type OpenedEvent = ComponentEvent<dxColorBox>;

export type OptionChangedEvent = ComponentEvent<dxColorBox> & ChangedOptionInfo;

export type PasteEvent = ComponentNativeEvent<dxColorBox>;

export type ValueChangedEvent = ComponentNativeEvent<dxColorBox> & ValueChangedInfo;

export type DropDownButtonTemplateData = DropDownButtonTemplateDataModel;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxColorBoxOptions extends dxDropDownEditorOptions<dxColorBox> {
    /**
     * Specifies the text displayed on the button that applies changes and closes the drop-down editor.
     */
    applyButtonText?: string;
    /**
     * 
     */
    applyValueMode?: 'instantly' | 'useButtons';
    /**
     * Specifies the text displayed on the button that cancels changes and closes the drop-down editor.
     */
    cancelButtonText?: string;
    /**
     * Specifies whether or not the UI component value includes the alpha channel component.
     */
    editAlphaChannel?: boolean;
    /**
     * Specifies a custom template for the input field. Must contain the TextBox UI component.
     */
    fieldTemplate?: template | ((value: string, fieldElement: TElement) => string | TElement);
    /**
     * Specifies the size of a step by which a handle is moved using a keyboard shortcut.
     */
    keyStep?: number;
    /**
     * 
     */
    value?: string;
}
/**
 * The ColorBox is a UI component that allows an end user to enter a color or pick it out from the drop-down editor.
 */
export default class dxColorBox extends dxDropDownEditor {
    constructor(element: TElement, options?: dxColorBoxOptions)
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxColorBoxOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxColorBoxOptions;
