/**
* DevExtreme (ui/context_menu.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    positionConfig
} from '../animation/position';

import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import DataSource, {
    DataSourceOptions
} from '../data/data_source';

import {
    TEvent,
    Cancelable,
    ComponentEvent,
    ComponentNativeEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo,
    ItemInfo
} from '../events/index';

import dxMenuBase, {
    dxMenuBaseOptions
} from './context_menu/ui.menu_base';

import {
    dxMenuBaseItem
} from './menu';

import {
    SelectionChangedInfo
} from './collection/ui.collection_widget.base';

export type ContentReadyEvent = ComponentEvent<dxContextMenu>;

export type DisposingEvent = ComponentEvent<dxContextMenu>;

export type HiddenEvent = ComponentEvent<dxContextMenu>;

export type HidingEvent = ComponentEvent<dxContextMenu> & Cancelable;

export type InitializedEvent = ComponentInitializedEvent<dxContextMenu>;

export type ItemClickEvent = ComponentNativeEvent<dxContextMenu> & ItemInfo;

export type ItemContextMenuEvent = ComponentNativeEvent<dxContextMenu> & ItemInfo;

export type ItemRenderedEvent = ComponentNativeEvent<dxContextMenu> & ItemInfo;

export type OptionChangedEvent = ComponentEvent<dxContextMenu> & ChangedOptionInfo;

export type PositioningEvent = ComponentNativeEvent<dxContextMenu> & {
    readonly position: positionConfig;
}

export type SelectionChangedEvent = ComponentEvent<dxContextMenu> & SelectionChangedInfo;

export type ShowingEvent = ComponentEvent<dxContextMenu> & Cancelable;

export type ShownEvent = ComponentEvent<dxContextMenu>;

export interface dxContextMenuOptions extends dxMenuBaseOptions<dxContextMenu> {
    /**
     * Specifies whether to close the UI component if a user clicks outside it.
     */
    closeOnOutsideClick?: boolean | ((event: TEvent) => boolean);
    /**
     * 
     */
    dataSource?: string | Array<dxContextMenuItem> | DataSource | DataSourceOptions;
    /**
     * 
     */
    items?: Array<dxContextMenuItem>;
    /**
     * A function that is executed after the ContextMenu is hidden.
     */
    onHidden?: ((e: HiddenEvent) => void);
    /**
     * A function that is executed before the ContextMenu is hidden.
     */
    onHiding?: ((e: HidingEvent) => void);
    /**
     * A function that is executed before the ContextMenu is positioned.
     */
    onPositioning?: ((e: PositioningEvent) => void);
    /**
     * A function that is executed before the ContextMenu is shown.
     */
    onShowing?: ((e: ShowingEvent) => void);
    /**
     * A function that is executed after the ContextMenu is shown.
     */
    onShown?: ((e: ShownEvent) => void);
    /**
     * An object defining UI component positioning properties.
     */
    position?: positionConfig;
    /**
     * Specifies properties for displaying the UI component.
     */
    showEvent?: {
      /**
       * The delay in milliseconds after which the UI component is displayed.
       */
      delay?: number,
      /**
       * Specifies the event names on which the UI component is shown.
       */
      name?: string
    } | string;
    /**
     * Specifies the direction at which submenus are displayed.
     */
    submenuDirection?: 'auto' | 'left' | 'right';
    /**
     * The target element associated with the context menu.
     */
    target?: string | TElement;
    /**
     * A Boolean value specifying whether or not the UI component is visible.
     */
    visible?: boolean;
}
/**
 * The ContextMenu UI component displays a single- or multi-level context menu. An end user invokes this menu by a right click or a long press.
 */
export default class dxContextMenu extends dxMenuBase {
    constructor(element: TElement, options?: dxContextMenuOptions)
    /**
     * Hides the UI component.
     */
    hide(): TPromise<void>;
    /**
     * Shows the UI component.
     */
    show(): TPromise<void>;
    /**
     * Shows or hides the UI component depending on the argument.
     */
    toggle(showing: boolean): TPromise<void>;
}

/**
 * 
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxContextMenuItem extends dxMenuBaseItem {
    /**
     * 
     */
    items?: Array<dxContextMenuItem>;
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxContextMenuOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxContextMenuOptions;
