/**
* DevExtreme (ui/defer_rendering.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    animationConfig
} from '../animation/fx';

import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import {
    ComponentEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import Widget, {
    WidgetOptions
} from './widget/ui.widget';

export type ContentReadyEvent = ComponentEvent<dxDeferRendering>;

export type DisposingEvent = ComponentEvent<dxDeferRendering>;

export type InitializedEvent = ComponentInitializedEvent<dxDeferRendering>;

export type OptionChangedEvent = ComponentEvent<dxDeferRendering> & ChangedOptionInfo;

export type RenderedEvent = ComponentEvent<dxDeferRendering>;

export type ShownEvent = ComponentEvent<dxDeferRendering>;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDeferRenderingOptions extends WidgetOptions<dxDeferRendering> {
    /**
     * Specifies the animation to be used to show the rendered content.
     */
    animation?: animationConfig;
    /**
     * A function that is executed when the content is rendered but not yet displayed.
     */
    onRendered?: ((e: { component?: dxDeferRendering, element?: TElement, model?: any }) => void);
    /**
     * A function that is executed when the content is displayed and animation is completed.
     */
    onShown?: ((e: { component?: dxDeferRendering, element?: TElement, model?: any }) => void);
    /**
     * Specifies when the UI component content is rendered.
     */
    renderWhen?: TPromise<void> | boolean;
    /**
     * Indicates if a load indicator should be shown until the UI component&apos;s content is rendered.
     */
    showLoadIndicator?: boolean;
    /**
     * Specifies a jQuery selector of items that should be rendered using a staggered animation.
     */
    staggerItemSelector?: string;
}
/**
 * The DeferRendering is a UI component that waits for its content to be ready before rendering it. While the content is getting ready, the DeferRendering displays a loading indicator.
 */
export default class dxDeferRendering extends Widget {
    constructor(element: TElement, options?: dxDeferRenderingOptions)
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxDeferRenderingOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxDeferRenderingOptions;
