/**
* DevExtreme (ui/drawer.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import {
    template
} from '../core/templates/template';

import {
    TEvent,
    ComponentEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import Widget, {
    WidgetOptions
} from './widget/ui.widget';

export type DisposingEvent = ComponentEvent<dxDrawer>;

export type InitializedEvent = ComponentInitializedEvent<dxDrawer>;

export type OptionChangedEvent = ComponentEvent<dxDrawer> & ChangedOptionInfo;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDrawerOptions extends WidgetOptions<dxDrawer> {
    /**
     * Specifies the duration of the drawer&apos;s opening and closing animation (in milliseconds). Applies only if animationEnabled is true.
     */
    animationDuration?: number;
    /**
     * Specifies whether to use an opening and closing animation.
     */
    animationEnabled?: boolean;
    /**
     * Specifies whether to close the drawer if a user clicks or taps the view area.
     */
    closeOnOutsideClick?: boolean | ((event: TEvent) => boolean);
    /**
     * Specifies the drawer&apos;s width or height (depending on the drawer&apos;s position) in the opened state.
     */
    maxSize?: number;
    /**
     * Specifies the drawer&apos;s width or height (depending on the drawer&apos;s position) in the closed state.
     */
    minSize?: number;
    /**
     * Specifies whether the drawer is opened.
     */
    opened?: boolean;
    /**
     * Specifies how the drawer interacts with the view in the opened state.
     */
    openedStateMode?: 'overlap' | 'shrink' | 'push';
    /**
     * Specifies the drawer&apos;s position in relation to the view.
     */
    position?: 'left' | 'right' | 'top' | 'bottom' | 'before' | 'after';
    /**
     * Specifies the drawer&apos;s reveal mode.
     */
    revealMode?: 'slide' | 'expand';
    /**
     * Specifies whether to shade the view when the drawer is opened.
     */
    shading?: boolean;
    /**
     * Specifies a CSS selector for the element in which the drawer should be rendered. Applies only when the openedStateMode is &apos;overlap&apos;.
     * @deprecated 
     */
    target?: string | TElement;
    /**
     * Specifies the drawer&apos;s content.
     */
    template?: template | ((Element: TElement) => any);
}
/**
 * The Drawer is a dismissible or permanently visible panel used for navigation in responsive web application layouts.
 */
export default class dxDrawer extends Widget {
    constructor(element: TElement, options?: dxDrawerOptions)
    /**
     * Gets the drawer&apos;s content.
     */
    content(): TElement;
    /**
     * Closes the drawer.
     */
    hide(): TPromise<void>;
    /**
     * Opens the drawer.
     */
    show(): TPromise<void>;
    /**
     * Opens or closes the drawer, reversing the current state.
     */
    toggle(): TPromise<void>;
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxDrawerOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxDrawerOptions;
