/**
* DevExtreme (ui/drop_down_box.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    template
} from '../core/templates/template';

import DataSource, {
    DataSourceOptions
} from '../data/data_source';

import {
    ComponentEvent,
    ComponentNativeEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import dxDropDownEditor, {
    dxDropDownEditorOptions,
    DropDownButtonTemplateDataModel
} from './drop_down_editor/ui.drop_down_editor';

import {
    ValueChangedInfo
} from './editor/editor';

import {
    DataExpressionMixinOptions
} from './editor/ui.data_expression';

export type ChangeEvent = ComponentNativeEvent<dxDropDownBox>;

export type ClosedEvent = ComponentEvent<dxDropDownBox>;

export type CopyEvent = ComponentNativeEvent<dxDropDownBox>;

export type CutEvent = ComponentNativeEvent<dxDropDownBox>;

export type DisposingEvent = ComponentEvent<dxDropDownBox>;

export type EnterKeyEvent = ComponentNativeEvent<dxDropDownBox>;

export type FocusInEvent = ComponentNativeEvent<dxDropDownBox>;

export type FocusOutEvent = ComponentNativeEvent<dxDropDownBox>;

export type InitializedEvent = ComponentInitializedEvent<dxDropDownBox>;

export type InputEvent = ComponentNativeEvent<dxDropDownBox>;

export type KeyDownEvent = ComponentNativeEvent<dxDropDownBox>;

export type KeyPressEvent = ComponentNativeEvent<dxDropDownBox>;

export type KeyUpEvent = ComponentNativeEvent<dxDropDownBox>;

export type OpenedEvent = ComponentEvent<dxDropDownBox>;

export type OptionChangedEvent = ComponentEvent<dxDropDownBox> & ChangedOptionInfo;

export type PasteEvent = ComponentNativeEvent<dxDropDownBox>;

export type ValueChangedEvent = ComponentNativeEvent<dxDropDownBox> & ValueChangedInfo;

export type ContentTemplateData = {
    component: dxDropDownBox;
    readonly value?: any;
}

export type DropDownButtonTemplateData = DropDownButtonTemplateDataModel;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxDropDownBoxOptions extends DataExpressionMixinOptions<dxDropDownBox>, dxDropDownEditorOptions<dxDropDownBox> {
    /**
     * Specifies whether the UI component allows a user to enter a custom value.
     */
    acceptCustomValue?: boolean;
    /**
     * Specifies a custom template for the drop-down content.
     */
    contentTemplate?: template | ((templateData: ContentTemplateData, contentElement: TElement) => string | TElement);
    /**
     * 
     */
    dataSource?: string | Array<any> | DataSource | DataSourceOptions;
    /**
     * Customizes text before it is displayed in the input field.
     */
    displayValueFormatter?: ((value: string | Array<any>) => string);
    /**
     * Specifies a custom template for the text field. Must contain the TextBox UI component.
     */
    fieldTemplate?: template | ((value: any, fieldElement: TElement) => string | TElement);
    /**
     * An array of items used to synchronize the DropDownBox with an embedded UI component.
     */
    items?: Array<any>;
    /**
     * 
     */
    openOnFieldClick?: boolean;
    /**
     * Specifies the DOM events after which the UI component&apos;s value should be updated.
     */
    valueChangeEvent?: string;
}
/**
 * The DropDownBox UI component consists of a text field, which displays the current value, and a drop-down field, which can contain any UI element.
 */
export default class dxDropDownBox extends dxDropDownEditor {
    constructor(element: TElement, options?: dxDropDownBoxOptions)
    getDataSource(): DataSource;
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxDropDownBoxOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxDropDownBoxOptions;
