/**
* DevExtreme (ui/editor/editor.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../../core/element';

import {
    ComponentNativeEvent
} from '../../events/index';

import Widget, {
    WidgetOptions
} from '../widget/ui.widget';

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface ValueChangedInfo {
    readonly previousValue?: any;
    readonly value?: any;
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface EditorOptions<T = Editor> extends WidgetOptions<T> {
    /**
     * Specifies or indicates whether the editor&apos;s value is valid.
     */
    isValid?: boolean;
    /**
     * A function that is executed after the UI component&apos;s value is changed.
     */
    onValueChanged?: ((e: ComponentNativeEvent<T> & ValueChangedInfo) => void);
    /**
     * Specifies whether the editor is read-only.
     */
    readOnly?: boolean;
    /**
     * Information on the broken validation rule. Contains the first item from the validationErrors array.
     */
    validationError?: any;
    /**
     * An array of the validation rules that failed.
     */
    validationErrors?: Array<any>;
    /**
     * Specifies how the message about the validation rules that are not satisfied by this editor&apos;s value is displayed.
     */
    validationMessageMode?: 'always' | 'auto';
    /**
     * Indicates or specifies the current validation status.
     */
    validationStatus?: 'valid' | 'invalid' | 'pending';
    /**
     * Specifies the UI component&apos;s value.
     */
    value?: any;
    /**
     * 
     */
    stylingMode?: 'outlined' | 'underlined' | 'filled';
}
/**
 * A base class for editors.
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export default class Editor extends Widget {
    constructor(element: TElement, options?: EditorOptions)
    /**
     * Resets the value property to the default value.
     */
    reset(): void;
}
