/**
* DevExtreme (ui/popover.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    animationConfig
} from '../animation/fx';

import {
    positionConfig
} from '../animation/position';

import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import {
    TEvent,
    Cancelable,
    ComponentEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import dxPopup, {
    dxPopupAnimation,
    dxPopupOptions,
    TitleRenderedInfo
} from './popup';

export type ContentReadyEvent = ComponentEvent<dxPopover>;

export type DisposingEvent = ComponentEvent<dxPopover>;

export type HidingEvent = ComponentEvent<dxPopover> & Cancelable;

export type HiddenEvent = ComponentEvent<dxPopover>;

export type InitializedEvent = ComponentInitializedEvent<dxPopover>;

export type OptionChangedEvent = ComponentEvent<dxPopover> & ChangedOptionInfo;

export type ShowingEvent = ComponentEvent<dxPopover>;

export type ShownEvent = ComponentEvent<dxPopover>;

export type TitleRenderedEvent = ComponentEvent<dxPopup> & TitleRenderedInfo;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPopoverOptions<T = dxPopover> extends dxPopupOptions<T> {
    /**
     * Configures UI component visibility animations. This object contains two fields: show and hide.
     */
    animation?: dxPopoverAnimation;
    /**
     * Specifies whether to close the UI component if a user clicks outside the popover window or outside the target element.
     */
    closeOnOutsideClick?: boolean | ((event: TEvent) => boolean);
    /**
     * Specifies the UI component&apos;s height.
     */
    height?: number | string | (() => number | string);
    /**
     * Specifies properties of popover hiding.
     */
    hideEvent?: {
      /**
       * The delay in milliseconds after which the UI component is hidden.
       */
      delay?: number,
      /**
       * Specifies the event names on which the UI component is hidden.
       */
      name?: string
    } | string;
    /**
     * An object defining UI component positioning properties.
     */
    position?: 'bottom' | 'left' | 'right' | 'top' | positionConfig;
    /**
     * 
     */
    shading?: boolean;
    /**
     * Specifies properties for displaying the UI component.
     */
    showEvent?: {
      /**
       * The delay in milliseconds after which the UI component is displayed.
       */
      delay?: number,
    /**
       * Specifies the event names on which the UI component is shown.
       */
      name?: string
    } | string;
    /**
     * A Boolean value specifying whether or not to display the title in the overlay window.
     */
    showTitle?: boolean;
    /**
     * The target element associated with the widget.
     */
    target?: string | TElement;
    /**
     * Specifies the UI component&apos;s width.
     */
    width?: number | string | (() => number | string);
}
/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPopoverAnimation extends dxPopupAnimation {
    /**
     * An object that defines the animation properties used when the UI component is being hidden.
     */
    hide?: animationConfig;
    /**
     * An object that defines the animation properties used when the UI component is being shown.
     */
    show?: animationConfig;
}
/**
 * The Popover is a UI component that shows notifications within a box with an arrow pointing to a specified UI element.
 */
export default class dxPopover extends dxPopup {
    constructor(element: TElement, options?: dxPopoverOptions)
    show(): TPromise<boolean>;
    /**
     * Shows the UI component for a target element.
     */
    show(target: string | TElement): TPromise<boolean>;
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxPopoverOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxPopoverOptions;
