/**
* DevExtreme (ui/responsive_box.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
  TElement
} from '../core/element';

import DataSource, {
    DataSourceOptions
} from '../data/data_source';

import {
  ComponentEvent,
  ComponentNativeEvent,
  ComponentInitializedEvent,
  ChangedOptionInfo,
  ItemInfo
} from '../events/index';

import CollectionWidget, {
    CollectionWidgetItem,
    CollectionWidgetOptions
} from './collection/ui.collection_widget.base';

export type ContentReadyEvent = ComponentEvent<dxResponsiveBox>;

export type DisposingEvent = ComponentEvent<dxResponsiveBox>;

export type InitializedEvent = ComponentInitializedEvent<dxResponsiveBox>;

export type ItemClickEvent = ComponentNativeEvent<dxResponsiveBox> & ItemInfo;

export type ItemContextMenuEvent = ComponentNativeEvent<dxResponsiveBox> & ItemInfo;

export type ItemHoldEvent = ComponentNativeEvent<dxResponsiveBox> & ItemInfo;

export type ItemRenderedEvent = ComponentNativeEvent<dxResponsiveBox> & ItemInfo;

export type OptionChangedEvent = ComponentEvent<dxResponsiveBox> & ChangedOptionInfo;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxResponsiveBoxOptions extends CollectionWidgetOptions<dxResponsiveBox> {
    /**
     * Specifies the collection of columns for the grid used to position layout elements.
     */
    cols?: Array<{
      /**
       * The column&apos;s base width. Calculated automatically when the singleColumnScreen property arranges all elements in a single column.
       */
      baseSize?: number | 'auto',
      /**
       * The column width ratio.
       */
      ratio?: number,
      /**
       * Decides on which screens the current column is rendered.
       */
      screen?: string,
      /**
       * A factor that defines how much a column width shrinks relative to the rest of the columns in the container.
       */
      shrink?: number
    }>;
    /**
     * 
     */
    dataSource?: string | Array<string | dxResponsiveBoxItem | any> | DataSource | DataSourceOptions;
    /**
     * 
     */
    height?: number | string | (() => number | string);
    /**
     * 
     */
    items?: Array<string | dxResponsiveBoxItem | any>;
    /**
     * Specifies the collection of rows for the grid used to position layout elements.
     */
    rows?: Array<{
      /**
       * The row&apos;s base height. Calculated automatically when the singleColumnScreen property arranges all elements in a single column.
       */
      baseSize?: number | 'auto',
      /**
       * The row height ratio.
       */
      ratio?: number,
      /**
       * Decides on which screens the current row is rendered.
       */
      screen?: string,
      /**
       * A factor that defines how much a row height shrinks relative to the rest of the rows in the container.
       */
      shrink?: number
    }>;
    /**
     * Specifies the function returning the size qualifier depending on the screen&apos;s width.
     */
    screenByWidth?: Function;
    /**
     * Specifies on which screens all layout elements should be arranged in a single column. Accepts a single or several size qualifiers separated by a space.
     */
    singleColumnScreen?: string;
    /**
     * 
     */
    width?: number | string | (() => number | string);
}
/**
 * The ResponsiveBox UI component allows you to create an application or a website with a layout adapted to different screen sizes.
 */
export default class dxResponsiveBox extends CollectionWidget {
    constructor(element: TElement, options?: dxResponsiveBoxOptions)
}

/**
 * 
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxResponsiveBoxItem extends CollectionWidgetItem {
    /**
     * Specifies the item location and size against the UI component grid.
     */
    location?: {
      /**
       * Specifies which column the element should occupy. Accepts an index from the cols array.
       */
      col?: number,
      /**
       * Specifies how many columns the element should span.
       */
      colspan?: number,
      /**
       * Specifies which row the element should occupy. Accepts an index from the rows array.
       */
      row?: number,
      /**
       * Specifies how many rows the element should span.
       */
      rowspan?: number,
      /**
       * Decides on which screens the current location settings should be applied to the element.
       */
      screen?: string
    } | Array<{ col?: number, colspan?: number, row?: number, rowspan?: number, screen?: string }>;
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxResponsiveBoxOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxResponsiveBoxOptions;
