/**
* DevExtreme (ui/scroll_view.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import {
    ComponentEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import dxScrollable, {
    dxScrollableOptions,
    ComponentScrollEvent
} from './scroll_view/ui.scrollable';

export type DisposingEvent = ComponentEvent<dxScrollView>;

export type InitializedEvent = ComponentInitializedEvent<dxScrollView>;

export type OptionChangedEvent = ComponentEvent<dxScrollView> & ChangedOptionInfo;

export type PullDownEvent = ComponentEvent<dxScrollView>;

export type ReachBottomEvent = ComponentEvent<dxScrollView>;

export type ScrollEvent = ComponentScrollEvent<dxScrollView>;

export type UpdatedEvent = ComponentScrollEvent<dxScrollView>;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxScrollViewOptions extends dxScrollableOptions<dxScrollView> {
    /**
     * A function that is executed when the &apos;pull to refresh&apos; gesture is performed. Supported in mobile themes only.
     */
    onPullDown?: ((e: PullDownEvent) => void);
    /**
     * A function that is executed when the content is scrolled down to the bottom.
     */
    onReachBottom?: ((e: ReachBottomEvent) => void);
    /**
     * Specifies the text shown in the pullDown panel when pulling the content down lowers the refresh threshold.
     */
    pulledDownText?: string;
    /**
     * Specifies the text shown in the pullDown panel while pulling the content down to the refresh threshold.
     */
    pullingDownText?: string;
    /**
     * Specifies the text shown in the pullDown panel displayed when content is scrolled to the bottom.
     */
    reachBottomText?: string;
    /**
     * Specifies the text shown in the pullDown panel displayed when the content is being refreshed.
     */
    refreshingText?: string;
}
/**
 * The ScrollView is a UI component that enables a user to scroll its content.
 */
export default class dxScrollView extends dxScrollable {
    constructor(element: TElement, options?: dxScrollViewOptions)
    /**
     * Locks the UI component until the release(preventScrollBottom) method is called and executes the function passed to the onPullDown property and the handler assigned to the pullDown event.
     */
    refresh(): void;
    /**
     * Notifies the ScrollView that data loading is finished.
     */
    release(preventScrollBottom: boolean): TPromise<void>;
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxScrollViewOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxScrollViewOptions;
