/**
* DevExtreme (ui/select_box.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import {
    template
} from '../core/templates/template';

import {
    ComponentEvent,
    ComponentNativeEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo,
    ItemInfo
} from '../events/index';

import {
    SelectionChangedInfo
} from './collection/ui.collection_widget.base';

import {
    DropDownButtonTemplateDataModel
} from './drop_down_editor/ui.drop_down_editor';

import dxDropDownList, {
    dxDropDownListOptions
} from './drop_down_editor/ui.drop_down_list';

import {
    ValueChangedInfo
} from './editor/editor';

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface CustomItemCreatingInfo {
    readonly text?: string;
    customItem?: string | any | TPromise<any>;
}

export type ChangeEvent = ComponentNativeEvent<dxSelectBox>;

export type ClosedEvent = ComponentEvent<dxSelectBox>;

export type ContentReadyEvent = ComponentEvent<dxSelectBox>;

export type CopyEvent = ComponentNativeEvent<dxSelectBox>;
export type CustomItemCreatingEvent = ComponentEvent<dxSelectBox> & CustomItemCreatingInfo;

export type CutEvent = ComponentNativeEvent<dxSelectBox>;

export type DisposingEvent = ComponentEvent<dxSelectBox>;

export type EnterKeyEvent = ComponentNativeEvent<dxSelectBox>;

export type FocusInEvent = ComponentNativeEvent<dxSelectBox>;

export type FocusOutEvent = ComponentNativeEvent<dxSelectBox>;

export type InitializedEvent = ComponentInitializedEvent<dxSelectBox>;

export type InputEvent = ComponentNativeEvent<dxSelectBox>;

export type ItemClickEvent = ComponentNativeEvent<dxSelectBox> & ItemInfo;

export type KeyDownEvent = ComponentNativeEvent<dxSelectBox>;

export type KeyPressEvent = ComponentNativeEvent<dxSelectBox>;

export type KeyUpEvent = ComponentNativeEvent<dxSelectBox>;

export type OpenedEvent = ComponentEvent<dxSelectBox>;

export type OptionChangedEvent = ComponentEvent<dxSelectBox> & ChangedOptionInfo;

export type PasteEvent = ComponentNativeEvent<dxSelectBox>;

export type SelectionChangedEvent = ComponentEvent<dxSelectBox> & SelectionChangedInfo;

export type ValueChangedEvent = ComponentNativeEvent<dxSelectBox> & ValueChangedInfo;

export type DropDownButtonTemplateData = DropDownButtonTemplateDataModel;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxSelectBoxOptions<T = dxSelectBox> extends dxDropDownListOptions<T> {
    /**
     * Specifies whether the UI component allows a user to enter a custom value. Requires the onCustomItemCreating handler implementation.
     */
    acceptCustomValue?: boolean;
    /**
     * Specifies a custom template for the text field. Must contain the TextBox UI component.
     */
    fieldTemplate?: template | ((selectedItem: any, fieldElement: TElement) => string | TElement);
    /**
     * A function that is executed when a user adds a custom item. Requires acceptCustomValue to be set to true.
     */
    onCustomItemCreating?: ((e: CustomItemCreatingEvent) => void);
    /**
     * 
     */
    openOnFieldClick?: boolean;
    /**
     * The text that is provided as a hint in the select box editor.
     */
    placeholder?: string;
    /**
     * 
     */
    showDropDownButton?: boolean;
    /**
     * Specifies whether or not to display selection controls.
     */
    showSelectionControls?: boolean;
    /**
     * Specifies the DOM events after which the UI component&apos;s value should be updated. Applies only if acceptCustomValue is set to true.
     */
    valueChangeEvent?: string;
}
/**
 * The SelectBox UI component is an editor that allows an end user to select an item from a drop-down list.
 */
export default class dxSelectBox extends dxDropDownList {
    constructor(element: TElement, options?: dxSelectBoxOptions)
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxSelectBoxOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxSelectBoxOptions;
