/**
* DevExtreme (ui/slide_out_view.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    TPromise
} from '../core/utils/deferred';

import {
    template
} from '../core/templates/template';

import {
    ComponentEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import Widget, {
    WidgetOptions
} from './widget/ui.widget';

export type DisposingEvent = ComponentEvent<dxSlideOutView>;

export type InitializedEvent = ComponentInitializedEvent<dxSlideOutView>;

export type OptionChangedEvent = ComponentEvent<dxSlideOutView> & ChangedOptionInfo;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxSlideOutViewOptions extends WidgetOptions<dxSlideOutView> {
    /**
     * Specifies a custom template for the UI component content.
     */
    contentTemplate?: template | ((contentElement: TElement) => any);
    /**
     * Specifies the current menu position.
     */
    menuPosition?: 'inverted' | 'normal';
    /**
     * Specifies a custom template for the menu content.
     */
    menuTemplate?: template | ((menuElement: TElement) => any);
    /**
     * Specifies whether or not the menu panel is visible.
     */
    menuVisible?: boolean;
    /**
     * Specifies whether or not the menu is shown when a user swipes the UI component content.
     */
    swipeEnabled?: boolean;
}
/**
 * The SlideOutView UI component is a classic slide-out menu paired with a view. This UI component is very similar to the SlideOut with only one difference - the SlideOut always contains the List in the slide-out menu, while the SlideOutView can hold any collection there.
 */
export default class dxSlideOutView extends Widget {
    constructor(element: TElement, options?: dxSlideOutViewOptions)
    /**
     * Gets the UI component&apos;s content.
     */
    content(): TElement;
    /**
     * Hides the UI component&apos;s slide-out menu.
     */
    hideMenu(): TPromise<void>;
    /**
     * Gets the slide-out menu&apos;s content.
     */
    menuContent(): TElement;
    /**
     * Shows the slide-out menu.
     */
    showMenu(): TPromise<void>;
    /**
     * Shows or hides the slide-out menu depending on the argument.
     */
    toggleMenuVisibility(): TPromise<void>;
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxSlideOutViewOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxSlideOutViewOptions;
