/**
* DevExtreme (ui/text_area.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    ComponentEvent,
    ComponentNativeEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import {
    ValueChangedInfo
} from './editor/editor';

import dxTextBox, {
    dxTextBoxOptions
} from './text_box';

export type ChangeEvent = ComponentNativeEvent<dxTextArea>;

export type ContentReadyEvent = ComponentEvent<dxTextArea>;

export type CopyEvent = ComponentNativeEvent<dxTextArea>;

export type CutEvent = ComponentNativeEvent<dxTextArea>;

export type DisposingEvent = ComponentEvent<dxTextArea>;

export type EnterKeyEvent = ComponentNativeEvent<dxTextArea>;

export type FocusInEvent = ComponentNativeEvent<dxTextArea>;

export type FocusOutEvent = ComponentNativeEvent<dxTextArea>;

export type InitializedEvent = ComponentInitializedEvent<dxTextArea>;

export type InputEvent = ComponentNativeEvent<dxTextArea>;

export type KeyDownEvent = ComponentNativeEvent<dxTextArea>;

export type KeyPressEvent = ComponentNativeEvent<dxTextArea>;

export type KeyUpEvent = ComponentNativeEvent<dxTextArea>;

export type OptionChangedEvent = ComponentEvent<dxTextArea> & ChangedOptionInfo;

export type PasteEvent = ComponentNativeEvent<dxTextArea>;

export type ValueChangedEvent = ComponentNativeEvent<dxTextArea> & ValueChangedInfo;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxTextAreaOptions extends dxTextBoxOptions<dxTextArea> {
    /**
     * A Boolean value specifying whether or not the auto resizing mode is enabled.
     */
    autoResizeEnabled?: boolean;
    /**
     * Specifies the maximum height of the UI component.
     */
    maxHeight?: number | string;
    /**
     * Specifies the minimum height of the UI component.
     */
    minHeight?: number | string;
    /**
     * Specifies whether or not the UI component checks the inner text for spelling mistakes.
     */
    spellcheck?: boolean;
}
/**
 * The TextArea is a UI component that enables a user to enter and edit a multi-line text.
 */
export default class dxTextArea extends dxTextBox {
    constructor(element: TElement, options?: dxTextAreaOptions)
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxTextAreaOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxTextAreaOptions;
