/**
* DevExtreme (ui/text_box.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    ComponentEvent,
    ComponentNativeEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import {
    ValueChangedInfo
} from './editor/editor';

import dxTextEditor, {
    dxTextEditorOptions
} from './text_box/ui.text_editor.base';

export type ChangeEvent = ComponentNativeEvent<dxTextBox>;

export type ContentReadyEvent = ComponentEvent<dxTextBox>;

export type CopyEvent = ComponentNativeEvent<dxTextBox>;

export type CutEvent = ComponentNativeEvent<dxTextBox>;

export type DisposingEvent = ComponentEvent<dxTextBox>;

export type EnterKeyEvent = ComponentNativeEvent<dxTextBox>;

export type FocusInEvent = ComponentNativeEvent<dxTextBox>;

export type FocusOutEvent = ComponentNativeEvent<dxTextBox>;

export type InitializedEvent = ComponentInitializedEvent<dxTextBox>;

export type InputEvent = ComponentNativeEvent<dxTextBox>;

export type KeyDownEvent = ComponentNativeEvent<dxTextBox>;

export type KeyPressEvent = ComponentNativeEvent<dxTextBox>;

export type KeyUpEvent = ComponentNativeEvent<dxTextBox>;

export type OptionChangedEvent = ComponentEvent<dxTextBox> & ChangedOptionInfo;

export type PasteEvent = ComponentNativeEvent<dxTextBox>;

export type ValueChangedEvent = ComponentNativeEvent<dxTextBox> & ValueChangedInfo;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxTextBoxOptions<T = dxTextBox> extends dxTextEditorOptions<T> {
    /**
     * Specifies the maximum number of characters you can enter into the textbox.
     */
    maxLength?: string | number;
    /**
     * The &apos;mode&apos; attribute value of the actual HTML input element representing the text box.
     */
    mode?: 'email' | 'password' | 'search' | 'tel' | 'text' | 'url';
    /**
     * Specifies a value the UI component displays.
     */
    value?: string;
}
/**
 * The TextBox is a UI component that enables a user to enter and edit a single line of text.
 */
export default class dxTextBox extends dxTextEditor {
    constructor(element: TElement, options?: dxTextBoxOptions)
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxTextBoxOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxTextBoxOptions;
