/**
* DevExtreme (viz/bullet.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    Cancelable,
    ComponentEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import {
    ComponentFileSavingEvent,
    ExportInfo,
    IncidentInfo
} from './core/base_widget';

import BaseSparkline, {
    BaseSparklineOptions
} from './sparklines/base_sparkline';

export type DisposingEvent = ComponentEvent<dxBullet>;

export type DrawnEvent = ComponentEvent<dxBullet>;

export type ExportedEvent = ComponentEvent<dxBullet>;

export type ExportingEvent = ComponentEvent<dxBullet> & ExportInfo;

export type FileSavingEvent = Cancelable & ComponentFileSavingEvent<dxBullet>;

export type IncidentOccurredEvent = ComponentEvent<dxBullet> & IncidentInfo;

export type InitializedEvent = ComponentInitializedEvent<dxBullet>;

export type OptionChangedEvent = ComponentEvent<dxBullet> & ChangedOptionInfo;

export type TooltipHiddenEvent = ComponentEvent<dxBullet>;

export type TooltipShownEvent = ComponentEvent<dxBullet>;


/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxBulletOptions extends BaseSparklineOptions<dxBullet> {
    /**
     * Specifies a color for the bullet bar.
     */
    color?: string;
    /**
     * Specifies an end value for the invisible scale.
     */
    endScaleValue?: number;
    /**
     * Specifies whether or not to show the target line.
     */
    showTarget?: boolean;
    /**
     * Specifies whether or not to show the line indicating zero on the invisible scale.
     */
    showZeroLevel?: boolean;
    /**
     * Specifies a start value for the invisible scale.
     */
    startScaleValue?: number;
    /**
     * Specifies the value indicated by the target line.
     */
    target?: number;
    /**
     * Specifies a color for both the target and zero level lines.
     */
    targetColor?: string;
    /**
     * Specifies the width of the target line.
     */
    targetWidth?: number;
    /**
     * Specifies the primary value indicated by the bullet bar.
     */
    value?: number;
}
/**
 * The Bullet UI component is useful when you need to compare a single measure to a target value. The UI component comprises a horizontal bar indicating the measure and a vertical line indicating the target value.
 */
export default class dxBullet extends BaseSparkline {
    constructor(element: TElement, options?: dxBulletOptions)
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxBulletOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxBulletOptions;
