/**
* DevExtreme (viz/circular_gauge.d.ts)
* Version: 21.1.2
* Build date: Fri Apr 09 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    TElement
} from '../core/element';

import {
    Cancelable,
    ComponentEvent,
    ComponentInitializedEvent,
    ChangedOptionInfo
} from '../events/index';

import {
    ComponentFileSavingEvent,
    ExportInfo,
    IncidentInfo
} from './core/base_widget';

import {
    BaseGauge,
    BaseGaugeOptions,
    BaseGaugeRangeContainer,
    BaseGaugeScale,
    BaseGaugeScaleLabel,
    GaugeIndicator,
    TooltipInfo
} from './gauges/base_gauge';

export type DisposingEvent = ComponentEvent<dxCircularGauge>;

export type DrawnEvent = ComponentEvent<dxCircularGauge>;

export type ExportedEvent = ComponentEvent<dxCircularGauge>;

export type ExportingEvent = ComponentEvent<dxCircularGauge> & ExportInfo;

export type FileSavingEvent = Cancelable & ComponentFileSavingEvent<dxCircularGauge>;

export type IncidentOccurredEvent = ComponentEvent<dxCircularGauge> & IncidentInfo;

export type InitializedEvent = ComponentInitializedEvent<dxCircularGauge>;

export type OptionChangedEvent = ComponentEvent<dxCircularGauge> & ChangedOptionInfo;

export type TooltipHiddenEvent = ComponentEvent<dxCircularGauge> & TooltipInfo;

export type TooltipShownEvent = ComponentEvent<dxCircularGauge> & TooltipInfo;

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxCircularGaugeOptions extends BaseGaugeOptions<dxCircularGauge> {
    /**
     * Specifies the properties required to set the geometry of the CircularGauge UI component.
     */
    geometry?: {
      /**
       * Specifies the end angle of the circular gauge&apos;s arc.
       */
      endAngle?: number,
      /**
       * Specifies the start angle of the circular gauge&apos;s arc.
       */
      startAngle?: number
    };
    /**
     * Specifies gauge range container properties.
     */
    rangeContainer?: dxCircularGaugeRangeContainer;
    /**
     * Specifies a gauge&apos;s scale properties.
     */
    scale?: dxCircularGaugeScale;
    /**
     * Specifies the appearance properties of subvalue indicators.
     */
    subvalueIndicator?: GaugeIndicator;
    /**
     * Specifies the appearance properties of the value indicator.
     */
    valueIndicator?: GaugeIndicator;
}
/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxCircularGaugeRangeContainer extends BaseGaugeRangeContainer {
    /**
     * Specifies the orientation of the range container in the CircularGauge UI component.
     */
    orientation?: 'center' | 'inside' | 'outside';
    /**
     * Specifies the range container&apos;s width in pixels.
     */
    width?: number;
}
/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxCircularGaugeScale extends BaseGaugeScale {
    /**
     * Specifies common properties for scale labels.
     */
    label?: dxCircularGaugeScaleLabel;
    /**
     * Specifies the orientation of scale ticks.
     */
    orientation?: 'center' | 'inside' | 'outside';
}
/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxCircularGaugeScaleLabel extends BaseGaugeScaleLabel {
    /**
     * Specifies which label to hide in case of overlapping.
     */
    hideFirstOrLast?: 'first' | 'last';
    /**
     * Specifies the spacing between scale labels and ticks.
     */
    indentFromTick?: number;
}
/**
 * The CircularGauge is a UI component that indicates values on a circular numeric scale.
 */
export default class dxCircularGauge extends BaseGauge {
    constructor(element: TElement, options?: dxCircularGaugeOptions)
}

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxCircularGaugeOptions;

/**
 * @deprecated
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxCircularGaugeOptions;
